/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    Pseudo_QC_List.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QoreListIterator.h"
#include "qore/intern/RangeIterator.h"
#include "qore/intern/qore_list_private.h"

extern QoreClass* QC_LISTITERATOR;
extern QoreClass* QC_RANGEITERATOR;
/* Qore class Qore::<list> */

qore_classid_t CID_PSEUDOLIST;
QoreClass* QC_PSEUDOLIST;

// bool PseudoList::complexType(){}
static QoreValue PseudoList_complexType(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = v.get<const QoreListNode>();
# 76 "Pseudo_QC_List.qpp"
    const qore_list_private* lp = qore_list_private::get(*l);
   return lp->complexTypeInfo ? true : false;
}

// bool PseudoList::contains(auto arg){}
static QoreValue PseudoList_contains_VA(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
    const QoreListNode* l = v.get<const QoreListNode>();
# 213 "Pseudo_QC_List.qpp"
    return inlist_intern(arg, l, xsink);
}

// bool PseudoList::empty(){}
static QoreValue PseudoList_empty(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = v.get<const QoreListNode>();
# 106 "Pseudo_QC_List.qpp"
    return l->empty();
}

// auto PseudoList::first(){}
static QoreValue PseudoList_first(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = v.get<const QoreListNode>();
# 140 "Pseudo_QC_List.qpp"
    return l->getReferencedEntry(0);
}

// AbstractIterator PseudoList::iterator(){}
static QoreValue PseudoList_iterator(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = v.get<const QoreListNode>();
# 227 "Pseudo_QC_List.qpp"
    return new QoreObject(QC_LISTITERATOR, 0, new QoreListIterator(l));
}

// string PseudoList::join(string str){}
static QoreValue PseudoList_join_Vs(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* l = v.get<const QoreListNode>();
# 178 "Pseudo_QC_List.qpp"
    return join_intern(str, l, 0, xsink);
}

// auto PseudoList::last(){}
static QoreValue PseudoList_last(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = v.get<const QoreListNode>();
# 156 "Pseudo_QC_List.qpp"
    return l->getReferencedEntry((size_t)(l->size() - 1));
}

// int PseudoList::lsize(){}
static QoreValue PseudoList_lsize(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = v.get<const QoreListNode>();
# 195 "Pseudo_QC_List.qpp"
    return l->size();
}

// AbstractIterator PseudoList::rangeIterator(auto val){}
static QoreValue PseudoList_rangeIterator_VA(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue val = get_param_value(args, 0);
    const QoreListNode* l = v.get<const QoreListNode>();
# 245 "Pseudo_QC_List.qpp"
    if (l->empty())
      return new QoreObject(QC_SINGLEVALUEITERATOR, 0, new SingleValueIterator);

   ReferenceHolder<RangeIterator> r(new RangeIterator(0, l->size(), 1, val, xsink), xsink);
   if (*xsink)
      return QoreValue();

   return new QoreObject(QC_RANGEITERATOR, 0, r.release());
}

// int PseudoList::size(){}
static QoreValue PseudoList_size(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = v.get<const QoreListNode>();
# 92 "Pseudo_QC_List.qpp"
    return l->size();
}

// bool PseudoList::sizep(){}
static QoreValue PseudoList_sizep(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 263 "Pseudo_QC_List.qpp"
    return true;
}

// int PseudoList::typeCode(){}
static QoreValue PseudoList_typeCode(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 59 "Pseudo_QC_List.qpp"
    return NT_LIST;
}

// bool PseudoList::val(){}
static QoreValue PseudoList_val(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = v.get<const QoreListNode>();
# 124 "Pseudo_QC_List.qpp"
    return !l->empty();
}

DLLLOCAL void preinitPseudoListClass() {
    QC_PSEUDOLIST = new QoreBuiltinClass("<list>", "::Qore::<list>", QDOM_DEFAULT);
    CID_PSEUDOLIST = QC_PSEUDOLIST->getID();
    QC_PSEUDOLIST->setSystem();
}

DLLLOCAL QoreClass* initPseudoListClass() {
    if (!QC_PSEUDOLIST)
        preinitPseudoListClass();

    // set parent class
    assert(QC_PSEUDOVALUE);
    QC_PSEUDOLIST->addBuiltinVirtualBaseClass(QC_PSEUDOVALUE);

    // bool PseudoList::complexType(){}
    QC_PSEUDOLIST->addMethod("complexType", (q_method_n_t)PseudoList_complexType, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool PseudoList::contains(auto arg){}
    QC_PSEUDOLIST->addMethod("contains", (q_method_n_t)PseudoList_contains_VA, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "arg");

    // bool PseudoList::empty(){}
    QC_PSEUDOLIST->addMethod("empty", (q_method_n_t)PseudoList_empty, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // auto PseudoList::first(){}
    QC_PSEUDOLIST->addMethod("first", (q_method_n_t)PseudoList_first, Public, QCF_CONSTANT, QDOM_DEFAULT, autoTypeInfo);

    // AbstractIterator PseudoList::iterator(){}
    QC_PSEUDOLIST->addMethod("iterator", (q_method_n_t)PseudoList_iterator, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_ABSTRACTITERATOR->getTypeInfo());

    // string PseudoList::join(string str){}
    QC_PSEUDOLIST->addMethod("join", (q_method_n_t)PseudoList_join_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "str");

    // auto PseudoList::last(){}
    QC_PSEUDOLIST->addMethod("last", (q_method_n_t)PseudoList_last, Public, QCF_CONSTANT, QDOM_DEFAULT, autoTypeInfo);

    // int PseudoList::lsize(){}
    QC_PSEUDOLIST->addMethod("lsize", (q_method_n_t)PseudoList_lsize, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // AbstractIterator PseudoList::rangeIterator(auto val){}
    QC_PSEUDOLIST->addMethod("rangeIterator", (q_method_n_t)PseudoList_rangeIterator_VA, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_ABSTRACTITERATOR->getTypeInfo(), 1, autoTypeInfo, QORE_PARAM_NO_ARG, "val");

    // int PseudoList::size(){}
    QC_PSEUDOLIST->addMethod("size", (q_method_n_t)PseudoList_size, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool PseudoList::sizep(){}
    QC_PSEUDOLIST->addMethod("sizep", (q_method_n_t)PseudoList_sizep, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // int PseudoList::typeCode(){}
    QC_PSEUDOLIST->addMethod("typeCode", (q_method_n_t)PseudoList_typeCode, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool PseudoList::val(){}
    QC_PSEUDOLIST->addMethod("val", (q_method_n_t)PseudoList_val, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    return QC_PSEUDOLIST;
}
