/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    Pseudo_QC_Object.cpp

    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QoreObjectIntern.h"
#include "qore/intern/QoreObjectIterator.h"
#include "qore/intern/QoreClassIntern.h"

extern QoreClass* QC_OBJECTITERATOR;
extern QoreClass* QC_OBJECTKEYITERATOR;
extern QoreClass* QC_OBJECTPAIRITERATOR;
extern QoreClass* QC_OBJECTKEYITERATOR;
extern QoreClass* QC_OBJECTPAIRITERATOR;
/* Qore class Qore::<object> */

qore_classid_t CID_PSEUDOOBJECT;
QoreClass* QC_PSEUDOOBJECT;

// string PseudoObject::className(){}
static QoreValue PseudoObject_className(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = const_cast<QoreObject*>(v.get<QoreObject>());
# 166 "Pseudo_QC_Object.qpp"
    return new QoreStringNode(obj->getClassName());
}

// bool PseudoObject::complexType(){}
static QoreValue PseudoObject_complexType(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 79 "Pseudo_QC_Object.qpp"
    return true;
}

// bool PseudoObject::empty(){}
static QoreValue PseudoObject_empty(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = const_cast<QoreObject*>(v.get<QoreObject>());
# 136 "Pseudo_QC_Object.qpp"
    return !obj->size(xsink);
}

// *string PseudoObject::firstKey(){}
static QoreValue PseudoObject_firstKey(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = const_cast<QoreObject*>(v.get<QoreObject>());
# 107 "Pseudo_QC_Object.qpp"
    return qore_object_private::firstKey(obj, xsink);
}

// code PseudoObject::getCallReference(string identifier){}
static QoreValue PseudoObject_getCallReference_Vs(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* identifier = HARD_QORE_VALUE_STRING(args, 0);
    QoreObject* obj = const_cast<QoreObject*>(v.get<QoreObject>());
# 362 "Pseudo_QC_Object.qpp"
    return get_call_reference_intern(obj, identifier, xsink);
}

// bool PseudoObject::hasCallableMethod(string name){}
static QoreValue PseudoObject_hasCallableMethod_Vs(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    QoreObject* obj = const_cast<QoreObject*>(v.get<QoreObject>());
# 265 "Pseudo_QC_Object.qpp"
    return qore_class_private::runtimeHasCallableMethod(*(obj->getClass()), name->getBuffer());
}

// bool PseudoObject::hasCallableNormalMethod(string name){}
static QoreValue PseudoObject_hasCallableNormalMethod_Vs(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    QoreObject* obj = const_cast<QoreObject*>(v.get<QoreObject>());
# 288 "Pseudo_QC_Object.qpp"
    return qore_class_private::runtimeHasCallableNormalMethod(*(obj->getClass()), name->getBuffer());
}

// bool PseudoObject::hasCallableStaticMethod(string name){}
static QoreValue PseudoObject_hasCallableStaticMethod_Vs(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    QoreObject* obj = const_cast<QoreObject*>(v.get<QoreObject>());
# 310 "Pseudo_QC_Object.qpp"
    return qore_class_private::runtimeHasCallableStaticMethod(*(obj->getClass()), name->getBuffer());
}

// bool PseudoObject::isSystem(){}
static QoreValue PseudoObject_isSystem(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = const_cast<QoreObject*>(v.get<QoreObject>());
# 179 "Pseudo_QC_Object.qpp"
    return obj->isSystemObject();
}

// AbstractIterator PseudoObject::iterator(){}
static QoreValue PseudoObject_iterator(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = const_cast<QoreObject*>(v.get<QoreObject>());
# 214 "Pseudo_QC_Object.qpp"
    return new QoreObject(QC_OBJECTITERATOR, 0, new QoreObjectIterator(obj));
}

// ObjectKeyIterator PseudoObject::keyIterator(){}
static QoreValue PseudoObject_keyIterator(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = const_cast<QoreObject*>(v.get<QoreObject>());
# 228 "Pseudo_QC_Object.qpp"
    return new QoreObject(QC_OBJECTKEYITERATOR, 0, new QoreObjectIterator(obj));
}

// list<string> PseudoObject::keys(){}
static QoreValue PseudoObject_keys(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = const_cast<QoreObject*>(v.get<QoreObject>());
# 92 "Pseudo_QC_Object.qpp"
    return obj->getMemberList(xsink);
}

// *string PseudoObject::lastKey(){}
static QoreValue PseudoObject_lastKey(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = const_cast<QoreObject*>(v.get<QoreObject>());
# 122 "Pseudo_QC_Object.qpp"
    return qore_object_private::lastKey(obj, xsink);
}

// ObjectPairIterator PseudoObject::pairIterator(){}
static QoreValue PseudoObject_pairIterator(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = const_cast<QoreObject*>(v.get<QoreObject>());
# 242 "Pseudo_QC_Object.qpp"
    return new QoreObject(QC_OBJECTPAIRITERATOR, 0, new QoreObjectIterator(obj));
}

// int PseudoObject::size(){}
static QoreValue PseudoObject_size(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = const_cast<QoreObject*>(v.get<QoreObject>());
# 151 "Pseudo_QC_Object.qpp"
    return obj->size(xsink);
}

// bool PseudoObject::sizep(){}
static QoreValue PseudoObject_sizep(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 321 "Pseudo_QC_Object.qpp"
    return true;
}

// int PseudoObject::typeCode(){}
static QoreValue PseudoObject_typeCode(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 62 "Pseudo_QC_Object.qpp"
    return NT_OBJECT;
}

// string PseudoObject::uniqueHash(){}
static QoreValue PseudoObject_uniqueHash(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = const_cast<QoreObject*>(v.get<QoreObject>());
# 332 "Pseudo_QC_Object.qpp"
    QoreStringNode* rv = new QoreStringNode;
    qore_get_ptr_hash(*rv, obj);
    return rv;
}

// bool PseudoObject::val(){}
static QoreValue PseudoObject_val(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = const_cast<QoreObject*>(v.get<QoreObject>());
# 197 "Pseudo_QC_Object.qpp"
    return (bool)obj->size(xsink);
}

DLLLOCAL void preinitPseudoObjectClass() {
    QC_PSEUDOOBJECT = new QoreBuiltinClass("<object>", "::Qore::<object>", QDOM_DEFAULT);
    CID_PSEUDOOBJECT = QC_PSEUDOOBJECT->getID();
    QC_PSEUDOOBJECT->setSystem();
}

DLLLOCAL QoreClass* initPseudoObjectClass() {
    if (!QC_PSEUDOOBJECT)
        preinitPseudoObjectClass();

    // set parent class
    assert(QC_PSEUDOVALUE);
    QC_PSEUDOOBJECT->addBuiltinVirtualBaseClass(QC_PSEUDOVALUE);

    // string PseudoObject::className(){}
    QC_PSEUDOOBJECT->addMethod("className", (q_method_n_t)PseudoObject_className, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // bool PseudoObject::complexType(){}
    QC_PSEUDOOBJECT->addMethod("complexType", (q_method_n_t)PseudoObject_complexType, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool PseudoObject::empty(){}
    QC_PSEUDOOBJECT->addMethod("empty", (q_method_n_t)PseudoObject_empty, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // *string PseudoObject::firstKey(){}
    QC_PSEUDOOBJECT->addMethod("firstKey", (q_method_n_t)PseudoObject_firstKey, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // code PseudoObject::getCallReference(string identifier){}
    QC_PSEUDOOBJECT->addMethod("getCallReference", (q_method_n_t)PseudoObject_getCallReference_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, codeTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "identifier");

    // bool PseudoObject::hasCallableMethod(string name){}
    QC_PSEUDOOBJECT->addMethod("hasCallableMethod", (q_method_n_t)PseudoObject_hasCallableMethod_Vs, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // bool PseudoObject::hasCallableNormalMethod(string name){}
    QC_PSEUDOOBJECT->addMethod("hasCallableNormalMethod", (q_method_n_t)PseudoObject_hasCallableNormalMethod_Vs, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // bool PseudoObject::hasCallableStaticMethod(string name){}
    QC_PSEUDOOBJECT->addMethod("hasCallableStaticMethod", (q_method_n_t)PseudoObject_hasCallableStaticMethod_Vs, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // bool PseudoObject::isSystem(){}
    QC_PSEUDOOBJECT->addMethod("isSystem", (q_method_n_t)PseudoObject_isSystem, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // AbstractIterator PseudoObject::iterator(){}
    QC_PSEUDOOBJECT->addMethod("iterator", (q_method_n_t)PseudoObject_iterator, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_ABSTRACTITERATOR->getTypeInfo());

    // ObjectKeyIterator PseudoObject::keyIterator(){}
    QC_PSEUDOOBJECT->addMethod("keyIterator", (q_method_n_t)PseudoObject_keyIterator, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_OBJECTKEYITERATOR->getTypeInfo());

    // list<string> PseudoObject::keys(){}
    QC_PSEUDOOBJECT->addMethod("keys", (q_method_n_t)PseudoObject_keys, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo));

    // *string PseudoObject::lastKey(){}
    QC_PSEUDOOBJECT->addMethod("lastKey", (q_method_n_t)PseudoObject_lastKey, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // ObjectPairIterator PseudoObject::pairIterator(){}
    QC_PSEUDOOBJECT->addMethod("pairIterator", (q_method_n_t)PseudoObject_pairIterator, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_OBJECTPAIRITERATOR->getTypeInfo());

    // int PseudoObject::size(){}
    QC_PSEUDOOBJECT->addMethod("size", (q_method_n_t)PseudoObject_size, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);

    // bool PseudoObject::sizep(){}
    QC_PSEUDOOBJECT->addMethod("sizep", (q_method_n_t)PseudoObject_sizep, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // int PseudoObject::typeCode(){}
    QC_PSEUDOOBJECT->addMethod("typeCode", (q_method_n_t)PseudoObject_typeCode, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // string PseudoObject::uniqueHash(){}
    QC_PSEUDOOBJECT->addMethod("uniqueHash", (q_method_n_t)PseudoObject_uniqueHash, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // bool PseudoObject::val(){}
    QC_PSEUDOOBJECT->addMethod("val", (q_method_n_t)PseudoObject_val, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    return QC_PSEUDOOBJECT;
}
// bool <object>::hasKey(softstring key) [flags=RET_VALUE_ONLY] {}
// bool <object>::hasKeyValue(softstring key) [flags=RET_VALUE_ONLY] {}
