/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_AbstractDatasource.qpp AbstractDatasource class (interface) definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>

class AbstractDatasource {};

DLLLOCAL extern QoreClass* QC_ABSTRACTSQLSTATEMENT;
/* Qore class Qore::SQL::AbstractDatasource */

qore_classid_t CID_ABSTRACTDATASOURCE;
QoreClass* QC_ABSTRACTDATASOURCE;

// bool AbstractDatasource::currentThreadInTransaction(){}
static QoreValue AbstractDatasource_currentThreadInTransaction(QoreObject* self, AbstractDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 417 "QC_AbstractDatasource.qpp"
    xsink->raiseException("ABSTRACT-DATASOURCE-ERROR", "This method must be re-implemented in subclasses to provide the desired functionality; it is implemented in Datasource and DatasourcePool for example");
   return false;
}

// string AbstractDatasource::getDriverRealName(){}
static QoreValue AbstractDatasource_getDriverRealName(QoreObject* self, AbstractDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 479 "QC_AbstractDatasource.qpp"
    return self->evalMethod("getDriverName", nullptr, xsink);
}

// auto AbstractDatasource::getOption(string opt){}
static QoreValue AbstractDatasource_getOption_Vs(QoreObject* self, AbstractDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 459 "QC_AbstractDatasource.qpp"
    xsink->raiseException("DBI-OPTION-ERROR", "This method must be re-implemented in subclasses to provide the "
        "desired functionality; it is implemented in Datasource and DatasourcePool for example");
    return QoreValue();
}

// hash<auto> AbstractDatasource::getOptionHash(){}
static QoreValue AbstractDatasource_getOptionHash(QoreObject* self, AbstractDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 447 "QC_AbstractDatasource.qpp"
    return new QoreHashNode(autoTypeInfo);
}

// AbstractSQLStatement AbstractDatasource::getSQLStatement(){}
static QoreValue AbstractDatasource_getSQLStatement(QoreObject* self, AbstractDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 431 "QC_AbstractDatasource.qpp"
    xsink->raiseException("ABSTRACT-DATASOURCE-ERROR", "This method must be re-implemented in subclasses to provide "
        "the desired functionality; it is implemented in Datasource and DatasourcePool for example");
    return QoreValue();
}

DLLLOCAL void preinitAbstractDatasourceClass() {
    QC_ABSTRACTDATASOURCE = new QoreBuiltinClass("AbstractDatasource", "::Qore::SQL::AbstractDatasource", QDOM_DATABASE);
    CID_ABSTRACTDATASOURCE = QC_ABSTRACTDATASOURCE->getID();
    QC_ABSTRACTDATASOURCE->setSystem();
}

DLLLOCAL QoreClass* initAbstractDatasourceClass(QoreNamespace& ns) {
    if (!QC_ABSTRACTDATASOURCE)
        preinitAbstractDatasourceClass();

    // nothing AbstractDatasource::beginTransaction();
    QC_ABSTRACTDATASOURCE->addAbstractMethod("beginTransaction", Public, QCF_NO_FLAGS, nothingTypeInfo);

    // nothing AbstractDatasource::commit();
    QC_ABSTRACTDATASOURCE->addAbstractMethod("commit", Public, QCF_NO_FLAGS, nothingTypeInfo);

    // bool AbstractDatasource::currentThreadInTransaction(){}
    QC_ABSTRACTDATASOURCE->addMethod("currentThreadInTransaction", (q_method_n_t)AbstractDatasource_currentThreadInTransaction, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // auto AbstractDatasource::exec(string sql, ...);
    QC_ABSTRACTDATASOURCE->addAbstractMethod("exec", Public, QCF_USES_EXTRA_ARGS, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "sql");

    // auto AbstractDatasource::execRaw(string sql);
    QC_ABSTRACTDATASOURCE->addAbstractMethod("execRaw", Public, QCF_NO_FLAGS, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "sql");

    // auto AbstractDatasource::getClientVersion();
    QC_ABSTRACTDATASOURCE->addAbstractMethod("getClientVersion", Public, QCF_NO_FLAGS, autoTypeInfo);

    // hash<auto> AbstractDatasource::getConfigHash();
    QC_ABSTRACTDATASOURCE->addAbstractMethod("getConfigHash", Public, QCF_NO_FLAGS, autoHashTypeInfo);

    // string AbstractDatasource::getConfigString();
    QC_ABSTRACTDATASOURCE->addAbstractMethod("getConfigString", Public, QCF_NO_FLAGS, stringTypeInfo);

    // *string AbstractDatasource::getDBEncoding();
    QC_ABSTRACTDATASOURCE->addAbstractMethod("getDBEncoding", Public, QCF_NO_FLAGS, stringOrNothingTypeInfo);

    // *string AbstractDatasource::getDBName();
    QC_ABSTRACTDATASOURCE->addAbstractMethod("getDBName", Public, QCF_NO_FLAGS, stringOrNothingTypeInfo);

    // string AbstractDatasource::getDriverName();
    QC_ABSTRACTDATASOURCE->addAbstractMethod("getDriverName", Public, QCF_NO_FLAGS, stringTypeInfo);

    // string AbstractDatasource::getDriverRealName(){}
    QC_ABSTRACTDATASOURCE->addMethod("getDriverRealName", (q_method_n_t)AbstractDatasource_getDriverRealName, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);

    // *string AbstractDatasource::getHostName();
    QC_ABSTRACTDATASOURCE->addAbstractMethod("getHostName", Public, QCF_NO_FLAGS, stringOrNothingTypeInfo);

    // *string AbstractDatasource::getOSEncoding();
    QC_ABSTRACTDATASOURCE->addAbstractMethod("getOSEncoding", Public, QCF_NO_FLAGS, stringOrNothingTypeInfo);

    // auto AbstractDatasource::getOption(string opt){}
    QC_ABSTRACTDATASOURCE->addMethod("getOption", (q_method_n_t)AbstractDatasource_getOption_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "opt");

    // hash<auto> AbstractDatasource::getOptionHash(){}
    QC_ABSTRACTDATASOURCE->addMethod("getOptionHash", (q_method_n_t)AbstractDatasource_getOptionHash, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoHashTypeInfo);

    // *string AbstractDatasource::getPassword();
    QC_ABSTRACTDATASOURCE->addAbstractMethod("getPassword", Public, QCF_NO_FLAGS, stringOrNothingTypeInfo);

    // *int AbstractDatasource::getPort();
    QC_ABSTRACTDATASOURCE->addAbstractMethod("getPort", Public, QCF_NO_FLAGS, bigIntOrNothingTypeInfo);

    // AbstractSQLStatement AbstractDatasource::getSQLStatement(){}
    QC_ABSTRACTDATASOURCE->addMethod("getSQLStatement", (q_method_n_t)AbstractDatasource_getSQLStatement, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_ABSTRACTSQLSTATEMENT->getTypeInfo());

    // auto AbstractDatasource::getServerVersion();
    QC_ABSTRACTDATASOURCE->addAbstractMethod("getServerVersion", Public, QCF_NO_FLAGS, autoTypeInfo);

    // *string AbstractDatasource::getUserName();
    QC_ABSTRACTDATASOURCE->addAbstractMethod("getUserName", Public, QCF_NO_FLAGS, stringOrNothingTypeInfo);

    // bool AbstractDatasource::inTransaction();
    QC_ABSTRACTDATASOURCE->addAbstractMethod("inTransaction", Public, QCF_NO_FLAGS, boolTypeInfo);

    // nothing AbstractDatasource::rollback();
    QC_ABSTRACTDATASOURCE->addAbstractMethod("rollback", Public, QCF_NO_FLAGS, nothingTypeInfo);

    // auto AbstractDatasource::select(string sql, ...);
    QC_ABSTRACTDATASOURCE->addAbstractMethod("select", Public, QCF_USES_EXTRA_ARGS, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "sql");

    // auto AbstractDatasource::selectRow(string sql, ...);
    QC_ABSTRACTDATASOURCE->addAbstractMethod("selectRow", Public, QCF_USES_EXTRA_ARGS, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "sql");

    // auto AbstractDatasource::selectRows(string sql, ...);
    QC_ABSTRACTDATASOURCE->addAbstractMethod("selectRows", Public, QCF_USES_EXTRA_ARGS, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "sql");

    // auto AbstractDatasource::vexec(string sql, *softlist<auto> vargs);
    QC_ABSTRACTDATASOURCE->addAbstractMethod("vexec", Public, QCF_NO_FLAGS, autoTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "sql", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "vargs");

    // auto AbstractDatasource::vselect(string sql, *softlist<auto> vargs);
    QC_ABSTRACTDATASOURCE->addAbstractMethod("vselect", Public, QCF_NO_FLAGS, autoTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "sql", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "vargs");

    // auto AbstractDatasource::vselectRow(string sql, *softlist<auto> vargs);
    QC_ABSTRACTDATASOURCE->addAbstractMethod("vselectRow", Public, QCF_NO_FLAGS, autoTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "sql", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "vargs");

    // auto AbstractDatasource::vselectRows(string sql, *softlist<auto> vargs);
    QC_ABSTRACTDATASOURCE->addAbstractMethod("vselectRows", Public, QCF_NO_FLAGS, autoTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "sql", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "vargs");

    return QC_ABSTRACTDATASOURCE;
}
