/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_AbstractSQLStatement.qpp AbstractSQLStatement class (interface) definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>

class AbstractSQLStatement {};
/* Qore class Qore::SQL::AbstractSQLStatement */

qore_classid_t CID_ABSTRACTSQLSTATEMENT;
QoreClass* QC_ABSTRACTSQLSTATEMENT;

DLLLOCAL void preinitAbstractSQLStatementClass() {
    QC_ABSTRACTSQLSTATEMENT = new QoreBuiltinClass("AbstractSQLStatement", "::Qore::SQL::AbstractSQLStatement", QDOM_DATABASE);
    CID_ABSTRACTSQLSTATEMENT = QC_ABSTRACTSQLSTATEMENT->getID();
    QC_ABSTRACTSQLSTATEMENT->setSystem();
}

DLLLOCAL QoreClass* initAbstractSQLStatementClass(QoreNamespace& ns) {
    if (!QC_ABSTRACTSQLSTATEMENT)
        preinitAbstractSQLStatementClass();

    // set parent class
    assert(QC_ABSTRACTITERATOR);
    QC_ABSTRACTSQLSTATEMENT->addBuiltinVirtualBaseClass(QC_ABSTRACTITERATOR);

    // bool AbstractSQLStatement::active();
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("active", Public, QCF_NO_FLAGS, boolTypeInfo);

    // int AbstractSQLStatement::affectedRows();
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("affectedRows", Public, QCF_NO_FLAGS, bigIntTypeInfo);

    // nothing AbstractSQLStatement::beginTransaction();
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("beginTransaction", Public, QCF_NO_FLAGS, nothingTypeInfo);

    // nothing AbstractSQLStatement::bind(...);
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("bind", Public, QCF_USES_EXTRA_ARGS, nothingTypeInfo);

    // nothing AbstractSQLStatement::bindArgs(softlist<auto> vargs);
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("bindArgs", Public, QCF_NO_FLAGS, nothingTypeInfo, 1, softAutoListTypeInfo, QORE_PARAM_NO_ARG, "vargs");

    // nothing AbstractSQLStatement::bindPlaceholders(...);
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("bindPlaceholders", Public, QCF_USES_EXTRA_ARGS, nothingTypeInfo);

    // nothing AbstractSQLStatement::bindPlaceholdersArgs(softlist<auto> vargs);
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("bindPlaceholdersArgs", Public, QCF_NO_FLAGS, nothingTypeInfo, 1, softAutoListTypeInfo, QORE_PARAM_NO_ARG, "vargs");

    // nothing AbstractSQLStatement::bindValues(...);
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("bindValues", Public, QCF_USES_EXTRA_ARGS, nothingTypeInfo);

    // nothing AbstractSQLStatement::bindValuesArgs(softlist<auto> vargs);
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("bindValuesArgs", Public, QCF_NO_FLAGS, nothingTypeInfo, 1, softAutoListTypeInfo, QORE_PARAM_NO_ARG, "vargs");

    // nothing AbstractSQLStatement::close();
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("close", Public, QCF_NO_FLAGS, nothingTypeInfo);

    // nothing AbstractSQLStatement::commit();
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("commit", Public, QCF_NO_FLAGS, nothingTypeInfo);

    // bool AbstractSQLStatement::currentThreadInTransaction();
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("currentThreadInTransaction", Public, QCF_NO_FLAGS, boolTypeInfo);

    // nothing AbstractSQLStatement::define();
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("define", Public, QCF_NO_FLAGS, nothingTypeInfo);

    // hash<auto> AbstractSQLStatement::describe();
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("describe", Public, QCF_NO_FLAGS, autoHashTypeInfo);

    // nothing AbstractSQLStatement::exec(...);
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("exec", Public, QCF_USES_EXTRA_ARGS, nothingTypeInfo);

    // nothing AbstractSQLStatement::execArgs(softlist<auto> vargs);
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("execArgs", Public, QCF_NO_FLAGS, nothingTypeInfo, 1, softAutoListTypeInfo, QORE_PARAM_NO_ARG, "vargs");

    // hash<auto> AbstractSQLStatement::fetchColumns(softint rows = -1);
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("fetchColumns", Public, QCF_NO_FLAGS, autoHashTypeInfo, 1, softBigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "rows");

    // *hash<auto> AbstractSQLStatement::fetchRow();
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("fetchRow", Public, QCF_NO_FLAGS, autoHashOrNothingTypeInfo);

    // list<auto> AbstractSQLStatement::fetchRows(softint rows = -1);
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("fetchRows", Public, QCF_NO_FLAGS, autoListTypeInfo, 1, softBigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "rows");

    // hash<auto> AbstractSQLStatement::getOutput();
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("getOutput", Public, QCF_NO_FLAGS, autoHashTypeInfo);

    // hash<auto> AbstractSQLStatement::getOutputRows();
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("getOutputRows", Public, QCF_NO_FLAGS, autoHashTypeInfo);

    // *string AbstractSQLStatement::getSQL();
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("getSQL", Public, QCF_NO_FLAGS, stringOrNothingTypeInfo);

    // *hash<auto> AbstractSQLStatement::getValue();
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("getValue", Public, QCF_NO_FLAGS, autoHashOrNothingTypeInfo);

    // auto AbstractSQLStatement::memberGate(string member);
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("memberGate", Public, QCF_NO_FLAGS, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "member");

    // bool AbstractSQLStatement::next();
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("next", Public, QCF_NO_FLAGS, boolTypeInfo);

    // nothing AbstractSQLStatement::prepare(string sql, ...);
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("prepare", Public, QCF_USES_EXTRA_ARGS, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "sql");

    // nothing AbstractSQLStatement::prepareRaw(string sql);
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("prepareRaw", Public, QCF_NO_FLAGS, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "sql");

    // nothing AbstractSQLStatement::rollback();
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("rollback", Public, QCF_NO_FLAGS, nothingTypeInfo);

    // bool AbstractSQLStatement::valid();
    QC_ABSTRACTSQLSTATEMENT->addAbstractMethod("valid", Public, QCF_NO_FLAGS, boolTypeInfo);

    return QC_ABSTRACTSQLSTATEMENT;
}
