/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    QC_AbstractSmartLock.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_AbstractSmartLock.h"
/* Qore class Qore::Thread::AbstractSmartLock */

qore_classid_t CID_ABSTRACTSMARTLOCK;
QoreClass* QC_ABSTRACTSMARTLOCK;

// AbstractSmartLock::constructor() {}
static void AbstractSmartLock_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 48 "QC_AbstractSmartLock.qpp"
xsink->raiseException("ABSTRACTSMARTLOCK-CONSTRUCTOR-ERROR", "this class is an abstract class and cannot be " \
        "constructed directly or inherited directly by a user-defined class");
}

// string AbstractSmartLock::getName(){}
static QoreValue AbstractSmartLock_getName(QoreObject* self, AbstractSmartLock* asl, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 61 "QC_AbstractSmartLock.qpp"
    return new QoreStringNode(asl->getName());
}

// bool AbstractSmartLock::lockOwner(){}
static QoreValue AbstractSmartLock_lockOwner(QoreObject* self, AbstractSmartLock* asl, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 79 "QC_AbstractSmartLock.qpp"
    return asl->get_tid() == q_gettid();
}

// int AbstractSmartLock::lockTID(){}
static QoreValue AbstractSmartLock_lockTID(QoreObject* self, AbstractSmartLock* asl, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 93 "QC_AbstractSmartLock.qpp"
    int tid = asl->get_tid();
    return !tid ? -1 : tid;
}

DLLLOCAL void preinitAbstractSmartLockClass() {
    QC_ABSTRACTSMARTLOCK = new QoreBuiltinClass("AbstractSmartLock", "::Qore::Thread::AbstractSmartLock", QDOM_THREAD_CLASS);
    CID_ABSTRACTSMARTLOCK = QC_ABSTRACTSMARTLOCK->getID();
    QC_ABSTRACTSMARTLOCK->setSystem();
}

DLLLOCAL QoreClass* initAbstractSmartLockClass(QoreNamespace& ns) {
    if (!QC_ABSTRACTSMARTLOCK)
        preinitAbstractSmartLockClass();

    // AbstractSmartLock::constructor() {}
    QC_ABSTRACTSMARTLOCK->addConstructor(AbstractSmartLock_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // string AbstractSmartLock::getName(){}
    QC_ABSTRACTSMARTLOCK->addMethod("getName", (q_method_n_t)AbstractSmartLock_getName, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // bool AbstractSmartLock::lockOwner(){}
    QC_ABSTRACTSMARTLOCK->addMethod("lockOwner", (q_method_n_t)AbstractSmartLock_lockOwner, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // int AbstractSmartLock::lockTID(){}
    QC_ABSTRACTSMARTLOCK->addMethod("lockTID", (q_method_n_t)AbstractSmartLock_lockTID, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    return QC_ABSTRACTSMARTLOCK;
}
