/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_AutoGate.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2023 David Nichols

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_AutoGate.h"
#include "qore/intern/QC_Gate.h"
/* Qore class Qore::Thread::AutoGate */

qore_classid_t CID_AUTOGATE;
QoreClass* QC_AUTOGATE;

// AutoGate::constructor(Gate gate) {}
static void AutoGate_constructor_C4Gate(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(gate, QoreGate, args, 0, CID_GATE, "AutoGate::constructor()", "Gate", xsink);
    if (*xsink)
        return;
# 71 "QC_AutoGate.qpp"
QoreAutoGate* ag = new QoreAutoGate(gate, xsink);
    if (*xsink) {
        ag->deref(xsink);
    } else {
        self->setPrivate(CID_AUTOGATE, ag);
    }
}

// AutoGate::copy() {}
static void AutoGate_copy(QoreObject* self, QoreObject* old, QoreAutoGate* ag, ExceptionSink* xsink) {
# 91 "QC_AutoGate.qpp"
xsink->raiseException("AUTOGATE-COPY-ERROR", "objects of this class cannot be copied");
}

// AutoGate::destructor() {}
static void AutoGate_destructor(QoreObject* self, QoreAutoGate* ag, ExceptionSink* xsink) {
# 83 "QC_AutoGate.qpp"
ag->deref(xsink);
}

DLLLOCAL void preinitAutoGateClass() {
    QC_AUTOGATE = new QoreBuiltinClass("AutoGate", "::Qore::Thread::AutoGate", QDOM_THREAD_CLASS);
    CID_AUTOGATE = QC_AUTOGATE->getID();
    QC_AUTOGATE->setSystem();
}

DLLLOCAL QoreClass* initAutoGateClass(QoreNamespace& ns) {
    if (!QC_AUTOGATE)
        preinitAutoGateClass();

    // AutoGate::constructor(Gate gate) {}
    QC_AUTOGATE->addConstructor(AutoGate_constructor_C4Gate, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, QC_GATE->getTypeInfo(), QORE_PARAM_NO_ARG, "gate");

    // AutoGate::copy() {}
    QC_AUTOGATE->setCopy((q_copy_t)AutoGate_copy);

    // AutoGate::destructor() {}
    QC_AUTOGATE->setDestructor((q_destructor_t)AutoGate_destructor);

    return QC_AUTOGATE;
}
