/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    QC_AutoWriteLock.cpp

    Qore Programming Language

    Copyright (C) 2003 - 2023 David Nichols

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_AutoWriteLock.h"
#include "qore/intern/QC_RWLock.h"
/* Qore class Qore::Thread::AutoWriteLock */

qore_classid_t CID_AUTOWRITELOCK;
QoreClass* QC_AUTOWRITELOCK;

// AutoWriteLock::constructor(RWLock rwl) {}
static void AutoWriteLock_constructor_C6RWLock(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(rwl, RWLock, args, 0, CID_RWLOCK, "AutoWriteLock::constructor()", "RWLock", xsink);
    if (*xsink)
        return;
# 81 "QC_AutoWriteLock.qpp"
ReferenceHolder<QoreAutoWriteLock> arwl(new QoreAutoWriteLock(rwl, xsink), xsink);
    if (!*xsink) {
        self->setPrivate(CID_AUTOWRITELOCK, arwl.release());
    }
}

// AutoWriteLock::copy() {}
static void AutoWriteLock_copy(QoreObject* self, QoreObject* old, QoreAutoWriteLock* arwl, ExceptionSink* xsink) {
# 107 "QC_AutoWriteLock.qpp"
xsink->raiseException("AUTOWRITELOCK-COPY-ERROR", "objects of this class cannot be copied");
}

// AutoWriteLock::destructor() {}
static void AutoWriteLock_destructor(QoreObject* self, QoreAutoWriteLock* arwl, ExceptionSink* xsink) {
# 98 "QC_AutoWriteLock.qpp"
arwl->destructor(xsink);
    arwl->deref(xsink);
}

DLLLOCAL void preinitAutoWriteLockClass() {
    QC_AUTOWRITELOCK = new QoreBuiltinClass("AutoWriteLock", "::Qore::Thread::AutoWriteLock", QDOM_THREAD_CLASS);
    CID_AUTOWRITELOCK = QC_AUTOWRITELOCK->getID();
    QC_AUTOWRITELOCK->setSystem();
}

DLLLOCAL QoreClass* initAutoWriteLockClass(QoreNamespace& ns) {
    if (!QC_AUTOWRITELOCK)
        preinitAutoWriteLockClass();

    // AutoWriteLock::constructor(RWLock rwl) {}
    QC_AUTOWRITELOCK->addConstructor(AutoWriteLock_constructor_C6RWLock, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, QC_RWLOCK->getTypeInfo(), QORE_PARAM_NO_ARG, "rwl");

    // AutoWriteLock::copy() {}
    QC_AUTOWRITELOCK->setCopy((q_copy_t)AutoWriteLock_copy);

    // AutoWriteLock::destructor() {}
    QC_AUTOWRITELOCK->setDestructor((q_destructor_t)AutoWriteLock_destructor);

    return QC_AUTOWRITELOCK;
}
