/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_BinaryInputStream.qpp BinaryInputStream class definition */
/*
    Qore Programming Language

    Copyright (C) 2016 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/BinaryInputStream.h"
/* Qore class Qore::BinaryInputStream */

qore_classid_t CID_BINARYINPUTSTREAM;
QoreClass* QC_BINARYINPUTSTREAM;

// BinaryInputStream::constructor(data src) {}
static void BinaryInputStream_constructor_VD(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue src = get_param_value(args, 0);
# 69 "QC_BinaryInputStream.qpp"
SimpleRefHolder<BinaryInputStream> bis(new BinaryInputStream(src));
   self->setPrivate(CID_BINARYINPUTSTREAM, bis.release());
}

// int BinaryInputStream::peek(){}
static QoreValue BinaryInputStream_peek(QoreObject* self, BinaryInputStream* is, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 118 "QC_BinaryInputStream.qpp"
    return is->peekHelper(xsink);
}

// *binary BinaryInputStream::read(int limit){}
static QoreValue BinaryInputStream_read_Vi(QoreObject* self, BinaryInputStream* is, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 limit = HARD_QORE_VALUE_INT(args, 0);
# 96 "QC_BinaryInputStream.qpp"
    return is->readHelper(limit, xsink);
}

DLLLOCAL void preinitBinaryInputStreamClass() {
    QC_BINARYINPUTSTREAM = new QoreBuiltinClass("BinaryInputStream", "::Qore::BinaryInputStream", QDOM_DEFAULT);
    CID_BINARYINPUTSTREAM = QC_BINARYINPUTSTREAM->getID();
    QC_BINARYINPUTSTREAM->setSystem();
}

DLLLOCAL QoreClass* initBinaryInputStreamClass(QoreNamespace& ns) {
    if (!QC_BINARYINPUTSTREAM)
        preinitBinaryInputStreamClass();

    // set parent class
    assert(QC_INPUTSTREAM);
    QC_BINARYINPUTSTREAM->addBuiltinVirtualBaseClass(QC_INPUTSTREAM);

    QC_BINARYINPUTSTREAM->setFinal();

    // BinaryInputStream::constructor(data src) {}
    QC_BINARYINPUTSTREAM->addConstructor(BinaryInputStream_constructor_VD, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "src");

    // int BinaryInputStream::peek(){}
    QC_BINARYINPUTSTREAM->addMethod("peek", (q_method_n_t)BinaryInputStream_peek, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // *binary BinaryInputStream::read(int limit){}
    QC_BINARYINPUTSTREAM->addMethod("read", (q_method_n_t)BinaryInputStream_read_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, binaryOrNothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "limit");

    return QC_BINARYINPUTSTREAM;
}
