/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_EncodingConversionInputStream.qpp EncodingConversionInputStream class definition */
/*
  Qore Programming Language

  Copyright (C) 2016 - 2024 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include "qore/Qore.h"
#include "qore/intern/EncodingConversionInputStream.h"
#include "qore/intern/QoreObjectIntern.h"
#include "qore/intern/QoreClassIntern.h"
/* Qore class Qore::EncodingConversionInputStream */

qore_classid_t CID_ENCODINGCONVERSIONINPUTSTREAM;
QoreClass* QC_ENCODINGCONVERSIONINPUTSTREAM;

// EncodingConversionInputStream::constructor(Qore::InputStream is, string sourceEncoding, string destEncoding) {}
static void EncodingConversionInputStream_constructor_C11InputStreamVsVs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(is, InputStream, args, 0, CID_INPUTSTREAM, "EncodingConversionInputStream::constructor()", "Qore::InputStream", xsink);
    if (*xsink)
        return;
    const QoreStringNode* sourceEncoding = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* destEncoding = HARD_QORE_VALUE_STRING(args, 2);
# 68 "QC_EncodingConversionInputStream.qpp"
SimpleRefHolder<EncodingConversionInputStream> ecis(new EncodingConversionInputStream(
         is, QEM.findCreate(sourceEncoding), QEM.findCreate(destEncoding), xsink));
   self->setPrivate(CID_ENCODINGCONVERSIONINPUTSTREAM, ecis.release());
   qore_object_private* o = qore_object_private::get(*self);
   const qore_class_private* cls = qore_class_private::get(*QC_ENCODINGCONVERSIONINPUTSTREAM);
   o->setValueIntern(cls, "is", static_cast<QoreObject *>(obj_is->refSelf()), xsink);
}

// int EncodingConversionInputStream::peek(){}
static QoreValue EncodingConversionInputStream_peek(QoreObject* self, EncodingConversionInputStream* is, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 113 "QC_EncodingConversionInputStream.qpp"
    return is->peekHelper(xsink);
}

// *binary EncodingConversionInputStream::read(int limit){}
static QoreValue EncodingConversionInputStream_read_Vi(QoreObject* self, EncodingConversionInputStream* is, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 limit = HARD_QORE_VALUE_INT(args, 0);
# 96 "QC_EncodingConversionInputStream.qpp"
    return is->readHelper(limit, xsink);
}

DLLLOCAL void preinitEncodingConversionInputStreamClass() {
    QC_ENCODINGCONVERSIONINPUTSTREAM = new QoreBuiltinClass("EncodingConversionInputStream", "::Qore::EncodingConversionInputStream", QDOM_DEFAULT);
    CID_ENCODINGCONVERSIONINPUTSTREAM = QC_ENCODINGCONVERSIONINPUTSTREAM->getID();
    QC_ENCODINGCONVERSIONINPUTSTREAM->setSystem();
}

DLLLOCAL QoreClass* initEncodingConversionInputStreamClass(QoreNamespace& ns) {
    if (!QC_ENCODINGCONVERSIONINPUTSTREAM)
        preinitEncodingConversionInputStreamClass();

    // set parent class
    assert(QC_INPUTSTREAM);
    QC_ENCODINGCONVERSIONINPUTSTREAM->addBuiltinVirtualBaseClass(QC_INPUTSTREAM);

    // private:internal members
    QC_ENCODINGCONVERSIONINPUTSTREAM->addMember("is", Internal, QC_INPUTSTREAM->getTypeInfo());

    QC_ENCODINGCONVERSIONINPUTSTREAM->setFinal();

    // EncodingConversionInputStream::constructor(Qore::InputStream is, string sourceEncoding, string destEncoding) {}
    QC_ENCODINGCONVERSIONINPUTSTREAM->addConstructor(EncodingConversionInputStream_constructor_C11InputStreamVsVs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 3, QC_INPUTSTREAM->getTypeInfo(), QORE_PARAM_NO_ARG, "is", stringTypeInfo, QORE_PARAM_NO_ARG, "sourceEncoding", stringTypeInfo, QORE_PARAM_NO_ARG, "destEncoding");

    // int EncodingConversionInputStream::peek(){}
    QC_ENCODINGCONVERSIONINPUTSTREAM->addMethod("peek", (q_method_n_t)EncodingConversionInputStream_peek, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // *binary EncodingConversionInputStream::read(int limit){}
    QC_ENCODINGCONVERSIONINPUTSTREAM->addMethod("read", (q_method_n_t)EncodingConversionInputStream_read_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, binaryOrNothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "limit");

    return QC_ENCODINGCONVERSIONINPUTSTREAM;
}
