/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    QC_Expresion.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_Expression.h"
#include "qore/intern/QC_Program.h"
#include "qore/intern/qore_program_private.h"
/* Qore class Qore::Expression */

qore_classid_t CID_EXPRESSION;
QoreClass* QC_EXPRESSION;

// Expression::constructor(Program pgm, string source, string label) {}
static void Expression_constructor_C7ProgramVsVs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 0, CID_PROGRAM, "Expression::constructor()", "Program", xsink);
    if (*xsink)
        return;
    const QoreStringNode* source = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* label = HARD_QORE_VALUE_STRING(args, 2);
# 49 "QC_Expression.qpp"
ReferenceHolder<QoreProgram> holder(pgm, xsink);

    ReferenceHolder<QoreExpression> exp(new QoreExpression(*pgm, *source, *label, xsink), xsink);
    if (!*xsink) {
        self->setPrivate(CID_EXPRESSION, exp.release());
    }
}

// auto Expression::eval(){}
static QoreValue Expression_eval(QoreObject* self, QoreExpression* e, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 61 "QC_Expression.qpp"
    return e->eval(xsink);
}

DLLLOCAL void preinitExpressionClass() {
    QC_EXPRESSION = new QoreBuiltinClass("Expression", "::Qore::Expression", QDOM_DEFAULT);
    CID_EXPRESSION = QC_EXPRESSION->getID();
    QC_EXPRESSION->setSystem();
}

DLLLOCAL QoreClass* initExpressionClass(QoreNamespace& ns) {
    if (!QC_EXPRESSION)
        preinitExpressionClass();

    // Expression::constructor(Program pgm, string source, string label) {}
    QC_EXPRESSION->addConstructor(Expression_constructor_C7ProgramVsVs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 3, QC_PROGRAM->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", stringTypeInfo, QORE_PARAM_NO_ARG, "source", stringTypeInfo, QORE_PARAM_NO_ARG, "label");

    // auto Expression::eval(){}
    QC_EXPRESSION->addMethod("eval", (q_method_n_t)Expression_eval, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo);

    return QC_EXPRESSION;
}
