/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_Gate.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2023 David Nichols

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_Gate.h"
/* Qore class Qore::Thread::Gate */

qore_classid_t CID_GATE;
QoreClass* QC_GATE;

// Gate::constructor() {}
static void Gate_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 72 "QC_Gate.qpp"
self->setPrivate(CID_GATE, new QoreGate);
}

// Gate::copy() {}
static void Gate_copy(QoreObject* self, QoreObject* old, QoreGate* g, ExceptionSink* xsink) {
# 98 "QC_Gate.qpp"
self->setPrivate(CID_GATE, new QoreGate());
}

// Gate::destructor() {}
static void Gate_destructor(QoreObject* self, QoreGate* g, ExceptionSink* xsink) {
# 86 "QC_Gate.qpp"
g->destructor(xsink);
   g->deref(xsink);
}

// int Gate::enter(timeout timeout_ms){}
static QoreValue Gate_enter_Vt(QoreObject* self, QoreGate* g, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 117 "QC_Gate.qpp"
    return g->grab(xsink, timeout_ms);
}

// nothing Gate::enter(){}
static QoreValue Gate_enter(QoreObject* self, QoreGate* g, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 131 "QC_Gate.qpp"
    g->grab(xsink);
    return QoreValue();
}

// int Gate::exit(){}
static QoreValue Gate_exit(QoreObject* self, QoreGate* g, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 146 "QC_Gate.qpp"
    return g->release(xsink);
}

// int Gate::numInside(){}
static QoreValue Gate_numInside(QoreObject* self, QoreGate* g, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 176 "QC_Gate.qpp"
    return g->get_count();
}

// int Gate::numWaiting(){}
static QoreValue Gate_numWaiting(QoreObject* self, QoreGate* g, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 187 "QC_Gate.qpp"
    return g->get_waiting();
}

// int Gate::tryEnter(){}
static QoreValue Gate_tryEnter(QoreObject* self, QoreGate* g, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 164 "QC_Gate.qpp"
    return g->tryGrab();
}

DLLLOCAL void preinitGateClass() {
    QC_GATE = new QoreBuiltinClass("Gate", "::Qore::Thread::Gate", QDOM_THREAD_CLASS);
    CID_GATE = QC_GATE->getID();
    QC_GATE->setSystem();
}

DLLLOCAL QoreClass* initGateClass(QoreNamespace& ns) {
    if (!QC_GATE)
        preinitGateClass();

    // Gate::constructor() {}
    QC_GATE->addConstructor(Gate_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // Gate::copy() {}
    QC_GATE->setCopy((q_copy_t)Gate_copy);

    // Gate::destructor() {}
    QC_GATE->setDestructor((q_destructor_t)Gate_destructor);

    // int Gate::enter(timeout timeout_ms){}
    QC_GATE->addMethod("enter", (q_method_n_t)Gate_enter_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, QORE_PARAM_NO_ARG, "timeout_ms");

    // nothing Gate::enter(){}
    QC_GATE->addMethod("enter", (q_method_n_t)Gate_enter, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // int Gate::exit(){}
    QC_GATE->addMethod("exit", (q_method_n_t)Gate_exit, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // int Gate::numInside(){}
    QC_GATE->addMethod("numInside", (q_method_n_t)Gate_numInside, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int Gate::numWaiting(){}
    QC_GATE->addMethod("numWaiting", (q_method_n_t)Gate_numWaiting, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int Gate::tryEnter(){}
    QC_GATE->addMethod("tryEnter", (q_method_n_t)Gate_tryEnter, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    return QC_GATE;
}
