/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_HashKeyReverseIterator.qpp HashKeyReverseIterator class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QoreHashIterator.h"

extern QoreClass* QC_HASHREVERSEITERATOR;
/* Qore class Qore::HashKeyReverseIterator */

qore_classid_t CID_HASHKEYREVERSEITERATOR;
QoreClass* QC_HASHKEYREVERSEITERATOR;

// HashKeyReverseIterator::constructor(hash<auto> h) {}
static void HashKeyReverseIterator_constructor_C10hash_auto_(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = HARD_QORE_VALUE_HASH(args, 0);
# 81 "QC_HashKeyReverseIterator.qpp"
self->setPrivate(CID_HASHKEYREVERSEITERATOR, new QoreHashReverseIterator(h));
}

// HashKeyReverseIterator::constructor() {}
static void HashKeyReverseIterator_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 92 "QC_HashKeyReverseIterator.qpp"
self->setPrivate(CID_HASHKEYREVERSEITERATOR, new QoreHashReverseIterator);
}

// HashKeyReverseIterator::copy() {}
static void HashKeyReverseIterator_copy(QoreObject* self, QoreObject* old, QoreHashReverseIterator* i, ExceptionSink* xsink) {
# 102 "QC_HashKeyReverseIterator.qpp"
self->setPrivate(CID_HASHKEYREVERSEITERATOR, new QoreHashReverseIterator(*i));
}

// string HashKeyReverseIterator::getValue(){}
static QoreValue HashKeyReverseIterator_getValue(QoreObject* self, QoreHashReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 119 "QC_HashKeyReverseIterator.qpp"
    if (i->check(xsink))
      return QoreValue();
   return i->getKey(xsink);
}

DLLLOCAL void preinitHashKeyReverseIteratorClass() {
    QC_HASHKEYREVERSEITERATOR = new QoreBuiltinClass("HashKeyReverseIterator", "::Qore::HashKeyReverseIterator", QDOM_DEFAULT);
    CID_HASHKEYREVERSEITERATOR = QC_HASHKEYREVERSEITERATOR->getID();
    QC_HASHKEYREVERSEITERATOR->setSystem();
}

DLLLOCAL QoreClass* initHashKeyReverseIteratorClass(QoreNamespace& ns) {
    if (!QC_HASHKEYREVERSEITERATOR)
        preinitHashKeyReverseIteratorClass();

    // set parent class
    assert(QC_HASHREVERSEITERATOR);
    QC_HASHKEYREVERSEITERATOR->addBuiltinVirtualBaseClass(QC_HASHREVERSEITERATOR);

    // HashKeyReverseIterator::constructor(hash<auto> h) {}
    QC_HASHKEYREVERSEITERATOR->addConstructor(HashKeyReverseIterator_constructor_C10hash_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, autoHashTypeInfo, QORE_PARAM_NO_ARG, "h");

    // HashKeyReverseIterator::constructor() {}
    QC_HASHKEYREVERSEITERATOR->addConstructor(HashKeyReverseIterator_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // HashKeyReverseIterator::copy() {}
    QC_HASHKEYREVERSEITERATOR->setCopy((q_copy_t)HashKeyReverseIterator_copy);

    // string HashKeyReverseIterator::getValue(){}
    QC_HASHKEYREVERSEITERATOR->addMethod("getValue", (q_method_n_t)HashKeyReverseIterator_getValue, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    return QC_HASHKEYREVERSEITERATOR;
}
