/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_HashListIterator.qpp HashListIterator class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QoreHashListIterator.h"
/* Qore class Qore::HashListIterator */

qore_classid_t CID_HASHLISTITERATOR;
QoreClass* QC_HASHLISTITERATOR;

// HashListIterator::constructor(hash<auto> h) {}
static void HashListIterator_constructor_C10hash_auto_(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = HARD_QORE_VALUE_HASH(args, 0);
# 82 "QC_HashListIterator.qpp"
self->setPrivate(CID_HASHLISTITERATOR, new QoreHashListIterator(h));
}

// HashListIterator::constructor() {}
static void HashListIterator_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 93 "QC_HashListIterator.qpp"
self->setPrivate(CID_HASHLISTITERATOR, new QoreHashListIterator);
}

// HashListIterator::copy() {}
static void HashListIterator_copy(QoreObject* self, QoreObject* old, QoreHashListIterator* i, ExceptionSink* xsink) {
# 103 "QC_HashListIterator.qpp"
self->setPrivate(CID_HASHLISTITERATOR, new QoreHashListIterator(*i));
}

// bool HashListIterator::empty(){}
static QoreValue HashListIterator_empty(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 158 "QC_HashListIterator.qpp"
    return i->empty();
}

// bool HashListIterator::first(){}
static QoreValue HashListIterator_first(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 173 "QC_HashListIterator.qpp"
    return i->first();
}

// auto HashListIterator::getKeyValue(string key){}
static QoreValue HashListIterator_getKeyValue_Vs(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 0);
# 327 "QC_HashListIterator.qpp"
    if (i->check(xsink))
      return QoreValue();

   return i->getReferencedKeyValue(key->getBuffer(), xsink);
}

// hash<auto> HashListIterator::getRow(){}
static QoreValue HashListIterator_getRow(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 231 "QC_HashListIterator.qpp"
    if (i->check(xsink))
      return QoreValue();
   return i->getRow(xsink);
}

// hash<auto> HashListIterator::getValue(){}
static QoreValue HashListIterator_getValue(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 209 "QC_HashListIterator.qpp"
    if (i->check(xsink))
      return QoreValue();
   return i->getRow(xsink);
}

// int HashListIterator::index(){}
static QoreValue HashListIterator_index(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 247 "QC_HashListIterator.qpp"
    return i->index();
}

// bool HashListIterator::last(){}
static QoreValue HashListIterator_last(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 188 "QC_HashListIterator.qpp"
    return i->last();
}

// int HashListIterator::max(){}
static QoreValue HashListIterator_max(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 261 "QC_HashListIterator.qpp"
    return i->max();
}

// auto HashListIterator::memberGate(string key){}
static QoreValue HashListIterator_memberGate_Vs(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 0);
# 302 "QC_HashListIterator.qpp"
    if (i->check(xsink))
      return QoreValue();

   return i->getReferencedKeyValue(key->getBuffer(), xsink);
}

// bool HashListIterator::next(){}
static QoreValue HashListIterator_next(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 122 "QC_HashListIterator.qpp"
    if (i->check(xsink))
      return false;
   return i->next();
}

// bool HashListIterator::prev(){}
static QoreValue HashListIterator_prev(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 143 "QC_HashListIterator.qpp"
    if (i->check(xsink))
      return false;
   return i->prev();
}

// nothing HashListIterator::reset(){}
static QoreValue HashListIterator_reset(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 357 "QC_HashListIterator.qpp"
    if (i->check(xsink))
       return QoreValue();
   i->reset();
    return QoreValue();
}

// bool HashListIterator::set(int pos){}
static QoreValue HashListIterator_set_Vi(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 pos = HARD_QORE_VALUE_INT(args, 0);
# 278 "QC_HashListIterator.qpp"
    if (i->check(xsink))
      return false;
   return !i->set(pos);
}

// bool HashListIterator::valid(){}
static QoreValue HashListIterator_valid(QoreObject* self, QoreHashListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 343 "QC_HashListIterator.qpp"
    return i->valid();
}

DLLLOCAL void preinitHashListIteratorClass() {
    QC_HASHLISTITERATOR = new QoreBuiltinClass("HashListIterator", "::Qore::HashListIterator", QDOM_DEFAULT);
    CID_HASHLISTITERATOR = QC_HASHLISTITERATOR->getID();
    QC_HASHLISTITERATOR->setSystem();
}

DLLLOCAL QoreClass* initHashListIteratorClass(QoreNamespace& ns) {
    if (!QC_HASHLISTITERATOR)
        preinitHashListIteratorClass();

    // set parent class
    assert(QC_ABSTRACTQUANTIFIEDBIDIRECTIONALITERATOR);
    QC_HASHLISTITERATOR->addBuiltinVirtualBaseClass(QC_ABSTRACTQUANTIFIEDBIDIRECTIONALITERATOR);

    // HashListIterator::constructor(hash<auto> h) {}
    QC_HASHLISTITERATOR->addConstructor(HashListIterator_constructor_C10hash_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, autoHashTypeInfo, QORE_PARAM_NO_ARG, "h");

    // HashListIterator::constructor() {}
    QC_HASHLISTITERATOR->addConstructor(HashListIterator_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // HashListIterator::copy() {}
    QC_HASHLISTITERATOR->setCopy((q_copy_t)HashListIterator_copy);

    // bool HashListIterator::empty(){}
    QC_HASHLISTITERATOR->addMethod("empty", (q_method_n_t)HashListIterator_empty, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool HashListIterator::first(){}
    QC_HASHLISTITERATOR->addMethod("first", (q_method_n_t)HashListIterator_first, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // auto HashListIterator::getKeyValue(string key){}
    QC_HASHLISTITERATOR->addMethod("getKeyValue", (q_method_n_t)HashListIterator_getKeyValue_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "key");

    // hash<auto> HashListIterator::getRow(){}
    QC_HASHLISTITERATOR->addMethod("getRow", (q_method_n_t)HashListIterator_getRow, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoHashTypeInfo);

    // hash<auto> HashListIterator::getValue(){}
    QC_HASHLISTITERATOR->addMethod("getValue", (q_method_n_t)HashListIterator_getValue, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoHashTypeInfo);

    // int HashListIterator::index(){}
    QC_HASHLISTITERATOR->addMethod("index", (q_method_n_t)HashListIterator_index, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool HashListIterator::last(){}
    QC_HASHLISTITERATOR->addMethod("last", (q_method_n_t)HashListIterator_last, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // int HashListIterator::max(){}
    QC_HASHLISTITERATOR->addMethod("max", (q_method_n_t)HashListIterator_max, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // auto HashListIterator::memberGate(string key){}
    QC_HASHLISTITERATOR->addMethod("memberGate", (q_method_n_t)HashListIterator_memberGate_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "key");

    // bool HashListIterator::next(){}
    QC_HASHLISTITERATOR->addMethod("next", (q_method_n_t)HashListIterator_next, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // bool HashListIterator::prev(){}
    QC_HASHLISTITERATOR->addMethod("prev", (q_method_n_t)HashListIterator_prev, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // nothing HashListIterator::reset(){}
    QC_HASHLISTITERATOR->addMethod("reset", (q_method_n_t)HashListIterator_reset, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // bool HashListIterator::set(int pos){}
    QC_HASHLISTITERATOR->addMethod("set", (q_method_n_t)HashListIterator_set_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "pos");

    // bool HashListIterator::valid(){}
    QC_HASHLISTITERATOR->addMethod("valid", (q_method_n_t)HashListIterator_valid, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    return QC_HASHLISTITERATOR;
}
