/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_HashReverseIterator.qpp HashReverseIterator class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QoreHashIterator.h"

extern QoreClass* QC_HASHITERATOR;
/* Qore class Qore::HashReverseIterator */

qore_classid_t CID_HASHREVERSEITERATOR;
QoreClass* QC_HASHREVERSEITERATOR;

// HashReverseIterator::constructor(hash<auto> h) {}
static void HashReverseIterator_constructor_C10hash_auto_(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = HARD_QORE_VALUE_HASH(args, 0);
# 77 "QC_HashReverseIterator.qpp"
self->setPrivate(CID_HASHREVERSEITERATOR, new QoreHashReverseIterator(h));
}

// HashReverseIterator::constructor() {}
static void HashReverseIterator_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 88 "QC_HashReverseIterator.qpp"
self->setPrivate(CID_HASHREVERSEITERATOR, new QoreHashReverseIterator);
}

// HashReverseIterator::copy() {}
static void HashReverseIterator_copy(QoreObject* self, QoreObject* old, QoreHashReverseIterator* i, ExceptionSink* xsink) {
# 98 "QC_HashReverseIterator.qpp"
self->setPrivate(CID_HASHREVERSEITERATOR, new QoreHashReverseIterator(*i));
}

// bool HashReverseIterator::first(){}
static QoreValue HashReverseIterator_first(QoreObject* self, QoreHashReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 161 "QC_HashReverseIterator.qpp"
    return i->last();
}

// bool HashReverseIterator::last(){}
static QoreValue HashReverseIterator_last(QoreObject* self, QoreHashReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 176 "QC_HashReverseIterator.qpp"
    return i->first();
}

// bool HashReverseIterator::next(){}
static QoreValue HashReverseIterator_next(QoreObject* self, QoreHashReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 120 "QC_HashReverseIterator.qpp"
    if (i->check(xsink))
        return false;
    return i->prev();
}

// bool HashReverseIterator::prev(){}
static QoreValue HashReverseIterator_prev(QoreObject* self, QoreHashReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 144 "QC_HashReverseIterator.qpp"
    if (i->check(xsink))
        return false;
    return i->next();
}

DLLLOCAL void preinitHashReverseIteratorClass() {
    QC_HASHREVERSEITERATOR = new QoreBuiltinClass("HashReverseIterator", "::Qore::HashReverseIterator", QDOM_DEFAULT);
    CID_HASHREVERSEITERATOR = QC_HASHREVERSEITERATOR->getID();
    QC_HASHREVERSEITERATOR->setSystem();
}

DLLLOCAL QoreClass* initHashReverseIteratorClass(QoreNamespace& ns) {
    if (!QC_HASHREVERSEITERATOR)
        preinitHashReverseIteratorClass();

    // set parent class
    assert(QC_HASHITERATOR);
    QC_HASHREVERSEITERATOR->addBuiltinVirtualBaseClass(QC_HASHITERATOR);

    // HashReverseIterator::constructor(hash<auto> h) {}
    QC_HASHREVERSEITERATOR->addConstructor(HashReverseIterator_constructor_C10hash_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, autoHashTypeInfo, QORE_PARAM_NO_ARG, "h");

    // HashReverseIterator::constructor() {}
    QC_HASHREVERSEITERATOR->addConstructor(HashReverseIterator_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // HashReverseIterator::copy() {}
    QC_HASHREVERSEITERATOR->setCopy((q_copy_t)HashReverseIterator_copy);

    // bool HashReverseIterator::first(){}
    QC_HASHREVERSEITERATOR->addMethod("first", (q_method_n_t)HashReverseIterator_first, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool HashReverseIterator::last(){}
    QC_HASHREVERSEITERATOR->addMethod("last", (q_method_n_t)HashReverseIterator_last, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool HashReverseIterator::next(){}
    QC_HASHREVERSEITERATOR->addMethod("next", (q_method_n_t)HashReverseIterator_next, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // bool HashReverseIterator::prev(){}
    QC_HASHREVERSEITERATOR->addMethod("prev", (q_method_n_t)HashReverseIterator_prev, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    return QC_HASHREVERSEITERATOR;
}
