/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_InputStream.qpp InputStream class (interface) definition */
/*
    Qore Programming Language

    Copyright (C) 2016 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/InputStreamWrapper.h"
/* Qore class Qore::InputStream */

qore_classid_t CID_INPUTSTREAM;
QoreClass* QC_INPUTSTREAM;

// InputStream::constructor() {}
static void InputStream_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 56 "QC_InputStream.qpp"
self->setPrivate(CID_INPUTSTREAM, new InputStreamWrapper(self));
}

DLLLOCAL void preinitInputStreamClass() {
    QC_INPUTSTREAM = new QoreBuiltinClass("InputStream", "::Qore::InputStream", QDOM_DEFAULT);
    CID_INPUTSTREAM = QC_INPUTSTREAM->getID();
    QC_INPUTSTREAM->setSystem();
}

DLLLOCAL QoreClass* initInputStreamClass(QoreNamespace& ns) {
    if (!QC_INPUTSTREAM)
        preinitInputStreamClass();

    // set parent class
    assert(QC_STREAMBASE);
    QC_INPUTSTREAM->addBuiltinVirtualBaseClass(QC_STREAMBASE);

    // InputStream::constructor() {}
    QC_INPUTSTREAM->addConstructor(InputStream_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // int InputStream::peek();
    QC_INPUTSTREAM->addAbstractMethod("peek", Public, QCF_NO_FLAGS, bigIntTypeInfo);

    // *binary InputStream::read(int limit);
    QC_INPUTSTREAM->addAbstractMethod("read", Public, QCF_NO_FLAGS, binaryOrNothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "limit");

    return QC_INPUTSTREAM;
}
