/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_InputStreamLineIterator.qpp InputStreamLineIterator class definition */
/*
  Qore Programming Language

  Copyright (C) 2016 - 2024 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include "qore/Qore.h"
#include "qore/InputStream.h"
#include "qore/intern/InputStreamLineIterator.h"
#include "qore/intern/QoreObjectIntern.h"
#include "qore/intern/QoreClassIntern.h"
#include "qore/intern/StreamReader.h"
/* Qore class Qore::InputStreamLineIterator */

qore_classid_t CID_INPUTSTREAMLINEITERATOR;
QoreClass* QC_INPUTSTREAMLINEITERATOR;

// InputStreamLineIterator::constructor(Qore::InputStream is, *string encoding, *string eol, bool trim = True, int bufsize = DefaultStreamBufferSize) {}
static void InputStreamLineIterator_constructor_C11InputStreamNsNsVbVi(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(is, InputStream, args, 0, CID_INPUTSTREAM, "InputStreamLineIterator::constructor()", "Qore::InputStream", xsink);
    if (*xsink)
        return;
    const QoreStringNode* encoding = get_param_value(args, 1).get<const QoreStringNode>();
    const QoreStringNode* eol = get_param_value(args, 2).get<const QoreStringNode>();
    bool trim = HARD_QORE_VALUE_BOOL(args, 3);
    int64 bufsize = HARD_QORE_VALUE_INT(args, 4);
# 76 "QC_InputStreamLineIterator.qpp"
    if (eol && eol->empty())
        eol = nullptr;
    SimpleRefHolder<InputStreamLineIterator> li(new InputStreamLineIterator(xsink, is,
        encoding ? QEM.findCreate(encoding) : QCS_DEFAULT, eol, trim, bufsize));
    if (*xsink)
        return;
    self->setPrivate(CID_INPUTSTREAMLINEITERATOR, li.release());
    qore_object_private* o = qore_object_private::get(*self);
    const qore_class_private* cls = qore_class_private::get(*QC_INPUTSTREAMLINEITERATOR);
    o->setValueIntern(cls, "is", static_cast<QoreObject *>(obj_is->refSelf()), xsink);
}

// InputStreamLineIterator::constructor(Qore::StreamReader sr, *string eol, bool trim = True) {}
static void InputStreamLineIterator_constructor_C12StreamReaderNsVb(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(sr, StreamReader, args, 0, CID_STREAMREADER, "InputStreamLineIterator::constructor()", "Qore::StreamReader", xsink);
    if (*xsink)
        return;
    const QoreStringNode* eol = get_param_value(args, 1).get<const QoreStringNode>();
    bool trim = HARD_QORE_VALUE_BOOL(args, 2);
# 100 "QC_InputStreamLineIterator.qpp"
if (eol && eol->empty())
        eol = nullptr;
    SimpleRefHolder<InputStreamLineIterator> li(new InputStreamLineIterator(xsink, sr, eol, trim));
    if (*xsink)
        return;
    self->setPrivate(CID_INPUTSTREAMLINEITERATOR, li.release());
    qore_object_private* o = qore_object_private::get(*self);
    const qore_class_private* cls = qore_class_private::get(*QC_INPUTSTREAMLINEITERATOR);
    o->setValueIntern(cls, "sr", static_cast<QoreObject *>(obj_sr->refSelf()), xsink);
}

// string InputStreamLineIterator::getEncoding(){}
static QoreValue InputStreamLineIterator_getEncoding(QoreObject* self, InputStreamLineIterator* isli, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 219 "QC_InputStreamLineIterator.qpp"
    return new QoreStringNode(isli->getEncoding());
}

// string InputStreamLineIterator::getLine(){}
static QoreValue InputStreamLineIterator_getLine(QoreObject* self, InputStreamLineIterator* isli, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 166 "QC_InputStreamLineIterator.qpp"
    return isli->checkValid(xsink) ? nullptr : isli->getValue();
}

// StreamReader InputStreamLineIterator::getStreamReader(){}
static QoreValue InputStreamLineIterator_getStreamReader(QoreObject* self, InputStreamLineIterator* isli, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 205 "QC_InputStreamLineIterator.qpp"
    StreamReader* sr = isli->getStreamReader();
    sr->ref();
    return new QoreObject(QC_STREAMREADER, getProgram(), sr);
}

// string InputStreamLineIterator::getValue(){}
static QoreValue InputStreamLineIterator_getValue(QoreObject* self, InputStreamLineIterator* isli, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 147 "QC_InputStreamLineIterator.qpp"
    return isli->checkValid(xsink) ? nullptr : isli->getValue();
}

// int InputStreamLineIterator::index(){}
static QoreValue InputStreamLineIterator_index(QoreObject* self, InputStreamLineIterator* isli, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 193 "QC_InputStreamLineIterator.qpp"
    return isli->index();
}

// bool InputStreamLineIterator::next(){}
static QoreValue InputStreamLineIterator_next(QoreObject* self, InputStreamLineIterator* isli, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 126 "QC_InputStreamLineIterator.qpp"
    if (isli->check(xsink))
        return false;
    return isli->next(xsink);
}

// bool InputStreamLineIterator::valid(){}
static QoreValue InputStreamLineIterator_valid(QoreObject* self, InputStreamLineIterator* isli, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 179 "QC_InputStreamLineIterator.qpp"
    return isli->valid();
}

DLLLOCAL void preinitInputStreamLineIteratorClass() {
    QC_INPUTSTREAMLINEITERATOR = new QoreBuiltinClass("InputStreamLineIterator", "::Qore::InputStreamLineIterator", QDOM_DEFAULT);
    CID_INPUTSTREAMLINEITERATOR = QC_INPUTSTREAMLINEITERATOR->getID();
    QC_INPUTSTREAMLINEITERATOR->setSystem();
}

DLLLOCAL QoreClass* initInputStreamLineIteratorClass(QoreNamespace& ns) {
    if (!QC_INPUTSTREAMLINEITERATOR)
        preinitInputStreamLineIteratorClass();

    // set parent class
    assert(QC_ABSTRACTLINEITERATOR);
    QC_INPUTSTREAMLINEITERATOR->addBuiltinVirtualBaseClass(QC_ABSTRACTLINEITERATOR);

    // private:internal members
    QC_INPUTSTREAMLINEITERATOR->addMember("is", Internal, QC_INPUTSTREAM->getTypeInfo());
    QC_INPUTSTREAMLINEITERATOR->addMember("sr", Internal, QC_STREAMREADER->getTypeInfo());

    // InputStreamLineIterator::constructor(Qore::InputStream is, *string encoding, *string eol, bool trim = True, int bufsize = DefaultStreamBufferSize) {}
    QC_INPUTSTREAMLINEITERATOR->addConstructor(InputStreamLineIterator_constructor_C11InputStreamNsNsVbVi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 5, QC_INPUTSTREAM->getTypeInfo(), QORE_PARAM_NO_ARG, "is", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "eol", boolTypeInfo, QoreSimpleValue().assign(true), "trim", bigIntTypeInfo, QoreSimpleValue().assign((int64)DefaultStreamBufferSize), "bufsize");

    // InputStreamLineIterator::constructor(Qore::StreamReader sr, *string eol, bool trim = True) {}
    QC_INPUTSTREAMLINEITERATOR->addConstructor(InputStreamLineIterator_constructor_C12StreamReaderNsVb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 3, QC_STREAMREADER->getTypeInfo(), QORE_PARAM_NO_ARG, "sr", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "eol", boolTypeInfo, QoreSimpleValue().assign(true), "trim");

    // string InputStreamLineIterator::getEncoding(){}
    QC_INPUTSTREAMLINEITERATOR->addMethod("getEncoding", (q_method_n_t)InputStreamLineIterator_getEncoding, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // string InputStreamLineIterator::getLine(){}
    QC_INPUTSTREAMLINEITERATOR->addMethod("getLine", (q_method_n_t)InputStreamLineIterator_getLine, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // StreamReader InputStreamLineIterator::getStreamReader(){}
    QC_INPUTSTREAMLINEITERATOR->addMethod("getStreamReader", (q_method_n_t)InputStreamLineIterator_getStreamReader, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_STREAMREADER->getTypeInfo());

    // string InputStreamLineIterator::getValue(){}
    QC_INPUTSTREAMLINEITERATOR->addMethod("getValue", (q_method_n_t)InputStreamLineIterator_getValue, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // int InputStreamLineIterator::index(){}
    QC_INPUTSTREAMLINEITERATOR->addMethod("index", (q_method_n_t)InputStreamLineIterator_index, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool InputStreamLineIterator::next(){}
    QC_INPUTSTREAMLINEITERATOR->addMethod("next", (q_method_n_t)InputStreamLineIterator_next, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // bool InputStreamLineIterator::valid(){}
    QC_INPUTSTREAMLINEITERATOR->addMethod("valid", (q_method_n_t)InputStreamLineIterator_valid, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    return QC_INPUTSTREAMLINEITERATOR;
}
