/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_ListHashIterator.qpp ListHashIterator class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QoreListHashIterator.h"

extern QoreClass* QC_LISTHASHITERATOR;

// internal reverse iterator class implementation only for the getName() function - the iterators remain
// forwards and are used in the reverse sense by the Qore language class implementation below
class QoreListHashReverseIterator : public QoreListHashIterator {
public:
   DLLLOCAL QoreListHashReverseIterator(const QoreListNode* n_l) : QoreListHashIterator(n_l) {
   }

   DLLLOCAL QoreListHashReverseIterator(const QoreListHashReverseIterator& old) : QoreListHashIterator(old) {
   }

   DLLLOCAL virtual const char* getName() const {
      return "ListHashReverseIterator";
   }
};
/* Qore class Qore::ListHashReverseIterator */

qore_classid_t CID_LISTHASHREVERSEITERATOR;
QoreClass* QC_LISTHASHREVERSEITERATOR;

// ListHashReverseIterator::constructor(softlist<auto> l) {}
static void ListHashReverseIterator_constructor_C14softlist_auto_(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 96 "QC_ListHashReverseIterator.qpp"
self->setPrivate(CID_LISTHASHREVERSEITERATOR, new QoreListHashReverseIterator(l));
}

// ListHashReverseIterator::copy() {}
static void ListHashReverseIterator_copy(QoreObject* self, QoreObject* old, QoreListHashReverseIterator* i, ExceptionSink* xsink) {
# 106 "QC_ListHashReverseIterator.qpp"
self->setPrivate(CID_LISTHASHREVERSEITERATOR, new QoreListHashReverseIterator(*i));
}

// bool ListHashReverseIterator::first(){}
static QoreValue ListHashReverseIterator_first(QoreObject* self, QoreListHashReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 163 "QC_ListHashReverseIterator.qpp"
    return i->last();
}

// bool ListHashReverseIterator::last(){}
static QoreValue ListHashReverseIterator_last(QoreObject* self, QoreListHashReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 178 "QC_ListHashReverseIterator.qpp"
    return i->first();
}

// auto ListHashReverseIterator::memberGate(string key){}
static QoreValue ListHashReverseIterator_memberGate_Vs(QoreObject* self, QoreListHashReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 0);
# 199 "QC_ListHashReverseIterator.qpp"
    if (i->check(xsink))
      return QoreValue();

   return i->getReferencedKeyValue(key->getBuffer(), xsink);
}

// bool ListHashReverseIterator::next(){}
static QoreValue ListHashReverseIterator_next(QoreObject* self, QoreListHashReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 125 "QC_ListHashReverseIterator.qpp"
    if (i->check(xsink))
      return false;
   return i->prev();
}

// bool ListHashReverseIterator::prev(){}
static QoreValue ListHashReverseIterator_prev(QoreObject* self, QoreListHashReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 146 "QC_ListHashReverseIterator.qpp"
    if (i->check(xsink))
      return false;
   return i->next();
}

DLLLOCAL void preinitListHashReverseIteratorClass() {
    QC_LISTHASHREVERSEITERATOR = new QoreBuiltinClass("ListHashReverseIterator", "::Qore::ListHashReverseIterator", QDOM_DEFAULT);
    CID_LISTHASHREVERSEITERATOR = QC_LISTHASHREVERSEITERATOR->getID();
    QC_LISTHASHREVERSEITERATOR->setSystem();
}

DLLLOCAL QoreClass* initListHashReverseIteratorClass(QoreNamespace& ns) {
    if (!QC_LISTHASHREVERSEITERATOR)
        preinitListHashReverseIteratorClass();

    // set parent class
    assert(QC_LISTHASHITERATOR);
    QC_LISTHASHREVERSEITERATOR->addBuiltinVirtualBaseClass(QC_LISTHASHITERATOR);

    // ListHashReverseIterator::constructor(softlist<auto> l) {}
    QC_LISTHASHREVERSEITERATOR->addConstructor(ListHashReverseIterator_constructor_C14softlist_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, softAutoListTypeInfo, QORE_PARAM_NO_ARG, "l");

    // ListHashReverseIterator::copy() {}
    QC_LISTHASHREVERSEITERATOR->setCopy((q_copy_t)ListHashReverseIterator_copy);

    // bool ListHashReverseIterator::first(){}
    QC_LISTHASHREVERSEITERATOR->addMethod("first", (q_method_n_t)ListHashReverseIterator_first, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool ListHashReverseIterator::last(){}
    QC_LISTHASHREVERSEITERATOR->addMethod("last", (q_method_n_t)ListHashReverseIterator_last, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // auto ListHashReverseIterator::memberGate(string key){}
    QC_LISTHASHREVERSEITERATOR->addMethod("memberGate", (q_method_n_t)ListHashReverseIterator_memberGate_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "key");

    // bool ListHashReverseIterator::next(){}
    QC_LISTHASHREVERSEITERATOR->addMethod("next", (q_method_n_t)ListHashReverseIterator_next, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // bool ListHashReverseIterator::prev(){}
    QC_LISTHASHREVERSEITERATOR->addMethod("prev", (q_method_n_t)ListHashReverseIterator_prev, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    return QC_LISTHASHREVERSEITERATOR;
}
