/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_ObjectKeyIterator.qpp ObjectKeyIterator class definition */
/*
  Qore Programming Language

  Copyright (C) 2003 - 2023 David Nichols

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QoreObjectIterator.h"

extern QoreClass* QC_OBJECTITERATOR;
/* Qore class Qore::ObjectKeyIterator */

qore_classid_t CID_OBJECTKEYITERATOR;
QoreClass* QC_OBJECTKEYITERATOR;

// ObjectKeyIterator::constructor(object o) {}
static void ObjectKeyIterator_constructor_Vo(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* o = HARD_QORE_VALUE_OBJECT(args, 0);
# 81 "QC_ObjectKeyIterator.qpp"
self->setPrivate(CID_OBJECTKEYITERATOR, new QoreObjectIterator(o));
}

// ObjectKeyIterator::constructor() {}
static void ObjectKeyIterator_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 92 "QC_ObjectKeyIterator.qpp"
self->setPrivate(CID_OBJECTKEYITERATOR, new QoreObjectIterator);
}

// ObjectKeyIterator::copy() {}
static void ObjectKeyIterator_copy(QoreObject* self, QoreObject* old, QoreObjectIterator* i, ExceptionSink* xsink) {
# 102 "QC_ObjectKeyIterator.qpp"
self->setPrivate(CID_OBJECTKEYITERATOR, new QoreObjectIterator(*i));
}

// string ObjectKeyIterator::getValue(){}
static QoreValue ObjectKeyIterator_getValue(QoreObject* self, QoreObjectIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 118 "QC_ObjectKeyIterator.qpp"
    if (i->check(xsink))
      return QoreValue();
   return i->getKey(xsink);
}

DLLLOCAL void preinitObjectKeyIteratorClass() {
    QC_OBJECTKEYITERATOR = new QoreBuiltinClass("ObjectKeyIterator", "::Qore::ObjectKeyIterator", QDOM_DEFAULT);
    CID_OBJECTKEYITERATOR = QC_OBJECTKEYITERATOR->getID();
    QC_OBJECTKEYITERATOR->setSystem();
}

DLLLOCAL QoreClass* initObjectKeyIteratorClass(QoreNamespace& ns) {
    if (!QC_OBJECTKEYITERATOR)
        preinitObjectKeyIteratorClass();

    // set parent class
    assert(QC_OBJECTITERATOR);
    QC_OBJECTKEYITERATOR->addBuiltinVirtualBaseClass(QC_OBJECTITERATOR);

    // ObjectKeyIterator::constructor(object o) {}
    QC_OBJECTKEYITERATOR->addConstructor(ObjectKeyIterator_constructor_Vo, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, objectTypeInfo, QORE_PARAM_NO_ARG, "o");

    // ObjectKeyIterator::constructor() {}
    QC_OBJECTKEYITERATOR->addConstructor(ObjectKeyIterator_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // ObjectKeyIterator::copy() {}
    QC_OBJECTKEYITERATOR->setCopy((q_copy_t)ObjectKeyIterator_copy);

    // string ObjectKeyIterator::getValue(){}
    QC_OBJECTKEYITERATOR->addMethod("getValue", (q_method_n_t)ObjectKeyIterator_getValue, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    return QC_OBJECTKEYITERATOR;
}
