/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_ObjectIterator.qpp ObjectIterator class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QoreObjectIterator.h"

extern QoreClass* QC_OBJECTITERATOR;
/* Qore class Qore::ObjectPairIterator */

qore_classid_t CID_OBJECTPAIRITERATOR;
QoreClass* QC_OBJECTPAIRITERATOR;

// ObjectPairIterator::constructor(object o) {}
static void ObjectPairIterator_constructor_Vo(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* o = HARD_QORE_VALUE_OBJECT(args, 0);
# 81 "QC_ObjectPairIterator.qpp"
self->setPrivate(CID_OBJECTPAIRITERATOR, new QoreObjectIterator(o));
}

// ObjectPairIterator::constructor() {}
static void ObjectPairIterator_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 92 "QC_ObjectPairIterator.qpp"
self->setPrivate(CID_OBJECTPAIRITERATOR, new QoreObjectIterator);
}

// ObjectPairIterator::copy() {}
static void ObjectPairIterator_copy(QoreObject* self, QoreObject* old, QoreObjectIterator* i, ExceptionSink* xsink) {
# 102 "QC_ObjectPairIterator.qpp"
self->setPrivate(CID_OBJECTPAIRITERATOR, new QoreObjectIterator(*i));
}

// hash<auto> ObjectPairIterator::getValue(){}
static QoreValue ObjectPairIterator_getValue(QoreObject* self, QoreObjectIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 119 "QC_ObjectPairIterator.qpp"
    if (i->check(xsink))
      return QoreValue();
   return i->getReferencedValuePair(xsink);
}

DLLLOCAL void preinitObjectPairIteratorClass() {
    QC_OBJECTPAIRITERATOR = new QoreBuiltinClass("ObjectPairIterator", "::Qore::ObjectPairIterator", QDOM_DEFAULT);
    CID_OBJECTPAIRITERATOR = QC_OBJECTPAIRITERATOR->getID();
    QC_OBJECTPAIRITERATOR->setSystem();
}

DLLLOCAL QoreClass* initObjectPairIteratorClass(QoreNamespace& ns) {
    if (!QC_OBJECTPAIRITERATOR)
        preinitObjectPairIteratorClass();

    // set parent class
    assert(QC_OBJECTITERATOR);
    QC_OBJECTPAIRITERATOR->addBuiltinVirtualBaseClass(QC_OBJECTITERATOR);

    // ObjectPairIterator::constructor(object o) {}
    QC_OBJECTPAIRITERATOR->addConstructor(ObjectPairIterator_constructor_Vo, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, objectTypeInfo, QORE_PARAM_NO_ARG, "o");

    // ObjectPairIterator::constructor() {}
    QC_OBJECTPAIRITERATOR->addConstructor(ObjectPairIterator_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // ObjectPairIterator::copy() {}
    QC_OBJECTPAIRITERATOR->setCopy((q_copy_t)ObjectPairIterator_copy);

    // hash<auto> ObjectPairIterator::getValue(){}
    QC_OBJECTPAIRITERATOR->addMethod("getValue", (q_method_n_t)ObjectPairIterator_getValue, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoHashTypeInfo);

    return QC_OBJECTPAIRITERATOR;
}
