/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_OutputStream.qpp OutputStream class (interface) definition */
/*
  Qore Programming Language

  Copyright (C) 2023 - 2024 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/OutputStreamWrapper.h"
/* Qore class Qore::OutputStream */

qore_classid_t CID_OUTPUTSTREAM;
QoreClass* QC_OUTPUTSTREAM;

// OutputStream::constructor() {}
static void OutputStream_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 55 "QC_OutputStream.qpp"
self->setPrivate(CID_OUTPUTSTREAM, new OutputStreamWrapper(self));
}

DLLLOCAL void preinitOutputStreamClass() {
    QC_OUTPUTSTREAM = new QoreBuiltinClass("OutputStream", "::Qore::OutputStream", QDOM_DEFAULT);
    CID_OUTPUTSTREAM = QC_OUTPUTSTREAM->getID();
    QC_OUTPUTSTREAM->setSystem();
}

DLLLOCAL QoreClass* initOutputStreamClass(QoreNamespace& ns) {
    if (!QC_OUTPUTSTREAM)
        preinitOutputStreamClass();

    // set parent class
    assert(QC_STREAMBASE);
    QC_OUTPUTSTREAM->addBuiltinVirtualBaseClass(QC_STREAMBASE);

    // nothing OutputStream::close();
    QC_OUTPUTSTREAM->addAbstractMethod("close", Public, QCF_NO_FLAGS, nothingTypeInfo);

    // OutputStream::constructor() {}
    QC_OUTPUTSTREAM->addConstructor(OutputStream_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // nothing OutputStream::write(binary data);
    QC_OUTPUTSTREAM->addAbstractMethod("write", Public, QCF_NO_FLAGS, nothingTypeInfo, 1, binaryTypeInfo, QORE_PARAM_NO_ARG, "data");

    return QC_OUTPUTSTREAM;
}
