/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_Sequence.qpp
  
  Qore Programming Language
  
  Copyright (C) 2003 - 2023 David Nichols

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_Sequence.h"
/* Qore class Qore::Thread::Sequence */

qore_classid_t CID_SEQUENCE;
QoreClass* QC_SEQUENCE;

// Sequence::constructor() {}
static void Sequence_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 48 "QC_Sequence.qpp"
self->setPrivate(CID_SEQUENCE, new QoreSequence);
}

// Sequence::constructor(softint start) {}
static void Sequence_constructor_vi(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 start = HARD_QORE_VALUE_INT(args, 0);
# 59 "QC_Sequence.qpp"
self->setPrivate(CID_SEQUENCE, new QoreSequence((int)start));
}

// Sequence::copy() {}
static void Sequence_copy(QoreObject* self, QoreObject* old, QoreSequence* s, ExceptionSink* xsink) {
# 70 "QC_Sequence.qpp"
self->setPrivate(CID_SEQUENCE, new QoreSequence(s->getCurrent()));
}

// int Sequence::getCurrent(){}
static QoreValue Sequence_getCurrent(QoreObject* self, QoreSequence* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 92 "QC_Sequence.qpp"
    return s->getCurrent();
}

// int Sequence::next(){}
static QoreValue Sequence_next(QoreObject* self, QoreSequence* s, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 81 "QC_Sequence.qpp"
    return s->next();
}

DLLLOCAL void preinitSequenceClass() {
    QC_SEQUENCE = new QoreBuiltinClass("Sequence", "::Qore::Thread::Sequence", QDOM_DEFAULT);
    CID_SEQUENCE = QC_SEQUENCE->getID();
    QC_SEQUENCE->setSystem();
}

DLLLOCAL QoreClass* initSequenceClass(QoreNamespace& ns) {
    if (!QC_SEQUENCE)
        preinitSequenceClass();

    // Sequence::constructor() {}
    QC_SEQUENCE->addConstructor(Sequence_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // Sequence::constructor(softint start) {}
    QC_SEQUENCE->addConstructor(Sequence_constructor_vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "start");

    // Sequence::copy() {}
    QC_SEQUENCE->setCopy((q_copy_t)Sequence_copy);

    // int Sequence::getCurrent(){}
    QC_SEQUENCE->addMethod("getCurrent", (q_method_n_t)Sequence_getCurrent, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int Sequence::next(){}
    QC_SEQUENCE->addMethod("next", (q_method_n_t)Sequence_next, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    return QC_SEQUENCE;
}
