/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file SingleValueIterator.qpp SingleValueIterator class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/SingleValueIterator.h"
/* Qore class Qore::SingleValueIterator */

qore_classid_t CID_SINGLEVALUEITERATOR;
QoreClass* QC_SINGLEVALUEITERATOR;

// SingleValueIterator::constructor(auto v) {}
static void SingleValueIterator_constructor_VA(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue v = get_param_value(args, 0);
# 67 "QC_SingleValueIterator.qpp"
self->setPrivate(CID_SINGLEVALUEITERATOR, new SingleValueIterator(v));
}

// SingleValueIterator::copy() {}
static void SingleValueIterator_copy(QoreObject* self, QoreObject* old, SingleValueIterator* i, ExceptionSink* xsink) {
# 77 "QC_SingleValueIterator.qpp"
self->setPrivate(CID_SINGLEVALUEITERATOR, new SingleValueIterator(*i));
}

// auto SingleValueIterator::getValue(){}
static QoreValue SingleValueIterator_getValue(QoreObject* self, SingleValueIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 113 "QC_SingleValueIterator.qpp"
    return i->checkValid(xsink) ? QoreValue() : i->getValue();
}

// bool SingleValueIterator::next(){}
static QoreValue SingleValueIterator_next(QoreObject* self, SingleValueIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 95 "QC_SingleValueIterator.qpp"
    if (i->check(xsink))
      return false;
   return i->next();
}

// nothing SingleValueIterator::reset(){}
static QoreValue SingleValueIterator_reset(QoreObject* self, SingleValueIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 140 "QC_SingleValueIterator.qpp"
    if (i->check(xsink))
       return QoreValue();
   i->reset();
    return QoreValue();
}

// bool SingleValueIterator::valid(){}
static QoreValue SingleValueIterator_valid(QoreObject* self, SingleValueIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 126 "QC_SingleValueIterator.qpp"
    return i->valid();
}

DLLLOCAL void preinitSingleValueIteratorClass() {
    QC_SINGLEVALUEITERATOR = new QoreBuiltinClass("SingleValueIterator", "::Qore::SingleValueIterator", QDOM_DEFAULT);
    CID_SINGLEVALUEITERATOR = QC_SINGLEVALUEITERATOR->getID();
    QC_SINGLEVALUEITERATOR->setSystem();
}

DLLLOCAL QoreClass* initSingleValueIteratorClass(QoreNamespace& ns) {
    if (!QC_SINGLEVALUEITERATOR)
        preinitSingleValueIteratorClass();

    // set parent class
    assert(QC_ABSTRACTITERATOR);
    QC_SINGLEVALUEITERATOR->addBuiltinVirtualBaseClass(QC_ABSTRACTITERATOR);

    // SingleValueIterator::constructor(auto v) {}
    QC_SINGLEVALUEITERATOR->addConstructor(SingleValueIterator_constructor_VA, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "v");

    // SingleValueIterator::copy() {}
    QC_SINGLEVALUEITERATOR->setCopy((q_copy_t)SingleValueIterator_copy);

    // auto SingleValueIterator::getValue(){}
    QC_SINGLEVALUEITERATOR->addMethod("getValue", (q_method_n_t)SingleValueIterator_getValue, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo);

    // bool SingleValueIterator::next(){}
    QC_SINGLEVALUEITERATOR->addMethod("next", (q_method_n_t)SingleValueIterator_next, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // nothing SingleValueIterator::reset(){}
    QC_SINGLEVALUEITERATOR->addMethod("reset", (q_method_n_t)SingleValueIterator_reset, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // bool SingleValueIterator::valid(){}
    QC_SINGLEVALUEITERATOR->addMethod("valid", (q_method_n_t)SingleValueIterator_valid, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    return QC_SINGLEVALUEITERATOR;
}
