/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    QC_SocketPollOperationBase.qpp

    SocketPollOperationBase class definition

    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore/Qore.h"
#include "qore/intern/QC_SocketPollOperationBase.h"
/* Qore class Qore::SocketPollOperationBase */

qore_classid_t CID_SOCKETPOLLOPERATIONBASE;
QoreClass* QC_SOCKETPOLLOPERATIONBASE;

// nothing SocketPollOperationBase::abort(){}
static QoreValue SocketPollOperationBase_abort(QoreObject* self, SocketPollOperationBase* poll, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 102 "QC_SocketPollOperationBase.qpp"
    poll->abort(xsink);
    return QoreValue();
}

// SocketPollOperationBase::constructor() {}
static void SocketPollOperationBase_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 59 "QC_SocketPollOperationBase.qpp"
xsink->raiseException("INTERNAL-ONLY", "the SocketPollOperationBase class is not usable by %Qore classes; "
        "inherit a builtin derived class");
}

// SocketPollOperationBase::destructor() {}
static void SocketPollOperationBase_destructor(QoreObject* self, SocketPollOperationBase* poll, ExceptionSink* xsink) {
# 67 "QC_SocketPollOperationBase.qpp"
poll->deref(xsink);
}

// string SocketPollOperationBase::getGoal(){}
static QoreValue SocketPollOperationBase_getGoal(QoreObject* self, SocketPollOperationBase* poll, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 74 "QC_SocketPollOperationBase.qpp"
    return self->getReferencedMemberNoMethod("goal", xsink);
}

// auto SocketPollOperationBase::getOutput(){}
static QoreValue SocketPollOperationBase_getOutput(QoreObject* self, SocketPollOperationBase* poll, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 95 "QC_SocketPollOperationBase.qpp"
    return poll->getOutput();
}

// string SocketPollOperationBase::getState(){}
static QoreValue SocketPollOperationBase_getState(QoreObject* self, SocketPollOperationBase* poll, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 81 "QC_SocketPollOperationBase.qpp"
    return poll->getState();
}

// bool SocketPollOperationBase::goalReached(){}
static QoreValue SocketPollOperationBase_goalReached(QoreObject* self, SocketPollOperationBase* poll, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 88 "QC_SocketPollOperationBase.qpp"
    return poll->goalReached();
}

DLLLOCAL void preinitSocketPollOperationBaseClass() {
    QC_SOCKETPOLLOPERATIONBASE = new QoreBuiltinClass("SocketPollOperationBase", "::Qore::SocketPollOperationBase", QDOM_DEFAULT);
    CID_SOCKETPOLLOPERATIONBASE = QC_SOCKETPOLLOPERATIONBASE->getID();
    QC_SOCKETPOLLOPERATIONBASE->setSystem();
}

DLLLOCAL QoreClass* initSocketPollOperationBaseClass(QoreNamespace& ns) {
    if (!QC_SOCKETPOLLOPERATIONBASE)
        preinitSocketPollOperationBaseClass();

    // set parent class
    assert(QC_ABSTRACTPOLLOPERATION);
    QC_SOCKETPOLLOPERATIONBASE->addBuiltinVirtualBaseClass(QC_ABSTRACTPOLLOPERATION);

    // private members
    QC_SOCKETPOLLOPERATIONBASE->addMember("sock", Private, objectTypeInfo);
    QC_SOCKETPOLLOPERATIONBASE->addMember("goal", Private, stringTypeInfo);
    QC_SOCKETPOLLOPERATIONBASE->addMember("data_save", Private, autoTypeInfo);

    // nothing SocketPollOperationBase::abort(){}
    QC_SOCKETPOLLOPERATIONBASE->addMethod("abort", (q_method_n_t)SocketPollOperationBase_abort, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // SocketPollOperationBase::constructor() {}
    QC_SOCKETPOLLOPERATIONBASE->addConstructor(SocketPollOperationBase_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // SocketPollOperationBase::destructor() {}
    QC_SOCKETPOLLOPERATIONBASE->setDestructor((q_destructor_t)SocketPollOperationBase_destructor);

    // string SocketPollOperationBase::getGoal(){}
    QC_SOCKETPOLLOPERATIONBASE->addMethod("getGoal", (q_method_n_t)SocketPollOperationBase_getGoal, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);

    // auto SocketPollOperationBase::getOutput(){}
    QC_SOCKETPOLLOPERATIONBASE->addMethod("getOutput", (q_method_n_t)SocketPollOperationBase_getOutput, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo);

    // string SocketPollOperationBase::getState(){}
    QC_SOCKETPOLLOPERATIONBASE->addMethod("getState", (q_method_n_t)SocketPollOperationBase_getState, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // bool SocketPollOperationBase::goalReached(){}
    QC_SOCKETPOLLOPERATIONBASE->addMethod("goalReached", (q_method_n_t)SocketPollOperationBase_goalReached, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    return QC_SOCKETPOLLOPERATIONBASE;
}
