/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_StdoutOutputStream.qpp StdoutOutputStream class definition */
/*
  Qore Programming Language

  Copyright (C) 2016 - 2024 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/StdoutOutputStream.h"
/* Qore class Qore::StdoutOutputStream */

qore_classid_t CID_STDOUTOUTPUTSTREAM;
QoreClass* QC_STDOUTOUTPUTSTREAM;

// nothing StdoutOutputStream::close(){}
static QoreValue StdoutOutputStream_close(QoreObject* self, StdoutOutputStream* os, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 70 "QC_StdoutOutputStream.qpp"
    os->closeHelper(xsink);
    return QoreValue();
}

// StdoutOutputStream::constructor() {}
static void StdoutOutputStream_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 61 "QC_StdoutOutputStream.qpp"
SimpleRefHolder<StdoutOutputStream> stdos(new StdoutOutputStream());
   self->setPrivate(CID_STDOUTOUTPUTSTREAM, stdos.release());
}

// nothing StdoutOutputStream::write(binary data){}
static QoreValue StdoutOutputStream_write_Vy(QoreObject* self, StdoutOutputStream* os, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* data = HARD_QORE_VALUE_BINARY(args, 0);
# 88 "QC_StdoutOutputStream.qpp"
    os->writeHelper(data, xsink);
    return QoreValue();
}

DLLLOCAL void preinitStdoutOutputStreamClass() {
    QC_STDOUTOUTPUTSTREAM = new QoreBuiltinClass("StdoutOutputStream", "::Qore::StdoutOutputStream", QDOM_TERMINAL_IO);
    CID_STDOUTOUTPUTSTREAM = QC_STDOUTOUTPUTSTREAM->getID();
    QC_STDOUTOUTPUTSTREAM->setSystem();
}

DLLLOCAL QoreClass* initStdoutOutputStreamClass(QoreNamespace& ns) {
    if (!QC_STDOUTOUTPUTSTREAM)
        preinitStdoutOutputStreamClass();

    // set parent class
    assert(QC_OUTPUTSTREAM);
    QC_STDOUTOUTPUTSTREAM->addBuiltinVirtualBaseClass(QC_OUTPUTSTREAM);

    QC_STDOUTOUTPUTSTREAM->setFinal();

    // nothing StdoutOutputStream::close(){}
    QC_STDOUTOUTPUTSTREAM->addMethod("close", (q_method_n_t)StdoutOutputStream_close, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // StdoutOutputStream::constructor() {}
    QC_STDOUTOUTPUTSTREAM->addConstructor(StdoutOutputStream_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // nothing StdoutOutputStream::write(binary data){}
    QC_STDOUTOUTPUTSTREAM->addMethod("write", (q_method_n_t)StdoutOutputStream_write_Vy, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, binaryTypeInfo, QORE_PARAM_NO_ARG, "data");

    return QC_STDOUTOUTPUTSTREAM;
}
