/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_StringInputStream.qpp StringInputStream class definition */
/*
  Qore Programming Language

  Copyright (C) 2016 - 2024 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/StringInputStream.h"
/* Qore class Qore::StringInputStream */

qore_classid_t CID_STRINGINPUTSTREAM;
QoreClass* QC_STRINGINPUTSTREAM;

// StringInputStream::constructor(string src) {}
static void StringInputStream_constructor_Vs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* src = HARD_QORE_VALUE_STRING(args, 0);
# 67 "QC_StringInputStream.qpp"
SimpleRefHolder<StringInputStream> sis(new StringInputStream(src));
   self->setPrivate(CID_STRINGINPUTSTREAM, sis.release());
}

// string StringInputStream::getEncoding(){}
static QoreValue StringInputStream_getEncoding(QoreObject* self, StringInputStream* is, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 130 "QC_StringInputStream.qpp"
    return new QoreStringNode(is->getEncoding());
}

// int StringInputStream::peek(){}
static QoreValue StringInputStream_peek(QoreObject* self, StringInputStream* is, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 116 "QC_StringInputStream.qpp"
    return is->peekHelper(xsink);
}

// *binary StringInputStream::read(int limit){}
static QoreValue StringInputStream_read_Vi(QoreObject* self, StringInputStream* is, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 limit = HARD_QORE_VALUE_INT(args, 0);
# 94 "QC_StringInputStream.qpp"
    return is->readHelper(limit, xsink);
}

DLLLOCAL void preinitStringInputStreamClass() {
    QC_STRINGINPUTSTREAM = new QoreBuiltinClass("StringInputStream", "::Qore::StringInputStream", QDOM_DEFAULT);
    CID_STRINGINPUTSTREAM = QC_STRINGINPUTSTREAM->getID();
    QC_STRINGINPUTSTREAM->setSystem();
}

DLLLOCAL QoreClass* initStringInputStreamClass(QoreNamespace& ns) {
    if (!QC_STRINGINPUTSTREAM)
        preinitStringInputStreamClass();

    // set parent class
    assert(QC_INPUTSTREAM);
    QC_STRINGINPUTSTREAM->addBuiltinVirtualBaseClass(QC_INPUTSTREAM);

    QC_STRINGINPUTSTREAM->setFinal();

    // StringInputStream::constructor(string src) {}
    QC_STRINGINPUTSTREAM->addConstructor(StringInputStream_constructor_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "src");

    // string StringInputStream::getEncoding(){}
    QC_STRINGINPUTSTREAM->addMethod("getEncoding", (q_method_n_t)StringInputStream_getEncoding, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // int StringInputStream::peek(){}
    QC_STRINGINPUTSTREAM->addMethod("peek", (q_method_n_t)StringInputStream_peek, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // *binary StringInputStream::read(int limit){}
    QC_STRINGINPUTSTREAM->addMethod("read", (q_method_n_t)StringInputStream_read_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, binaryOrNothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "limit");

    return QC_STRINGINPUTSTREAM;
}
