/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_TransformInputStream.qpp TransformInputStream class definition */
/*
  Qore Programming Language

  Copyright (C) 2016 - 2024 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include "qore/Qore.h"
#include "qore/intern/TransformInputStream.h"
#include "qore/intern/QoreObjectIntern.h"
#include "qore/intern/QoreClassIntern.h"

extern qore_classid_t CID_TRANSFORM;
extern QoreClass* QC_TRANSFORM;
/* Qore class Qore::TransformInputStream */

qore_classid_t CID_TRANSFORMINPUTSTREAM;
QoreClass* QC_TRANSFORMINPUTSTREAM;

// TransformInputStream::constructor(Qore::InputStream is, Qore::Transform t) {}
static void TransformInputStream_constructor_C11InputStreamC9Transform(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(is, InputStream, args, 0, CID_INPUTSTREAM, "TransformInputStream::constructor()", "Qore::InputStream", xsink);
    if (*xsink)
        return;
    HARD_QORE_VALUE_OBJ_DATA(t, Transform, args, 1, CID_TRANSFORM, "TransformInputStream::constructor()", "Qore::Transform", xsink);
    if (*xsink)
        return;
# 61 "QC_TransformInputStream.qpp"
SimpleRefHolder<TransformInputStream> tis(new TransformInputStream(is, t));
   self->setPrivate(CID_TRANSFORMINPUTSTREAM, tis.release());
   qore_object_private* o = qore_object_private::get(*self);
   const qore_class_private* cls = qore_class_private::get(*QC_TRANSFORMINPUTSTREAM);
   o->setValueIntern(cls, "is", static_cast<QoreObject *>(obj_is->refSelf()), xsink);
   o->setValueIntern(cls, "t", static_cast<QoreObject *>(obj_t->refSelf()), xsink);
}

// int TransformInputStream::peek(){}
static QoreValue TransformInputStream_peek(QoreObject* self, TransformInputStream* is, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 94 "QC_TransformInputStream.qpp"
    return is->peekHelper(xsink);
}

// *binary TransformInputStream::read(int limit){}
static QoreValue TransformInputStream_read_Vi(QoreObject* self, TransformInputStream* is, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 limit = HARD_QORE_VALUE_INT(args, 0);
# 79 "QC_TransformInputStream.qpp"
    return is->readHelper(limit, xsink);
}

DLLLOCAL void preinitTransformInputStreamClass() {
    QC_TRANSFORMINPUTSTREAM = new QoreBuiltinClass("TransformInputStream", "::Qore::TransformInputStream", QDOM_DEFAULT);
    CID_TRANSFORMINPUTSTREAM = QC_TRANSFORMINPUTSTREAM->getID();
    QC_TRANSFORMINPUTSTREAM->setSystem();
}

DLLLOCAL QoreClass* initTransformInputStreamClass(QoreNamespace& ns) {
    if (!QC_TRANSFORMINPUTSTREAM)
        preinitTransformInputStreamClass();

    // set parent class
    assert(QC_INPUTSTREAM);
    QC_TRANSFORMINPUTSTREAM->addBuiltinVirtualBaseClass(QC_INPUTSTREAM);

    // private:internal members
    QC_TRANSFORMINPUTSTREAM->addMember("is", Internal, QC_INPUTSTREAM->getTypeInfo());
    QC_TRANSFORMINPUTSTREAM->addMember("t", Internal, QC_TRANSFORM->getTypeInfo());

    QC_TRANSFORMINPUTSTREAM->setFinal();

    // TransformInputStream::constructor(Qore::InputStream is, Qore::Transform t) {}
    QC_TRANSFORMINPUTSTREAM->addConstructor(TransformInputStream_constructor_C11InputStreamC9Transform, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 2, QC_INPUTSTREAM->getTypeInfo(), QORE_PARAM_NO_ARG, "is", QC_TRANSFORM->getTypeInfo(), QORE_PARAM_NO_ARG, "t");

    // int TransformInputStream::peek(){}
    QC_TRANSFORMINPUTSTREAM->addMethod("peek", (q_method_n_t)TransformInputStream_peek, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // *binary TransformInputStream::read(int limit){}
    QC_TRANSFORMINPUTSTREAM->addMethod("read", (q_method_n_t)TransformInputStream_read_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, binaryOrNothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "limit");

    return QC_TRANSFORMINPUTSTREAM;
}
