/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_TransformOutputStream.qpp TransformOutputStream class definition */
/*
    Qore Programming Language

    Copyright (C) 2016 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore/Qore.h"
#include "qore/intern/TransformOutputStream.h"
#include "qore/intern/QoreObjectIntern.h"
#include "qore/intern/QoreClassIntern.h"

extern qore_classid_t CID_TRANSFORM;
extern QoreClass* QC_TRANSFORM;
/* Qore class Qore::TransformOutputStream */

qore_classid_t CID_TRANSFORMOUTPUTSTREAM;
QoreClass* QC_TRANSFORMOUTPUTSTREAM;

// nothing TransformOutputStream::close(){}
static QoreValue TransformOutputStream_close(QoreObject* self, TransformOutputStream* os, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 79 "QC_TransformOutputStream.qpp"
    os->closeHelper(xsink);
    return QoreValue();
}

// TransformOutputStream::constructor(Qore::OutputStream os, Qore::Transform t) {}
static void TransformOutputStream_constructor_C12OutputStreamC9Transform(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(os, OutputStream, args, 0, CID_OUTPUTSTREAM, "TransformOutputStream::constructor()", "Qore::OutputStream", xsink);
    if (*xsink)
        return;
    HARD_QORE_VALUE_OBJ_DATA(t, Transform, args, 1, CID_TRANSFORM, "TransformOutputStream::constructor()", "Qore::Transform", xsink);
    if (*xsink)
        return;
# 62 "QC_TransformOutputStream.qpp"
// here we pass the references to os and t to "tos"
    SimpleRefHolder<TransformOutputStream> tos(new TransformOutputStream(os, t));
    self->setPrivate(CID_TRANSFORMOUTPUTSTREAM, tos.release());
    qore_object_private* o = qore_object_private::get(*self);
    const qore_class_private* cls = qore_class_private::get(*QC_TRANSFORMOUTPUTSTREAM);
    o->setValueIntern(cls, "os", static_cast<QoreObject *>(obj_os->refSelf()), xsink);
    o->setValueIntern(cls, "t", static_cast<QoreObject *>(obj_t->refSelf()), xsink);
}

// nothing TransformOutputStream::write(binary data){}
static QoreValue TransformOutputStream_write_Vy(QoreObject* self, TransformOutputStream* os, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* data = HARD_QORE_VALUE_BINARY(args, 0);
# 91 "QC_TransformOutputStream.qpp"
    os->writeHelper(data, xsink);
    return QoreValue();
}

DLLLOCAL void preinitTransformOutputStreamClass() {
    QC_TRANSFORMOUTPUTSTREAM = new QoreBuiltinClass("TransformOutputStream", "::Qore::TransformOutputStream", QDOM_DEFAULT);
    CID_TRANSFORMOUTPUTSTREAM = QC_TRANSFORMOUTPUTSTREAM->getID();
    QC_TRANSFORMOUTPUTSTREAM->setSystem();
}

DLLLOCAL QoreClass* initTransformOutputStreamClass(QoreNamespace& ns) {
    if (!QC_TRANSFORMOUTPUTSTREAM)
        preinitTransformOutputStreamClass();

    // set parent class
    assert(QC_OUTPUTSTREAM);
    QC_TRANSFORMOUTPUTSTREAM->addBuiltinVirtualBaseClass(QC_OUTPUTSTREAM);

    // private:internal members
    QC_TRANSFORMOUTPUTSTREAM->addMember("os", Internal, QC_OUTPUTSTREAM->getTypeInfo());
    QC_TRANSFORMOUTPUTSTREAM->addMember("t", Internal, QC_TRANSFORM->getTypeInfo());

    QC_TRANSFORMOUTPUTSTREAM->setFinal();

    // nothing TransformOutputStream::close(){}
    QC_TRANSFORMOUTPUTSTREAM->addMethod("close", (q_method_n_t)TransformOutputStream_close, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // TransformOutputStream::constructor(Qore::OutputStream os, Qore::Transform t) {}
    QC_TRANSFORMOUTPUTSTREAM->addConstructor(TransformOutputStream_constructor_C12OutputStreamC9Transform, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 2, QC_OUTPUTSTREAM->getTypeInfo(), QORE_PARAM_NO_ARG, "os", QC_TRANSFORM->getTypeInfo(), QORE_PARAM_NO_ARG, "t");

    // nothing TransformOutputStream::write(binary data){}
    QC_TRANSFORMOUTPUTSTREAM->addMethod("write", (q_method_n_t)TransformOutputStream_write_Vy, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, binaryTypeInfo, QORE_PARAM_NO_ARG, "data");

    return QC_TRANSFORMOUTPUTSTREAM;
}
