/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    ql_list.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/ql_list.h"
#include "qore/intern/qore_program_private.h"

ResolvedCallReferenceNode* getCallReference(const QoreString* str, ExceptionSink* xsink) {
   // ensure string is in default encoding
   TempEncodingHelper tmp(str, QCS_DEFAULT, xsink);
   if (!tmp)
      return 0;
   return qore_program_private::runtimeGetCallReference(getProgram(), tmp->getBuffer(), xsink);
}

bool inlist_intern(const QoreValue arg, const QoreListNode* l, ExceptionSink* xsink) {
    assert(xsink);
    ConstListIterator li(l);
    while (li.next()) {
        bool b = QoreLogicalEqualsOperatorNode::softEqual(arg, li.getValue(), xsink);
        if (*xsink) {
            return false;
        }
        if (b) {
            return true;
        }
    }
    return false;
}

QoreListNode* range_intern(int64 start, int64 stop, int64 step, ExceptionSink* xsink) {
    if (step < 1) {
        xsink->raiseException("RANGE-ERROR", "Value of the 'step' argument has to be greater than 0");
        return 0;
    }

    bool broken_range = runtime_get_parse_options() & PO_BROKEN_RANGE;
    QoreListNode* l = new QoreListNode(bigIntTypeInfo);
    if (start < stop) {
        if (broken_range) {
            ++stop;
        }

        for (int64 i = start; i < stop; i += step) {
            l->push(i, xsink);
        }
    } else {
        if (broken_range) {
            --stop;
        }

        for (int64 i = start; i > stop; i -= step) {
            l->push(i, xsink);
        }
    }
    return l;
}
# 88 "ql_list.qpp"
# 100 "ql_list.qpp"
# 125 "ql_list.qpp"
# 154 "ql_list.qpp"
# 184 "ql_list.qpp"
# 192 "ql_list.qpp"
# 214 "ql_list.qpp"
# 240 "ql_list.qpp"
# 266 "ql_list.qpp"
# 274 "ql_list.qpp"
# 301 "ql_list.qpp"
# 332 "ql_list.qpp"
# 364 "ql_list.qpp"
# 372 "ql_list.qpp"
# 390 "ql_list.qpp"
# 412 "ql_list.qpp"
# 440 "ql_list.qpp"
# 448 "ql_list.qpp"
# 471 "ql_list.qpp"
# 498 "ql_list.qpp"
# 526 "ql_list.qpp"
# 534 "ql_list.qpp"
# 557 "ql_list.qpp"
# 584 "ql_list.qpp"
# 612 "ql_list.qpp"
# 620 "ql_list.qpp"
# 643 "ql_list.qpp"
# 670 "ql_list.qpp"
# 698 "ql_list.qpp"
# 716 "ql_list.qpp"
# 738 "ql_list.qpp"
# 762 "ql_list.qpp"
# 780 "ql_list.qpp"
# 798 "ql_list.qpp"
# 820 "ql_list.qpp"
# 844 "ql_list.qpp"
# 862 "ql_list.qpp"
# 869 "ql_list.qpp"
# 886 "ql_list.qpp"
# 894 "ql_list.qpp"
# 902 "ql_list.qpp"
# 922 "ql_list.qpp"
# 930 "ql_list.qpp"
# 938 "ql_list.qpp"
# 982 "ql_list.qpp"
# 1016 "ql_list.qpp"
// bool inlist() {}
static QoreValue f_inlist(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 890 "ql_list.qpp"
   return false;
}

// bool inlist(any arg, nothing x) {}
static QoreValue f_inlist_VaVn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 898 "ql_list.qpp"
   return false;
}

// bool inlist(auto arg, softlist<auto> l) {}
static QoreValue f_inlist_VAC14softlist_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 1);
# 918 "ql_list.qpp"
   return inlist_intern(arg, l, xsink);
}

// bool inlist_hard() {}
static QoreValue f_inlist_hard(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 926 "ql_list.qpp"
   return false;
}

// bool inlist_hard(any arg, nothing x) {}
static QoreValue f_inlist_hard_VaVn(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 934 "ql_list.qpp"
   return false;
}

// bool inlist_hard(auto arg, softlist<auto> l) {}
static QoreValue f_inlist_hard_VAC14softlist_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 1);
# 954 "ql_list.qpp"
    bool arg_is_nothing = arg.isNothing();

    ConstListIterator li(l);
    while (li.next()) {
        QoreValue lp = li.getValue();

        bool b;

        // do hard comparison inline
        if (lp.isNothing()) {
            b = arg_is_nothing;
        } else if (arg_is_nothing) {
            b = false;
        } else {
            b = arg.isEqualHard(lp);
            if (*xsink) {
                return QoreValue();
            }
        }

        if (b) {
            return true;
        }
    }
    return false;
}

// auto max(list<auto> l) {}
static QoreValue f_max_C10list_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 794 "ql_list.qpp"
   return l->max(xsink);
}

// auto max(list<auto> l, string func) {}
static QoreValue f_max_C10list_auto_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const QoreStringNode* func = HARD_QORE_VALUE_STRING(args, 1);
# 815 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? QoreValue() : l->max(*fr, xsink);
}

// auto max(list<auto> l, code f) {}
static QoreValue f_max_C10list_auto_Vc(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const ResolvedCallReferenceNode* f = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
# 840 "ql_list.qpp"
   return l->max(f, xsink);
}

// auto max(...) {}
static QoreValue f_max_VV(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 858 "ql_list.qpp"
   return args ? args->max(xsink) : QoreValue();
}

// auto min(list<auto> l) {}
static QoreValue f_min_C10list_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 712 "ql_list.qpp"
   return l->min(xsink);
}

// auto min(list<auto> l, string func) {}
static QoreValue f_min_C10list_auto_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const QoreStringNode* func = HARD_QORE_VALUE_STRING(args, 1);
# 733 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? QoreValue() : l->min(*fr, xsink);
}

// auto min(list<auto> l, code f) {}
static QoreValue f_min_C10list_auto_Vc(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const ResolvedCallReferenceNode* f = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
# 758 "ql_list.qpp"
   return l->min(f, xsink);
}

// auto min(...) {}
static QoreValue f_min_VV(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 776 "ql_list.qpp"
   return args ? args->min(xsink) : QoreValue();
}

// list<int> range(int start, int stop, int step = 1) {}
static QoreValue f_range_ViViVi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 start = HARD_QORE_VALUE_INT(args, 0);
    int64 stop = HARD_QORE_VALUE_INT(args, 1);
    int64 step = HARD_QORE_VALUE_INT(args, 2);
# 1012 "ql_list.qpp"
    return range_intern(start, stop, step, xsink);
}

// list<int> range(int stop) {}
static QoreValue f_range_Vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 stop = HARD_QORE_VALUE_INT(args, 0);
# 1039 "ql_list.qpp"
    return range_intern(0, stop, 1, xsink);
}

// nothing reverse() {}
static QoreValue f_reverse(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 866 "ql_list.qpp"

    return QoreValue();
}

// list<auto> reverse(list<auto> l) {}
static QoreValue f_reverse_C10list_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 882 "ql_list.qpp"
   return l->reverse();
}

// auto sort(auto arg) {}
static QoreValue f_sort_VA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
# 96 "ql_list.qpp"
   return arg.refSelf();
}

// list<auto> sort(list<auto> l) {}
static QoreValue f_sort_C10list_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 121 "ql_list.qpp"
   return l->sort(xsink);
}

// list<auto> sort(list<auto> l, string func) {}
static QoreValue f_sort_C10list_auto_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const QoreStringNode* func = HARD_QORE_VALUE_STRING(args, 1);
# 149 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? QoreValue() : l->sort(*fr, xsink);
}

// list<auto> sort(list<auto> l, code f) {}
static QoreValue f_sort_C10list_auto_Vc(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const ResolvedCallReferenceNode* f = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
# 180 "ql_list.qpp"
   return l->sort(f, xsink);
}

// auto sortDescending(auto arg) {}
static QoreValue f_sortDescending_VA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
# 188 "ql_list.qpp"
   return arg.refSelf();
}

// list<auto> sortDescending(list<auto> l) {}
static QoreValue f_sortDescending_C10list_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 210 "ql_list.qpp"
   return l->sortDescending(xsink);
}

// list<auto> sortDescending(list<auto> l, string func) {}
static QoreValue f_sortDescending_C10list_auto_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const QoreStringNode* func = HARD_QORE_VALUE_STRING(args, 1);
# 235 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? QoreValue() : l->sortDescending(*fr, xsink);
}

// list<auto> sortDescending(list<auto> l, code f) {}
static QoreValue f_sortDescending_C10list_auto_Vc(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const ResolvedCallReferenceNode* f = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
# 262 "ql_list.qpp"
   return l->sortDescending(f, xsink);
}

// auto sortDescendingStable(auto arg) {}
static QoreValue f_sortDescendingStable_VA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
# 530 "ql_list.qpp"
   return arg.refSelf();
}

// list<auto> sortDescendingStable(list<auto> l) {}
static QoreValue f_sortDescendingStable_C10list_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 553 "ql_list.qpp"
   return l->sortDescendingStable(xsink);
}

// list<auto> sortDescendingStable(list<auto> l, string func) {}
static QoreValue f_sortDescendingStable_C10list_auto_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const QoreStringNode* func = HARD_QORE_VALUE_STRING(args, 1);
# 579 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? QoreValue() : l->sortDescendingStable(*fr, xsink);
}

// list<auto> sortDescendingStable(list<auto> l, code f) {}
static QoreValue f_sortDescendingStable_C10list_auto_Vc(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const ResolvedCallReferenceNode* f = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
# 608 "ql_list.qpp"
   return l->sortDescendingStable(f, xsink);
}

// auto sortStable(auto arg) {}
static QoreValue f_sortStable_VA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
# 368 "ql_list.qpp"
   return arg.refSelf();
}

// list<auto> sortStable(list<auto> l) {}
static QoreValue f_sortStable_C10list_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 386 "ql_list.qpp"
   return l->sortStable(xsink);
}

// list<auto> sortStable(list<auto> l, string func) {}
static QoreValue f_sortStable_C10list_auto_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const QoreStringNode* func = HARD_QORE_VALUE_STRING(args, 1);
# 407 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? QoreValue() : l->sortStable(*fr, xsink);
}

// list<auto> sortStable(list<auto> l, code f) {}
static QoreValue f_sortStable_C10list_auto_Vc(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const ResolvedCallReferenceNode* f = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
# 436 "ql_list.qpp"
   return l->sortStable(f, xsink);
}

// auto sort_descending(auto arg) {}
static QoreValue f_sort_descending_VA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
# 270 "ql_list.qpp"
   return arg.refSelf();
}

// list<auto> sort_descending(list<auto> l) {}
static QoreValue f_sort_descending_C10list_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 297 "ql_list.qpp"
   return l->sortDescending(xsink);
}

// list<auto> sort_descending(list<auto> l, string func) {}
static QoreValue f_sort_descending_C10list_auto_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const QoreStringNode* func = HARD_QORE_VALUE_STRING(args, 1);
# 327 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? QoreValue() : l->sortDescending(*fr, xsink);
}

// list<auto> sort_descending(list<auto> l, code f) {}
static QoreValue f_sort_descending_C10list_auto_Vc(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const ResolvedCallReferenceNode* f = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
# 360 "ql_list.qpp"
   return l->sortDescending(f, xsink);
}

// auto sort_descending_stable(auto arg) {}
static QoreValue f_sort_descending_stable_VA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
# 616 "ql_list.qpp"
   return arg.refSelf();
}

// list<auto> sort_descending_stable(list<auto> l) {}
static QoreValue f_sort_descending_stable_C10list_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 639 "ql_list.qpp"
   return l->sortDescendingStable(xsink);
}

// list<auto> sort_descending_stable(list<auto> l, string func) {}
static QoreValue f_sort_descending_stable_C10list_auto_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const QoreStringNode* func = HARD_QORE_VALUE_STRING(args, 1);
# 665 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? QoreValue() : l->sortDescendingStable(*fr, xsink);
}

// list<auto> sort_descending_stable(list<auto> l, code f) {}
static QoreValue f_sort_descending_stable_C10list_auto_Vc(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const ResolvedCallReferenceNode* f = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
# 694 "ql_list.qpp"
   return l->sortDescendingStable(f, xsink);
}

// auto sort_stable(auto arg) {}
static QoreValue f_sort_stable_VA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
# 444 "ql_list.qpp"
   return arg.refSelf();
}

// list<auto> sort_stable(list<auto> l) {}
static QoreValue f_sort_stable_C10list_auto_(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 467 "ql_list.qpp"
   return l->sortStable(xsink);
}

// list<auto> sort_stable(list<auto> l, string func) {}
static QoreValue f_sort_stable_C10list_auto_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const QoreStringNode* func = HARD_QORE_VALUE_STRING(args, 1);
# 493 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? QoreValue() : l->sortStable(*fr, xsink);
}

// list<auto> sort_stable(list<auto> l, code f) {}
static QoreValue f_sort_stable_C10list_auto_Vc(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const ResolvedCallReferenceNode* f = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
# 522 "ql_list.qpp"
   return l->sortStable(f, xsink);
}


DLLLOCAL void init_list_functions(QoreNamespace& ns) {
    // bool inlist() {}
    ns.addBuiltinVariant("inlist", (q_func_n_t)f_inlist, QCF_NOOP, QDOM_DEFAULT, boolTypeInfo);
    // bool inlist(any arg, nothing x) {}
    ns.addBuiltinVariant("inlist", (q_func_n_t)f_inlist_VaVn, QCF_NOOP, QDOM_DEFAULT, boolTypeInfo, 2, anyTypeInfo, QORE_PARAM_NO_ARG, "arg", nothingTypeInfo, QORE_PARAM_NO_ARG, "x");
    // bool inlist(auto arg, softlist<auto> l) {}
    ns.addBuiltinVariant("inlist", (q_func_n_t)f_inlist_VAC14softlist_auto_, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 2, autoTypeInfo, QORE_PARAM_NO_ARG, "arg", softAutoListTypeInfo, QORE_PARAM_NO_ARG, "l");
    // bool inlist_hard() {}
    ns.addBuiltinVariant("inlist_hard", (q_func_n_t)f_inlist_hard, QCF_NOOP, QDOM_DEFAULT, boolTypeInfo);
    // bool inlist_hard(any arg, nothing x) {}
    ns.addBuiltinVariant("inlist_hard", (q_func_n_t)f_inlist_hard_VaVn, QCF_NOOP, QDOM_DEFAULT, boolTypeInfo, 2, anyTypeInfo, QORE_PARAM_NO_ARG, "arg", nothingTypeInfo, QORE_PARAM_NO_ARG, "x");
    // bool inlist_hard(auto arg, softlist<auto> l) {}
    ns.addBuiltinVariant("inlist_hard", (q_func_n_t)f_inlist_hard_VAC14softlist_auto_, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 2, autoTypeInfo, QORE_PARAM_NO_ARG, "arg", softAutoListTypeInfo, QORE_PARAM_NO_ARG, "l");
    // auto max(list<auto> l) {}
    ns.addBuiltinVariant("max", (q_func_n_t)f_max_C10list_auto_, QCF_CONSTANT, QDOM_DEFAULT, autoTypeInfo, 1, autoListTypeInfo, QORE_PARAM_NO_ARG, "l");
    // auto max(list<auto> l, string func) {}
    ns.addBuiltinVariant("max", (q_func_n_t)f_max_C10list_auto_Vs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 2, autoListTypeInfo, QORE_PARAM_NO_ARG, "l", stringTypeInfo, QORE_PARAM_NO_ARG, "func");
    // auto max(list<auto> l, code f) {}
    ns.addBuiltinVariant("max", (q_func_n_t)f_max_C10list_auto_Vc, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 2, autoListTypeInfo, QORE_PARAM_NO_ARG, "l", codeTypeInfo, QORE_PARAM_NO_ARG, "f");
    // auto max(...) {}
    ns.addBuiltinVariant("max", (q_func_n_t)f_max_VV, QCF_CONSTANT|QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo);
    // auto min(list<auto> l) {}
    ns.addBuiltinVariant("min", (q_func_n_t)f_min_C10list_auto_, QCF_CONSTANT, QDOM_DEFAULT, autoTypeInfo, 1, autoListTypeInfo, QORE_PARAM_NO_ARG, "l");
    // auto min(list<auto> l, string func) {}
    ns.addBuiltinVariant("min", (q_func_n_t)f_min_C10list_auto_Vs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 2, autoListTypeInfo, QORE_PARAM_NO_ARG, "l", stringTypeInfo, QORE_PARAM_NO_ARG, "func");
    // auto min(list<auto> l, code f) {}
    ns.addBuiltinVariant("min", (q_func_n_t)f_min_C10list_auto_Vc, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 2, autoListTypeInfo, QORE_PARAM_NO_ARG, "l", codeTypeInfo, QORE_PARAM_NO_ARG, "f");
    // auto min(...) {}
    ns.addBuiltinVariant("min", (q_func_n_t)f_min_VV, QCF_CONSTANT|QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo);
    // list<int> range(int start, int stop, int step = 1) {}
    ns.addBuiltinVariant("range", (q_func_n_t)f_range_ViViVi, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(bigIntTypeInfo), 3, bigIntTypeInfo, QORE_PARAM_NO_ARG, "start", bigIntTypeInfo, QORE_PARAM_NO_ARG, "stop", bigIntTypeInfo, QoreSimpleValue().assign((int64)1), "step");
    // list<int> range(int stop) {}
    ns.addBuiltinVariant("range", (q_func_n_t)f_range_Vi, QCF_CONSTANT, QDOM_DEFAULT, qore_get_complex_list_type(bigIntTypeInfo), 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "stop");
    // nothing reverse() {}
    ns.addBuiltinVariant("reverse", (q_func_n_t)f_reverse, QCF_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // list<auto> reverse(list<auto> l) {}
    ns.addBuiltinVariant("reverse", (q_func_n_t)f_reverse_C10list_auto_, QCF_CONSTANT, QDOM_DEFAULT, autoListTypeInfo, 1, autoListTypeInfo, QORE_PARAM_NO_ARG, "l");
    // auto sort(auto arg) {}
    ns.addBuiltinVariant("sort", (q_func_n_t)f_sort_VA, QCF_NOOP, QDOM_DEFAULT, autoTypeInfo, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "arg");
    // list<auto> sort(list<auto> l) {}
    ns.addBuiltinVariant("sort", (q_func_n_t)f_sort_C10list_auto_, QCF_CONSTANT, QDOM_DEFAULT, autoListTypeInfo, 1, autoListTypeInfo, QORE_PARAM_NO_ARG, "l");
    // list<auto> sort(list<auto> l, string func) {}
    ns.addBuiltinVariant("sort", (q_func_n_t)f_sort_C10list_auto_Vs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoListTypeInfo, 2, autoListTypeInfo, QORE_PARAM_NO_ARG, "l", stringTypeInfo, QORE_PARAM_NO_ARG, "func");
    // list<auto> sort(list<auto> l, code f) {}
    ns.addBuiltinVariant("sort", (q_func_n_t)f_sort_C10list_auto_Vc, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoListTypeInfo, 2, autoListTypeInfo, QORE_PARAM_NO_ARG, "l", codeTypeInfo, QORE_PARAM_NO_ARG, "f");
    // auto sortDescending(auto arg) {}
    ns.addBuiltinVariant("sortDescending", (q_func_n_t)f_sortDescending_VA, QCF_NOOP|QCF_DEPRECATED, QDOM_DEFAULT, autoTypeInfo, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "arg");
    // list<auto> sortDescending(list<auto> l) {}
    ns.addBuiltinVariant("sortDescending", (q_func_n_t)f_sortDescending_C10list_auto_, QCF_CONSTANT|QCF_DEPRECATED, QDOM_DEFAULT, autoListTypeInfo, 1, autoListTypeInfo, QORE_PARAM_NO_ARG, "l");
    // list<auto> sortDescending(list<auto> l, string func) {}
    ns.addBuiltinVariant("sortDescending", (q_func_n_t)f_sortDescending_C10list_auto_Vs, QCF_CONSTANT|QCF_DEPRECATED, QDOM_DEFAULT, autoListTypeInfo, 2, autoListTypeInfo, QORE_PARAM_NO_ARG, "l", stringTypeInfo, QORE_PARAM_NO_ARG, "func");
    // list<auto> sortDescending(list<auto> l, code f) {}
    ns.addBuiltinVariant("sortDescending", (q_func_n_t)f_sortDescending_C10list_auto_Vc, QCF_RET_VALUE_ONLY|QCF_DEPRECATED, QDOM_DEFAULT, autoListTypeInfo, 2, autoListTypeInfo, QORE_PARAM_NO_ARG, "l", codeTypeInfo, QORE_PARAM_NO_ARG, "f");
    // auto sortDescendingStable(auto arg) {}
    ns.addBuiltinVariant("sortDescendingStable", (q_func_n_t)f_sortDescendingStable_VA, QCF_NOOP|QCF_DEPRECATED, QDOM_DEFAULT, autoTypeInfo, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "arg");
    // list<auto> sortDescendingStable(list<auto> l) {}
    ns.addBuiltinVariant("sortDescendingStable", (q_func_n_t)f_sortDescendingStable_C10list_auto_, QCF_CONSTANT|QCF_DEPRECATED, QDOM_DEFAULT, autoListTypeInfo, 1, autoListTypeInfo, QORE_PARAM_NO_ARG, "l");
    // list<auto> sortDescendingStable(list<auto> l, string func) {}
    ns.addBuiltinVariant("sortDescendingStable", (q_func_n_t)f_sortDescendingStable_C10list_auto_Vs, QCF_RET_VALUE_ONLY|QCF_DEPRECATED, QDOM_DEFAULT, autoListTypeInfo, 2, autoListTypeInfo, QORE_PARAM_NO_ARG, "l", stringTypeInfo, QORE_PARAM_NO_ARG, "func");
    // list<auto> sortDescendingStable(list<auto> l, code f) {}
    ns.addBuiltinVariant("sortDescendingStable", (q_func_n_t)f_sortDescendingStable_C10list_auto_Vc, QCF_RET_VALUE_ONLY|QCF_DEPRECATED, QDOM_DEFAULT, autoListTypeInfo, 2, autoListTypeInfo, QORE_PARAM_NO_ARG, "l", codeTypeInfo, QORE_PARAM_NO_ARG, "f");
    // auto sortStable(auto arg) {}
    ns.addBuiltinVariant("sortStable", (q_func_n_t)f_sortStable_VA, QCF_NOOP|QCF_DEPRECATED, QDOM_DEFAULT, autoTypeInfo, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "arg");
    // list<auto> sortStable(list<auto> l) {}
    ns.addBuiltinVariant("sortStable", (q_func_n_t)f_sortStable_C10list_auto_, QCF_CONSTANT|QCF_DEPRECATED, QDOM_DEFAULT, autoListTypeInfo, 1, autoListTypeInfo, QORE_PARAM_NO_ARG, "l");
    // list<auto> sortStable(list<auto> l, string func) {}
    ns.addBuiltinVariant("sortStable", (q_func_n_t)f_sortStable_C10list_auto_Vs, QCF_RET_VALUE_ONLY|QCF_DEPRECATED, QDOM_DEFAULT, autoListTypeInfo, 2, autoListTypeInfo, QORE_PARAM_NO_ARG, "l", stringTypeInfo, QORE_PARAM_NO_ARG, "func");
    // list<auto> sortStable(list<auto> l, code f) {}
    ns.addBuiltinVariant("sortStable", (q_func_n_t)f_sortStable_C10list_auto_Vc, QCF_RET_VALUE_ONLY|QCF_DEPRECATED, QDOM_DEFAULT, autoListTypeInfo, 2, autoListTypeInfo, QORE_PARAM_NO_ARG, "l", codeTypeInfo, QORE_PARAM_NO_ARG, "f");
    // auto sort_descending(auto arg) {}
    ns.addBuiltinVariant("sort_descending", (q_func_n_t)f_sort_descending_VA, QCF_NOOP, QDOM_DEFAULT, autoTypeInfo, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "arg");
    // list<auto> sort_descending(list<auto> l) {}
    ns.addBuiltinVariant("sort_descending", (q_func_n_t)f_sort_descending_C10list_auto_, QCF_CONSTANT, QDOM_DEFAULT, autoListTypeInfo, 1, autoListTypeInfo, QORE_PARAM_NO_ARG, "l");
    // list<auto> sort_descending(list<auto> l, string func) {}
    ns.addBuiltinVariant("sort_descending", (q_func_n_t)f_sort_descending_C10list_auto_Vs, QCF_CONSTANT, QDOM_DEFAULT, autoListTypeInfo, 2, autoListTypeInfo, QORE_PARAM_NO_ARG, "l", stringTypeInfo, QORE_PARAM_NO_ARG, "func");
    // list<auto> sort_descending(list<auto> l, code f) {}
    ns.addBuiltinVariant("sort_descending", (q_func_n_t)f_sort_descending_C10list_auto_Vc, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoListTypeInfo, 2, autoListTypeInfo, QORE_PARAM_NO_ARG, "l", codeTypeInfo, QORE_PARAM_NO_ARG, "f");
    // auto sort_descending_stable(auto arg) {}
    ns.addBuiltinVariant("sort_descending_stable", (q_func_n_t)f_sort_descending_stable_VA, QCF_NOOP, QDOM_DEFAULT, autoTypeInfo, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "arg");
    // list<auto> sort_descending_stable(list<auto> l) {}
    ns.addBuiltinVariant("sort_descending_stable", (q_func_n_t)f_sort_descending_stable_C10list_auto_, QCF_CONSTANT, QDOM_DEFAULT, autoListTypeInfo, 1, autoListTypeInfo, QORE_PARAM_NO_ARG, "l");
    // list<auto> sort_descending_stable(list<auto> l, string func) {}
    ns.addBuiltinVariant("sort_descending_stable", (q_func_n_t)f_sort_descending_stable_C10list_auto_Vs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoListTypeInfo, 2, autoListTypeInfo, QORE_PARAM_NO_ARG, "l", stringTypeInfo, QORE_PARAM_NO_ARG, "func");
    // list<auto> sort_descending_stable(list<auto> l, code f) {}
    ns.addBuiltinVariant("sort_descending_stable", (q_func_n_t)f_sort_descending_stable_C10list_auto_Vc, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoListTypeInfo, 2, autoListTypeInfo, QORE_PARAM_NO_ARG, "l", codeTypeInfo, QORE_PARAM_NO_ARG, "f");
    // auto sort_stable(auto arg) {}
    ns.addBuiltinVariant("sort_stable", (q_func_n_t)f_sort_stable_VA, QCF_NOOP, QDOM_DEFAULT, autoTypeInfo, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "arg");
    // list<auto> sort_stable(list<auto> l) {}
    ns.addBuiltinVariant("sort_stable", (q_func_n_t)f_sort_stable_C10list_auto_, QCF_CONSTANT, QDOM_DEFAULT, autoListTypeInfo, 1, autoListTypeInfo, QORE_PARAM_NO_ARG, "l");
    // list<auto> sort_stable(list<auto> l, string func) {}
    ns.addBuiltinVariant("sort_stable", (q_func_n_t)f_sort_stable_C10list_auto_Vs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoListTypeInfo, 2, autoListTypeInfo, QORE_PARAM_NO_ARG, "l", stringTypeInfo, QORE_PARAM_NO_ARG, "func");
    // list<auto> sort_stable(list<auto> l, code f) {}
    ns.addBuiltinVariant("sort_stable", (q_func_n_t)f_sort_stable_C10list_auto_Vc, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoListTypeInfo, 2, autoListTypeInfo, QORE_PARAM_NO_ARG, "l", codeTypeInfo, QORE_PARAM_NO_ARG, "f");

}
