/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_LoggerAppender.cpp LoggerAppender class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_logger.h"

#include "QC_LoggerAppender.h"
#include "QC_LoggerAppenderQueue.h"
#include "QC_LoggerFilter.h"
/* Qore class Qore::Logger::LoggerAppender */

qore_classid_t CID_LOGGERAPPENDER;
QoreClass* QC_LOGGERAPPENDER;

// nothing LoggerAppender::addFilter(LoggerFilter filter, bool top = False){}
static QoreValue LoggerAppender_addFilter_C12LoggerFilterVb(QoreObject* self, QoreLoggerAppender* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(filter, QoreLoggerFilter, args, 0, CID_LOGGERFILTER, "LoggerAppender::addFilter()", "LoggerFilter", xsink);
    if (*xsink)
        return 0;
    bool top = HARD_QORE_VALUE_BOOL(args, 1);
# 116 "QC_LoggerAppender.qpp"
    ReferenceHolder<QoreLoggerFilter> holder(filter, xsink);
    a->addFilter(xsink, obj_filter, top);
    return QoreValue();
}

// nothing LoggerAppender::close(){}
static QoreValue LoggerAppender_close(QoreObject* self, QoreLoggerAppender* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 98 "QC_LoggerAppender.qpp"
    a->close(xsink);
    return QoreValue();
}

// LoggerAppender::constructor() {}
static void LoggerAppender_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 46 "QC_LoggerAppender.qpp"
self->setPrivate(CID_LOGGERAPPENDER, new QoreLoggerAppender(xsink, self));
}

// LoggerAppender::constructor(string name) {}
static void LoggerAppender_constructor_Vs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 54 "QC_LoggerAppender.qpp"
self->setPrivate(CID_LOGGERAPPENDER, new QoreLoggerAppender(xsink, self, name));
}

// private *object LoggerAppender::ensureAtomicOperations(int type){}
static QoreValue LoggerAppender_ensureAtomicOperations_Vi(QoreObject* self, QoreLoggerAppender* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 200 "QC_LoggerAppender.qpp"
    // empty body
    return QoreValue();
}

// list<LoggerFilter> LoggerAppender::getFilters(){}
static QoreValue LoggerAppender_getFilters(QoreObject* self, QoreLoggerAppender* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 141 "QC_LoggerAppender.qpp"
    return a->getFilters();
}

// string LoggerAppender::getName(){}
static QoreValue LoggerAppender_getName(QoreObject* self, QoreLoggerAppender* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 61 "QC_LoggerAppender.qpp"
    return a->getName();
}

// *LoggerAppenderQueue LoggerAppender::getQueue(){}
static QoreValue LoggerAppender_getQueue(QoreObject* self, QoreLoggerAppender* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 83 "QC_LoggerAppender.qpp"
    return a->getQueue();
}

// bool LoggerAppender::isOpen(){}
static QoreValue LoggerAppender_isOpen(QoreObject* self, QoreLoggerAppender* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 105 "QC_LoggerAppender.qpp"
    return a->isOpen();
}

// nothing LoggerAppender::open(){}
static QoreValue LoggerAppender_open(QoreObject* self, QoreLoggerAppender* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 91 "QC_LoggerAppender.qpp"
    a->open(xsink);
    return QoreValue();
}

// bool LoggerAppender::post(LoggerEvent event){}
static QoreValue LoggerAppender_post_C11LoggerEvent(QoreObject* self, QoreLoggerAppender* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(event, QoreLoggerEvent, args, 0, CID_LOGGEREVENT, "LoggerAppender::post()", "LoggerEvent", xsink);
    if (*xsink)
        return 0;
# 170 "QC_LoggerAppender.qpp"
    ReferenceHolder<QoreLoggerEvent> holder(event, xsink);
    return a->post(xsink, obj_event);
}

// nothing LoggerAppender::processEvent(int type, auto params){}
static QoreValue LoggerAppender_processEvent_ViVA(QoreObject* self, QoreLoggerAppender* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 type = HARD_QORE_VALUE_INT(args, 0);
    QoreValue params = get_param_value(args, 1);
# 182 "QC_LoggerAppender.qpp"
    a->processEvent(type, params, xsink);
    return QoreValue();
}

// private bool LoggerAppender::pushEvent(int type, auto params){}
static QoreValue LoggerAppender_pushEvent_ViVA(QoreObject* self, QoreLoggerAppender* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 type = HARD_QORE_VALUE_INT(args, 0);
    QoreValue params = get_param_value(args, 1);
# 154 "QC_LoggerAppender.qpp"
    return a->pushEvent(xsink, type, params);
}

// nothing LoggerAppender::removeAllFilters(){}
static QoreValue LoggerAppender_removeAllFilters(QoreObject* self, QoreLoggerAppender* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 133 "QC_LoggerAppender.qpp"
    a->removeAllFilters(xsink);
    return QoreValue();
}

// nothing LoggerAppender::removeFilter(LoggerFilter filter){}
static QoreValue LoggerAppender_removeFilter_C12LoggerFilter(QoreObject* self, QoreLoggerAppender* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(filter, QoreLoggerFilter, args, 0, CID_LOGGERFILTER, "LoggerAppender::removeFilter()", "LoggerFilter", xsink);
    if (*xsink)
        return 0;
# 125 "QC_LoggerAppender.qpp"
    ReferenceHolder<QoreLoggerFilter> holder(filter, xsink);
    a->removeFilter(xsink, obj_filter);
    return QoreValue();
}

// nothing LoggerAppender::setQueue(*LoggerAppenderQueue queue){}
static QoreValue LoggerAppender_setQueue_C20_LoggerAppenderQueue(QoreObject* self, QoreLoggerAppender* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_OR_NOTHING_DATA(queue, QoreLoggerAppenderQueue, args, 0, CID_LOGGERAPPENDERQUEUE, xsink);
    if (*xsink)
        return 0;
# 75 "QC_LoggerAppender.qpp"
    ReferenceHolder<QoreLoggerAppenderQueue> holder(queue, xsink);
    a->setQueue(xsink, obj_queue, holder);
    return QoreValue();
}

DLLLOCAL void preinitLoggerAppenderClass() {
    QC_LOGGERAPPENDER = new QoreBuiltinClass("LoggerAppender", "::Qore::Logger::LoggerAppender", QDOM_DEFAULT);
    CID_LOGGERAPPENDER = QC_LOGGERAPPENDER->getID();
    QC_LOGGERAPPENDER->setSystem();
}

DLLLOCAL QoreClass* initLoggerAppenderClass(QoreNamespace& ns) {
    if (!QC_LOGGERAPPENDER)
        preinitLoggerAppenderClass();

    // private:internal members
    QC_LOGGERAPPENDER->addMember("q", Internal, QC_LOGGERAPPENDERQUEUE->getTypeInfo());

    // nothing LoggerAppender::addFilter(LoggerFilter filter, bool top = False){}
    QC_LOGGERAPPENDER->addMethod("addFilter", (q_method_n_t)LoggerAppender_addFilter_C12LoggerFilterVb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, QC_LOGGERFILTER->getTypeInfo(), QORE_PARAM_NO_ARG, "filter", boolTypeInfo, QoreSimpleValue().assign(false), "top");

    // nothing LoggerAppender::close(){}
    QC_LOGGERAPPENDER->addMethod("close", (q_method_n_t)LoggerAppender_close, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // LoggerAppender::constructor() {}
    QC_LOGGERAPPENDER->addConstructor(LoggerAppender_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // LoggerAppender::constructor(string name) {}
    QC_LOGGERAPPENDER->addConstructor(LoggerAppender_constructor_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "name");

    // private *object LoggerAppender::ensureAtomicOperations(int type){}
    QC_LOGGERAPPENDER->addMethod("ensureAtomicOperations", (q_method_n_t)LoggerAppender_ensureAtomicOperations_Vi, Private, QCF_NO_FLAGS, QDOM_DEFAULT, objectOrNothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "type");

    // list<LoggerFilter> LoggerAppender::getFilters(){}
    QC_LOGGERAPPENDER->addMethod("getFilters", (q_method_n_t)LoggerAppender_getFilters, Public, QCF_NO_FLAGS, QDOM_DEFAULT, qore_get_complex_list_type(QC_LOGGERFILTER->getTypeInfo()));

    // string LoggerAppender::getName(){}
    QC_LOGGERAPPENDER->addMethod("getName", (q_method_n_t)LoggerAppender_getName, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);

    // *LoggerAppenderQueue LoggerAppender::getQueue(){}
    QC_LOGGERAPPENDER->addMethod("getQueue", (q_method_n_t)LoggerAppender_getQueue, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_LOGGERAPPENDERQUEUE->getOrNothingTypeInfo());

    // bool LoggerAppender::isOpen(){}
    QC_LOGGERAPPENDER->addMethod("isOpen", (q_method_n_t)LoggerAppender_isOpen, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // nothing LoggerAppender::open(){}
    QC_LOGGERAPPENDER->addMethod("open", (q_method_n_t)LoggerAppender_open, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // bool LoggerAppender::post(LoggerEvent event){}
    QC_LOGGERAPPENDER->addMethod("post", (q_method_n_t)LoggerAppender_post_C11LoggerEvent, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, QC_LOGGEREVENT->getTypeInfo(), QORE_PARAM_NO_ARG, "event");

    // nothing LoggerAppender::processEvent(int type, auto params){}
    QC_LOGGERAPPENDER->addMethod("processEvent", (q_method_n_t)LoggerAppender_processEvent_ViVA, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, bigIntTypeInfo, QORE_PARAM_NO_ARG, "type", autoTypeInfo, QORE_PARAM_NO_ARG, "params");

    // nothing LoggerAppender::processEventImpl(int type, auto params);
    QC_LOGGERAPPENDER->addAbstractMethod("processEventImpl", Public, QCF_NO_FLAGS, nothingTypeInfo, 2, bigIntTypeInfo, QORE_PARAM_NO_ARG, "type", autoTypeInfo, QORE_PARAM_NO_ARG, "params");

    // private bool LoggerAppender::pushEvent(int type, auto params){}
    QC_LOGGERAPPENDER->addMethod("pushEvent", (q_method_n_t)LoggerAppender_pushEvent_ViVA, Private, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 2, bigIntTypeInfo, QORE_PARAM_NO_ARG, "type", autoTypeInfo, QORE_PARAM_NO_ARG, "params");

    // nothing LoggerAppender::removeAllFilters(){}
    QC_LOGGERAPPENDER->addMethod("removeAllFilters", (q_method_n_t)LoggerAppender_removeAllFilters, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing LoggerAppender::removeFilter(LoggerFilter filter){}
    QC_LOGGERAPPENDER->addMethod("removeFilter", (q_method_n_t)LoggerAppender_removeFilter_C12LoggerFilter, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_LOGGERFILTER->getTypeInfo(), QORE_PARAM_NO_ARG, "filter");

    // private auto LoggerAppender::serializeImpl(LoggerEvent event);
    QC_LOGGERAPPENDER->addAbstractMethod("serializeImpl", Private, QCF_NO_FLAGS, autoTypeInfo, 1, QC_LOGGEREVENT->getTypeInfo(), QORE_PARAM_NO_ARG, "event");

    // nothing LoggerAppender::setQueue(*LoggerAppenderQueue queue){}
    QC_LOGGERAPPENDER->addMethod("setQueue", (q_method_n_t)LoggerAppender_setQueue_C20_LoggerAppenderQueue, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_LOGGERAPPENDERQUEUE->getOrNothingTypeInfo(), QORE_PARAM_NO_ARG, "queue");

    return QC_LOGGERAPPENDER;
}
