/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_LoggerAppenderFile.cpp LoggerAppenderFile class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO LEVEL SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_logger.h"

#include "QC_LoggerAppenderFile.h"
/* Qore class Qore::Logger::LoggerAppenderFile */

qore_classid_t CID_LOGGERAPPENDERFILE;
QoreClass* QC_LOGGERAPPENDERFILE;

// nothing LoggerAppenderFile::closeFile(){}
static QoreValue LoggerAppenderFile_closeFile(QoreObject* self, QoreLoggerAppenderFile* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 109 "QC_LoggerAppenderFile.qpp"
    a->closeFile(xsink);
    return QoreValue();
}

// LoggerAppenderFile::constructor(*string name, LoggerLayout layout, string filename, *string encoding) {}
static void LoggerAppenderFile_constructor_NsC12LoggerLayoutVsNs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = get_param_value(args, 0).get<const QoreStringNode>();
    const QoreObject* layout = get_param_value(args, 1).get<const QoreObject>();
    const QoreStringNode* filename = HARD_QORE_VALUE_STRING(args, 2);
    const QoreStringNode* encoding = get_param_value(args, 3).get<const QoreStringNode>();
# 49 "QC_LoggerAppenderFile.qpp"
const QoreEncoding* qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;
    if (name) {
        self->setPrivate(CID_LOGGERAPPENDERFILE, new QoreLoggerAppenderFile(self, name, layout, filename, qe,
            xsink));
    } else {
        self->setPrivate(CID_LOGGERAPPENDERFILE, new QoreLoggerAppenderFile(self, layout, filename, qe, xsink));
    }
    if (!*xsink && encoding) {
        self->setValue("encoding", encoding->stringRefSelf(), xsink);
    }
}

// LoggerAppenderFile::constructor(LoggerLayout layout, string filename, *string encoding) {}
static void LoggerAppenderFile_constructor_C12LoggerLayoutVsNs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreObject* layout = get_param_value(args, 0).get<const QoreObject>();
    const QoreStringNode* filename = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* encoding = get_param_value(args, 2).get<const QoreStringNode>();
# 68 "QC_LoggerAppenderFile.qpp"
const QoreEncoding* qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;
    self->setPrivate(CID_LOGGERAPPENDERFILE, new QoreLoggerAppenderFile(self, layout, filename, qe, xsink));
    if (!*xsink && encoding) {
        self->setValue("encoding", encoding->stringRefSelf(), xsink);
    }
}

// File LoggerAppenderFile::getFile(){}
static QoreValue LoggerAppenderFile_getFile(QoreObject* self, QoreLoggerAppenderFile* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 79 "QC_LoggerAppenderFile.qpp"
    return a->getFile(xsink);
}

// string LoggerAppenderFile::getFileName(){}
static QoreValue LoggerAppenderFile_getFileName(QoreObject* self, QoreLoggerAppenderFile* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 86 "QC_LoggerAppenderFile.qpp"
    return a->getFileName(xsink);
}

// nothing LoggerAppenderFile::openFile(){}
static QoreValue LoggerAppenderFile_openFile(QoreObject* self, QoreLoggerAppenderFile* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 102 "QC_LoggerAppenderFile.qpp"
    a->openFile(xsink);
    return QoreValue();
}

// nothing LoggerAppenderFile::processEventImpl(int type, auto params){}
static QoreValue LoggerAppenderFile_processEventImpl_ViVA(QoreObject* self, QoreLoggerAppenderFile* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 type = HARD_QORE_VALUE_INT(args, 0);
    QoreValue params = get_param_value(args, 1);
# 95 "QC_LoggerAppenderFile.qpp"
    a->processEventImpl(xsink, type, params);
    return QoreValue();
}

// nothing LoggerAppenderFile::reopen(){}
static QoreValue LoggerAppenderFile_reopen(QoreObject* self, QoreLoggerAppenderFile* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 124 "QC_LoggerAppenderFile.qpp"
    a->reopen(xsink);
    return QoreValue();
}

// static nothing LoggerAppenderFile::closeFileStatic(File file){}
static QoreValue static_LoggerAppenderFile_closeFileStatic_C4File(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(file, File, args, 0, CID_FILE, "LoggerAppenderFile::closeFileStatic()", "File", xsink);
    if (*xsink)
        return 0;
# 116 "QC_LoggerAppenderFile.qpp"
ReferenceHolder<File> f(file, xsink);
    QoreLoggerAppenderFile::closeFileStatic(file, xsink);
    return QoreValue();
}

DLLLOCAL void preinitLoggerAppenderFileClass() {
    QC_LOGGERAPPENDERFILE = new QoreBuiltinClass("LoggerAppenderFile", "::Qore::Logger::LoggerAppenderFile", QDOM_DEFAULT);
    CID_LOGGERAPPENDERFILE = QC_LOGGERAPPENDERFILE->getID();
    QC_LOGGERAPPENDERFILE->setSystem();
}

DLLLOCAL QoreClass* initLoggerAppenderFileClass(QoreNamespace& ns) {
    if (!QC_LOGGERAPPENDERFILE)
        preinitLoggerAppenderFileClass();

    // set parent class
    assert(QC_LOGGERAPPENDERWITHLAYOUT);
    QC_LOGGERAPPENDERFILE->addBuiltinVirtualBaseClass(QC_LOGGERAPPENDERWITHLAYOUT);

    // private members
    QC_LOGGERAPPENDERFILE->addMember("file", Private, QC_FILE->getTypeInfo());
    QC_LOGGERAPPENDERFILE->addMember("fileName", Private, stringTypeInfo);
    QC_LOGGERAPPENDERFILE->addMember("encoding", Private, stringOrNothingTypeInfo);

    // nothing LoggerAppenderFile::closeFile(){}
    QC_LOGGERAPPENDERFILE->addMethod("closeFile", (q_method_n_t)LoggerAppenderFile_closeFile, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // LoggerAppenderFile::constructor(*string name, LoggerLayout layout, string filename, *string encoding) {}
    QC_LOGGERAPPENDERFILE->addConstructor(LoggerAppenderFile_constructor_NsC12LoggerLayoutVsNs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 4, stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "name", QC_LOGGERLAYOUT->getTypeInfo(), QORE_PARAM_NO_ARG, "layout", stringTypeInfo, QORE_PARAM_NO_ARG, "filename", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");

    // LoggerAppenderFile::constructor(LoggerLayout layout, string filename, *string encoding) {}
    QC_LOGGERAPPENDERFILE->addConstructor(LoggerAppenderFile_constructor_C12LoggerLayoutVsNs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 3, QC_LOGGERLAYOUT->getTypeInfo(), QORE_PARAM_NO_ARG, "layout", stringTypeInfo, QORE_PARAM_NO_ARG, "filename", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");

    // File LoggerAppenderFile::getFile(){}
    QC_LOGGERAPPENDERFILE->addMethod("getFile", (q_method_n_t)LoggerAppenderFile_getFile, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_FILE->getTypeInfo());

    // string LoggerAppenderFile::getFileName(){}
    QC_LOGGERAPPENDERFILE->addMethod("getFileName", (q_method_n_t)LoggerAppenderFile_getFileName, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);

    // nothing LoggerAppenderFile::openFile(){}
    QC_LOGGERAPPENDERFILE->addMethod("openFile", (q_method_n_t)LoggerAppenderFile_openFile, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing LoggerAppenderFile::processEventImpl(int type, auto params){}
    QC_LOGGERAPPENDERFILE->addMethod("processEventImpl", (q_method_n_t)LoggerAppenderFile_processEventImpl_ViVA, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, bigIntTypeInfo, QORE_PARAM_NO_ARG, "type", autoTypeInfo, QORE_PARAM_NO_ARG, "params");

    // nothing LoggerAppenderFile::reopen(){}
    QC_LOGGERAPPENDERFILE->addMethod("reopen", (q_method_n_t)LoggerAppenderFile_reopen, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // static nothing LoggerAppenderFile::closeFileStatic(File file){}
    QC_LOGGERAPPENDERFILE->addStaticMethod("closeFileStatic", (q_func_n_t)static_LoggerAppenderFile_closeFileStatic_C4File, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_FILE->getTypeInfo(), QORE_PARAM_NO_ARG, "file");

    return QC_LOGGERAPPENDERFILE;
}
