/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_LoggerAppenderQueue.cpp LoggerAppenderQueue class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_logger.h"

#include "QC_LoggerAppenderQueue.h"
#include "QC_LoggerAppender.h"
/* Qore class Qore::Logger::LoggerAppenderQueue */

qore_classid_t CID_LOGGERAPPENDERQUEUE;
QoreClass* QC_LOGGERAPPENDERQUEUE;

// LoggerAppenderQueue::constructor() {}
static void LoggerAppenderQueue_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 45 "QC_LoggerAppenderQueue.qpp"
Queue* queue = new Queue(-1);
    ReferenceHolder<QoreObject> q(new QoreObject(QC_QUEUE, getProgram(), queue), xsink);
    self->setPrivate(CID_LOGGERAPPENDERQUEUE, new QoreLoggerAppenderQueue(self, *q, *queue));
    self->setValue("q", q.release(), xsink);
}

// private *hash<auto> LoggerAppenderQueue::getEvent(timeout ms){}
static QoreValue LoggerAppenderQueue_getEvent_Vt(QoreObject* self, QoreLoggerAppenderQueue* q, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 ms = HARD_QORE_VALUE_INT(args, 0);
# 100 "QC_LoggerAppenderQueue.qpp"
    return q->getEvent(ms, xsink);
}

// nothing LoggerAppenderQueue::process(timeout ms = 0){}
static QoreValue LoggerAppenderQueue_process_Vt(QoreObject* self, QoreLoggerAppenderQueue* q, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 ms = HARD_QORE_VALUE_INT(args, 0);
# 77 "QC_LoggerAppenderQueue.qpp"
    q->process(ms, xsink);
    return QoreValue();
}

// bool LoggerAppenderQueue::push(LoggerAppender appender, int type, auto params){}
static QoreValue LoggerAppenderQueue_push_C14LoggerAppenderViVA(QoreObject* self, QoreLoggerAppenderQueue* q, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(appender, QoreLoggerAppender, args, 0, CID_LOGGERAPPENDER, "LoggerAppenderQueue::push()", "LoggerAppender", xsink);
    if (*xsink)
        return 0;
    int64 type = HARD_QORE_VALUE_INT(args, 1);
    QoreValue params = get_param_value(args, 2);
# 59 "QC_LoggerAppenderQueue.qpp"
    ReferenceHolder<AbstractPrivateData> holder(appender, xsink);
    q->push(xsink, obj_appender, type, params);
    return true;
}

// int LoggerAppenderQueue::size(){}
static QoreValue LoggerAppenderQueue_size(QoreObject* self, QoreLoggerAppenderQueue* q, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 84 "QC_LoggerAppenderQueue.qpp"
    return q->size();
}

DLLLOCAL void preinitLoggerAppenderQueueClass() {
    QC_LOGGERAPPENDERQUEUE = new QoreBuiltinClass("LoggerAppenderQueue", "::Qore::Logger::LoggerAppenderQueue", QDOM_DEFAULT);
    CID_LOGGERAPPENDERQUEUE = QC_LOGGERAPPENDERQUEUE->getID();
    QC_LOGGERAPPENDERQUEUE->setSystem();
}

DLLLOCAL QoreClass* initLoggerAppenderQueueClass(QoreNamespace& ns) {
    if (!QC_LOGGERAPPENDERQUEUE)
        preinitLoggerAppenderQueueClass();

    // private:internal members
    QC_LOGGERAPPENDERQUEUE->addMember("q", Internal, QC_QUEUE->getTypeInfo());

    // LoggerAppenderQueue::constructor() {}
    QC_LOGGERAPPENDERQUEUE->addConstructor(LoggerAppenderQueue_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // private *hash<auto> LoggerAppenderQueue::getEvent(timeout ms){}
    QC_LOGGERAPPENDERQUEUE->addMethod("getEvent", (q_method_n_t)LoggerAppenderQueue_getEvent_Vt, Private, QCF_NO_FLAGS, QDOM_DEFAULT, autoHashOrNothingTypeInfo, 1, timeoutTypeInfo, QORE_PARAM_NO_ARG, "ms");

    // nothing LoggerAppenderQueue::process(timeout ms = 0){}
    QC_LOGGERAPPENDERQUEUE->addMethod("process", (q_method_n_t)LoggerAppenderQueue_process_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)0), "ms");

    // bool LoggerAppenderQueue::push(LoggerAppender appender, int type, auto params){}
    QC_LOGGERAPPENDERQUEUE->addMethod("push", (q_method_n_t)LoggerAppenderQueue_push_C14LoggerAppenderViVA, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 3, QC_LOGGERAPPENDER->getTypeInfo(), QORE_PARAM_NO_ARG, "appender", bigIntTypeInfo, QORE_PARAM_NO_ARG, "type", autoTypeInfo, QORE_PARAM_NO_ARG, "params");

    // int LoggerAppenderQueue::size(){}
    QC_LOGGERAPPENDERQUEUE->addMethod("size", (q_method_n_t)LoggerAppenderQueue_size, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    return QC_LOGGERAPPENDERQUEUE;
}
