/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_LoggerAppenderWithLayout.cpp LoggerAppenderWithLayout class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_logger.h"

#include "QC_LoggerAppenderWithLayout.h"
#include "QC_LoggerLayout.h"
#include "QC_LoggerEvent.h"
/* Qore class Qore::Logger::LoggerAppenderWithLayout */

qore_classid_t CID_LOGGERAPPENDERWITHLAYOUT;
QoreClass* QC_LOGGERAPPENDERWITHLAYOUT;

// LoggerAppenderWithLayout::constructor(LoggerLayout layout) {}
static void LoggerAppenderWithLayout_constructor_C12LoggerLayout(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreObject* layout = get_param_value(args, 0).get<const QoreObject>();
# 46 "QC_LoggerAppenderWithLayout.qpp"
self->setPrivate(CID_LOGGERAPPENDERWITHLAYOUT, new QoreLoggerAppenderWithLayout(self, layout, xsink));
}

// LoggerAppenderWithLayout::constructor(*string name, LoggerLayout layout) {}
static void LoggerAppenderWithLayout_constructor_NsC12LoggerLayout(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = get_param_value(args, 0).get<const QoreStringNode>();
    const QoreObject* layout = get_param_value(args, 1).get<const QoreObject>();
# 53 "QC_LoggerAppenderWithLayout.qpp"
if (name) {
        self->setPrivate(CID_LOGGERAPPENDERWITHLAYOUT, new QoreLoggerAppenderWithLayout(self, name, layout, xsink));
    } else {
        self->setPrivate(CID_LOGGERAPPENDERWITHLAYOUT, new QoreLoggerAppenderWithLayout(self, layout, xsink));
    }
}

// LoggerLayout LoggerAppenderWithLayout::getLayout(){}
static QoreValue LoggerAppenderWithLayout_getLayout(QoreObject* self, QoreLoggerAppenderWithLayout* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 71 "QC_LoggerAppenderWithLayout.qpp"
    return a->getLayout();
}

// private auto LoggerAppenderWithLayout::serializeImpl(LoggerEvent event){}
static QoreValue LoggerAppenderWithLayout_serializeImpl_C11LoggerEvent(QoreObject* self, QoreLoggerAppenderWithLayout* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(event, QoreLoggerEvent, args, 0, CID_LOGGEREVENT, "LoggerAppenderWithLayout::serializeImpl()", "LoggerEvent", xsink);
    if (*xsink)
        return 0;
# 82 "QC_LoggerAppenderWithLayout.qpp"
    ReferenceHolder<QoreLoggerEvent> holder(event, xsink);
    return a->serializeImpl(xsink, obj_event, event);
}

// nothing LoggerAppenderWithLayout::setLayout(LoggerLayout layout){}
static QoreValue LoggerAppenderWithLayout_setLayout_C12LoggerLayout(QoreObject* self, QoreLoggerAppenderWithLayout* a, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreObject* layout = get_param_value(args, 0).get<const QoreObject>();
# 64 "QC_LoggerAppenderWithLayout.qpp"
    a->setLayout(xsink, layout);
    return QoreValue();
}

DLLLOCAL void preinitLoggerAppenderWithLayoutClass() {
    QC_LOGGERAPPENDERWITHLAYOUT = new QoreBuiltinClass("LoggerAppenderWithLayout", "::Qore::Logger::LoggerAppenderWithLayout", QDOM_DEFAULT);
    CID_LOGGERAPPENDERWITHLAYOUT = QC_LOGGERAPPENDERWITHLAYOUT->getID();
    QC_LOGGERAPPENDERWITHLAYOUT->setSystem();
}

DLLLOCAL QoreClass* initLoggerAppenderWithLayoutClass(QoreNamespace& ns) {
    if (!QC_LOGGERAPPENDERWITHLAYOUT)
        preinitLoggerAppenderWithLayoutClass();

    // set parent class
    assert(QC_LOGGERAPPENDER);
    QC_LOGGERAPPENDERWITHLAYOUT->addBuiltinVirtualBaseClass(QC_LOGGERAPPENDER);

    // LoggerAppenderWithLayout::constructor(LoggerLayout layout) {}
    QC_LOGGERAPPENDERWITHLAYOUT->addConstructor(LoggerAppenderWithLayout_constructor_C12LoggerLayout, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, QC_LOGGERLAYOUT->getTypeInfo(), QORE_PARAM_NO_ARG, "layout");

    // LoggerAppenderWithLayout::constructor(*string name, LoggerLayout layout) {}
    QC_LOGGERAPPENDERWITHLAYOUT->addConstructor(LoggerAppenderWithLayout_constructor_NsC12LoggerLayout, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 2, stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "name", QC_LOGGERLAYOUT->getTypeInfo(), QORE_PARAM_NO_ARG, "layout");

    // LoggerLayout LoggerAppenderWithLayout::getLayout(){}
    QC_LOGGERAPPENDERWITHLAYOUT->addMethod("getLayout", (q_method_n_t)LoggerAppenderWithLayout_getLayout, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_LOGGERLAYOUT->getTypeInfo());

    // private auto LoggerAppenderWithLayout::serializeImpl(LoggerEvent event){}
    QC_LOGGERAPPENDERWITHLAYOUT->addMethod("serializeImpl", (q_method_n_t)LoggerAppenderWithLayout_serializeImpl_C11LoggerEvent, Private, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 1, QC_LOGGEREVENT->getTypeInfo(), QORE_PARAM_NO_ARG, "event");

    // nothing LoggerAppenderWithLayout::setLayout(LoggerLayout layout){}
    QC_LOGGERAPPENDERWITHLAYOUT->addMethod("setLayout", (q_method_n_t)LoggerAppenderWithLayout_setLayout_C12LoggerLayout, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_LOGGERLAYOUT->getTypeInfo(), QORE_PARAM_NO_ARG, "layout");

    return QC_LOGGERAPPENDERWITHLAYOUT;
}
