/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_LoggerEvent.cpp LoggerEvent class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_logger.h"

#include "QC_LoggerEvent.h"
#include "QC_LoggerLevel.h"
/* Qore class Qore::Logger::LoggerEvent */

qore_classid_t CID_LOGGEREVENT;
QoreClass* QC_LOGGEREVENT;

// LoggerEvent::constructor(object logger, LoggerLevel level, string message, *list<auto> msg_args, *hash<CallStackInfo> location_info, *int thread_id, *date time_stamp, *hash<ExceptionInfo> throwable) {}
static void LoggerEvent_constructor_VoC11LoggerLevelVsC11_list_auto_C20_hash_CallStackInfo_NiNdC20_hash_ExceptionInfo_(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* logger = HARD_QORE_VALUE_OBJECT(args, 0);
    HARD_QORE_VALUE_OBJ_DATA(level, QoreLoggerLevel, args, 1, CID_LOGGERLEVEL, "LoggerEvent::constructor()", "LoggerLevel", xsink);
    if (*xsink)
        return;
    const QoreStringNode* message = HARD_QORE_VALUE_STRING(args, 2);
    const QoreListNode* msg_args = get_param_value(args, 3).get<const QoreListNode>();
    const QoreHashNode* location_info = get_param_value(args, 4).get<const QoreHashNode>();
    int64 thread_id = HARD_QORE_VALUE_INT(args, 5);
    const DateTimeNode* time_stamp = get_param_value(args, 6).get<const DateTimeNode>();
    const QoreHashNode* throwable = get_param_value(args, 7).get<const QoreHashNode>();
# 57 "QC_LoggerEvent.qpp"
    //ReferenceHolder<AbstractPrivateData> hlogger(logger, xsink);
    ReferenceHolder<AbstractPrivateData> hlevel(level, xsink);

    if (!thread_id) {
        thread_id = q_gettid();
    }

    SimpleRefHolder<DateTimeNode> ts;
    if (!time_stamp) {
        ts = DateTimeNode::makeNow();
    }

    ValueHolder v(logger->evalMethod("getName", nullptr, xsink), xsink);
    if (*xsink) {
        return;
    }
    assert(v);
    assert(v->getType() == NT_STRING);

    self->setPrivate(CID_LOGGEREVENT, new QoreLoggerEvent(logger, v->get<const QoreStringNode>(), obj_level, level,
        message, msg_args, 0, location_info, thread_id, time_stamp ? time_stamp : *ts, throwable));

    self->setValue("logger", logger->objectRefSelf(), nullptr);
}

// LoggerEvent::constructor(string fqcn, softstring category, LoggerLevel level, string message, *list<auto> msg_args, *hash<CallStackInfo> location_info, *int thread_id, *date time_stamp, *hash<ExceptionInfo> throwable) {}
static void LoggerEvent_constructor_VsvsC11LoggerLevelVsC11_list_auto_C20_hash_CallStackInfo_NiNdC20_hash_ExceptionInfo_(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* fqcn = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* category = HARD_QORE_VALUE_STRING(args, 1);
    HARD_QORE_VALUE_OBJ_DATA(level, QoreLoggerLevel, args, 2, CID_LOGGERLEVEL, "LoggerEvent::constructor()", "LoggerLevel", xsink);
    if (*xsink)
        return;
    const QoreStringNode* message = HARD_QORE_VALUE_STRING(args, 3);
    const QoreListNode* msg_args = get_param_value(args, 4).get<const QoreListNode>();
    const QoreHashNode* location_info = get_param_value(args, 5).get<const QoreHashNode>();
    int64 thread_id = HARD_QORE_VALUE_INT(args, 6);
    const DateTimeNode* time_stamp = get_param_value(args, 7).get<const DateTimeNode>();
    const QoreHashNode* throwable = get_param_value(args, 8).get<const QoreHashNode>();
# 97 "QC_LoggerEvent.qpp"
    ReferenceHolder<AbstractPrivateData> hlevel(level, xsink);

    if (!thread_id) {
        thread_id = q_gettid();
    }

    SimpleRefHolder<DateTimeNode> ts;
    if (!time_stamp) {
        ts = DateTimeNode::makeNow();
    }

    self->setPrivate(CID_LOGGEREVENT, new QoreLoggerEvent(fqcn->stringRefSelf(), category, obj_level, level, message,
        msg_args, 0, location_info, thread_id, time_stamp ? time_stamp : *ts, throwable));
}

// string LoggerEvent::getCategoryName(){}
static QoreValue LoggerEvent_getCategoryName(QoreObject* self, QoreLoggerEvent* event, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 151 "QC_LoggerEvent.qpp"
    return event->getCategoryName();
}

// string LoggerEvent::getFullQualifiedClassname(){}
static QoreValue LoggerEvent_getFullQualifiedClassname(QoreObject* self, QoreLoggerEvent* event, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 116 "QC_LoggerEvent.qpp"
    return event->getFullQualifiedClassname();
}

// LoggerLevel LoggerEvent::getLevel(){}
static QoreValue LoggerEvent_getLevel(QoreObject* self, QoreLoggerEvent* event, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 130 "QC_LoggerEvent.qpp"
    return event->getLevel();
}

// *hash<CallStackInfo> LoggerEvent::getLocationInfo(){}
static QoreValue LoggerEvent_getLocationInfo(QoreObject* self, QoreLoggerEvent* event, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 123 "QC_LoggerEvent.qpp"
    return event->getLocationInfo();
}

// *object LoggerEvent::getLogger(){}
static QoreValue LoggerEvent_getLogger(QoreObject* self, QoreLoggerEvent* event, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 137 "QC_LoggerEvent.qpp"
    return event->getLogger();
}

// string LoggerEvent::getMessage(){}
static QoreValue LoggerEvent_getMessage(QoreObject* self, QoreLoggerEvent* event, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 195 "QC_LoggerEvent.qpp"
    return event->getMessage(xsink);
}

// date LoggerEvent::getRelativeTime(){}
static QoreValue LoggerEvent_getRelativeTime(QoreObject* self, QoreLoggerEvent* event, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 169 "QC_LoggerEvent.qpp"
    return event->getRelativeTime();
}

// int LoggerEvent::getThreadId(){}
static QoreValue LoggerEvent_getThreadId(QoreObject* self, QoreLoggerEvent* event, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 144 "QC_LoggerEvent.qpp"
    return event->getThreadId();
}

// *hash<ExceptionInfo> LoggerEvent::getThrowableInfo(){}
static QoreValue LoggerEvent_getThrowableInfo(QoreObject* self, QoreLoggerEvent* event, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 176 "QC_LoggerEvent.qpp"
    return event->getThrowableInfo();
}

// date LoggerEvent::getTimeStamp(){}
static QoreValue LoggerEvent_getTimeStamp(QoreObject* self, QoreLoggerEvent* event, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 158 "QC_LoggerEvent.qpp"
    return event->getTimeStamp();
}

// int LoggerEvent::getUniqueId(){}
static QoreValue LoggerEvent_getUniqueId(QoreObject* self, QoreLoggerEvent* event, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 183 "QC_LoggerEvent.qpp"
    return event->getUniqueId();
}

// static date LoggerEvent::getStartTime(){}
static QoreValue static_LoggerEvent_getStartTime(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 202 "QC_LoggerEvent.qpp"
return QoreLoggerEvent::getStartTime();
}

// static nothing LoggerEvent::setStartTime(date time){}
static QoreValue static_LoggerEvent_setStartTime_Vd(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* time = HARD_QORE_VALUE_DATE(args, 0);
# 209 "QC_LoggerEvent.qpp"
QoreLoggerEvent::setStartTime(time);
    return QoreValue();
}

DLLLOCAL void preinitLoggerEventClass() {
    QC_LOGGEREVENT = new QoreBuiltinClass("LoggerEvent", "::Qore::Logger::LoggerEvent", QDOM_DEFAULT);
    CID_LOGGEREVENT = QC_LOGGEREVENT->getID();
    QC_LOGGEREVENT->setSystem();
}

DLLLOCAL QoreClass* initLoggerEventClass(QoreNamespace& ns) {
    if (!QC_LOGGEREVENT)
        preinitLoggerEventClass();

    // private:internal members
    QC_LOGGEREVENT->addMember("logger", Internal, QC_LOGGER->getOrNothingTypeInfo());

    // LoggerEvent::constructor(object logger, LoggerLevel level, string message, *list<auto> msg_args, *hash<CallStackInfo> location_info, *int thread_id, *date time_stamp, *hash<ExceptionInfo> throwable) {}
    QC_LOGGEREVENT->addConstructor(LoggerEvent_constructor_VoC11LoggerLevelVsC11_list_auto_C20_hash_CallStackInfo_NiNdC20_hash_ExceptionInfo_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 8, objectTypeInfo, QORE_PARAM_NO_ARG, "logger", QC_LOGGERLEVEL->getTypeInfo(), QORE_PARAM_NO_ARG, "level", stringTypeInfo, QORE_PARAM_NO_ARG, "message", autoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "msg_args", hashdeclCallStackInfo->getTypeInfo(true), QORE_PARAM_NO_ARG, "location_info", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "thread_id", dateOrNothingTypeInfo, QORE_PARAM_NO_ARG, "time_stamp", hashdeclExceptionInfo->getTypeInfo(true), QORE_PARAM_NO_ARG, "throwable");

    // LoggerEvent::constructor(string fqcn, softstring category, LoggerLevel level, string message, *list<auto> msg_args, *hash<CallStackInfo> location_info, *int thread_id, *date time_stamp, *hash<ExceptionInfo> throwable) {}
    QC_LOGGEREVENT->addConstructor(LoggerEvent_constructor_VsvsC11LoggerLevelVsC11_list_auto_C20_hash_CallStackInfo_NiNdC20_hash_ExceptionInfo_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 9, stringTypeInfo, QORE_PARAM_NO_ARG, "fqcn", softStringTypeInfo, QORE_PARAM_NO_ARG, "category", QC_LOGGERLEVEL->getTypeInfo(), QORE_PARAM_NO_ARG, "level", stringTypeInfo, QORE_PARAM_NO_ARG, "message", autoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "msg_args", hashdeclCallStackInfo->getTypeInfo(true), QORE_PARAM_NO_ARG, "location_info", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "thread_id", dateOrNothingTypeInfo, QORE_PARAM_NO_ARG, "time_stamp", hashdeclExceptionInfo->getTypeInfo(true), QORE_PARAM_NO_ARG, "throwable");

    // string LoggerEvent::getCategoryName(){}
    QC_LOGGEREVENT->addMethod("getCategoryName", (q_method_n_t)LoggerEvent_getCategoryName, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);

    // string LoggerEvent::getFullQualifiedClassname(){}
    QC_LOGGEREVENT->addMethod("getFullQualifiedClassname", (q_method_n_t)LoggerEvent_getFullQualifiedClassname, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);

    // LoggerLevel LoggerEvent::getLevel(){}
    QC_LOGGEREVENT->addMethod("getLevel", (q_method_n_t)LoggerEvent_getLevel, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_LOGGERLEVEL->getTypeInfo());

    // *hash<CallStackInfo> LoggerEvent::getLocationInfo(){}
    QC_LOGGEREVENT->addMethod("getLocationInfo", (q_method_n_t)LoggerEvent_getLocationInfo, Public, QCF_NO_FLAGS, QDOM_DEFAULT, hashdeclCallStackInfo->getTypeInfo(true));

    // *object LoggerEvent::getLogger(){}
    QC_LOGGEREVENT->addMethod("getLogger", (q_method_n_t)LoggerEvent_getLogger, Public, QCF_NO_FLAGS, QDOM_DEFAULT, objectOrNothingTypeInfo);

    // string LoggerEvent::getMessage(){}
    QC_LOGGEREVENT->addMethod("getMessage", (q_method_n_t)LoggerEvent_getMessage, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);

    // date LoggerEvent::getRelativeTime(){}
    QC_LOGGEREVENT->addMethod("getRelativeTime", (q_method_n_t)LoggerEvent_getRelativeTime, Public, QCF_NO_FLAGS, QDOM_DEFAULT, dateTypeInfo);

    // int LoggerEvent::getThreadId(){}
    QC_LOGGEREVENT->addMethod("getThreadId", (q_method_n_t)LoggerEvent_getThreadId, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // *hash<ExceptionInfo> LoggerEvent::getThrowableInfo(){}
    QC_LOGGEREVENT->addMethod("getThrowableInfo", (q_method_n_t)LoggerEvent_getThrowableInfo, Public, QCF_NO_FLAGS, QDOM_DEFAULT, hashdeclExceptionInfo->getTypeInfo(true));

    // date LoggerEvent::getTimeStamp(){}
    QC_LOGGEREVENT->addMethod("getTimeStamp", (q_method_n_t)LoggerEvent_getTimeStamp, Public, QCF_NO_FLAGS, QDOM_DEFAULT, dateTypeInfo);

    // int LoggerEvent::getUniqueId(){}
    QC_LOGGEREVENT->addMethod("getUniqueId", (q_method_n_t)LoggerEvent_getUniqueId, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // static date LoggerEvent::getStartTime(){}
    QC_LOGGEREVENT->addStaticMethod("getStartTime", (q_func_n_t)static_LoggerEvent_getStartTime, Public, QCF_NO_FLAGS, QDOM_DEFAULT, dateTypeInfo);

    // static nothing LoggerEvent::setStartTime(date time){}
    QC_LOGGEREVENT->addStaticMethod("setStartTime", (q_func_n_t)static_LoggerEvent_setStartTime_Vd, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, dateTypeInfo, QORE_PARAM_NO_ARG, "time");

    return QC_LOGGEREVENT;
}
