/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_LoggerFilter.cpp LoggerFilter class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_logger.h"

#include "QC_LoggerFilter.h"
#include "QC_LoggerEvent.h"

class QoreLoggerFilter : public AbstractPrivateData {
public:
    DLLLOCAL QoreLoggerFilter() {
    }

    DLLLOCAL virtual ~QoreLoggerFilter() = default;
};
/* Qore class Qore::Logger::LoggerFilter */

qore_classid_t CID_LOGGERFILTER;
QoreClass* QC_LOGGERFILTER;

// LoggerFilter::constructor() {}
static void LoggerFilter_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 53 "QC_LoggerFilter.qpp"
self->setPrivate(CID_LOGGERFILTER, new QoreLoggerFilter);
}

// int LoggerFilter::eval(LoggerEvent event){}
static QoreValue LoggerFilter_eval_C11LoggerEvent(QoreObject* self, QoreLoggerFilter* ll, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(event, QoreLoggerEvent, args, 0, CID_LOGGEREVENT, "LoggerFilter::eval()", "LoggerEvent", xsink);
    if (*xsink)
        return 0;
# 65 "QC_LoggerFilter.qpp"
    ReferenceHolder<AbstractPrivateData> holder(event, xsink);
    return NEUTRAL;
}

DLLLOCAL void preinitLoggerFilterClass() {
    QC_LOGGERFILTER = new QoreBuiltinClass("LoggerFilter", "::Qore::Logger::LoggerFilter", QDOM_DEFAULT);
    CID_LOGGERFILTER = QC_LOGGERFILTER->getID();
    QC_LOGGERFILTER->setSystem();
}

DLLLOCAL QoreClass* initLoggerFilterClass(QoreNamespace& ns) {
    if (!QC_LOGGERFILTER)
        preinitLoggerFilterClass();

    // LoggerFilter::constructor() {}
    QC_LOGGERFILTER->addConstructor(LoggerFilter_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // int LoggerFilter::eval(LoggerEvent event){}
    QC_LOGGERFILTER->addMethod("eval", (q_method_n_t)LoggerFilter_eval_C11LoggerEvent, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, QC_LOGGEREVENT->getTypeInfo(), QORE_PARAM_NO_ARG, "event");

    return QC_LOGGERFILTER;
}
