/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_LoggerInterface.cpp LoggerInterface class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_logger.h"

#include "QC_LoggerInterface.h"

static QoreHashNode* LoggerInterface_serializer(const QoreObject& self, const QoreLoggerInterface& level,
        QoreSerializationContext& context, ExceptionSink* xsink) {
    ReferenceHolder<QoreHashNode> rv(new QoreHashNode(autoTypeInfo), xsink);
    return rv.release();
}

static void LoggerInterface_deserializer(QoreObject& self, const QoreHashNode* sdata,
        QoreDeserializationContext& context, ExceptionSink* xsink) {
}
/* Qore class Qore::Logger::LoggerInterface */

qore_classid_t CID_LOGGERINTERFACE;
QoreClass* QC_LOGGERINTERFACE;

// LoggerInterface::constructor() {}
static void LoggerInterface_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 55 "QC_LoggerInterface.qpp"
self->setPrivate(CID_LOGGERINTERFACE, new QoreLoggerInterface);
}

// bool LoggerInterface::isLogger(LoggerInterface logger){}
static QoreValue LoggerInterface_isLogger_C15LoggerInterface(QoreObject* self, QoreLoggerInterface* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(logger, QoreLoggerInterface, args, 0, CID_LOGGERINTERFACE, "LoggerInterface::isLogger()", "LoggerInterface", xsink);
    if (*xsink)
        return 0;
# 62 "QC_LoggerInterface.qpp"
    ReferenceHolder<QoreLoggerInterface> holder(logger, xsink);
    return obj_logger == self;
}

DLLLOCAL void preinitLoggerInterfaceClass() {
    QC_LOGGERINTERFACE = new QoreBuiltinClass("LoggerInterface", "::Qore::Logger::LoggerInterface", QDOM_DEFAULT);
    CID_LOGGERINTERFACE = QC_LOGGERINTERFACE->getID();
    QC_LOGGERINTERFACE->setSystem();
}

DLLLOCAL QoreClass* initLoggerInterfaceClass(QoreNamespace& ns) {
    if (!QC_LOGGERINTERFACE)
        preinitLoggerInterfaceClass();

    // set serializer function
    QC_LOGGERINTERFACE->setSerializer(reinterpret_cast<q_serializer_t>(LoggerInterface_serializer));

    // set deserializer function
    QC_LOGGERINTERFACE->setDeserializer(LoggerInterface_deserializer);

    // set parent class
    assert(QC_SERIALIZABLE);
    QC_LOGGERINTERFACE->addBuiltinVirtualBaseClass(QC_SERIALIZABLE);

    // nothing LoggerInterface::assertLog(bool assertion, string message, ...);
    QC_LOGGERINTERFACE->addAbstractMethod("assertLog", Public, QCF_USES_EXTRA_ARGS, nothingTypeInfo, 2, boolTypeInfo, QORE_PARAM_NO_ARG, "assertion", stringTypeInfo, QORE_PARAM_NO_ARG, "message");

    // LoggerInterface::constructor() {}
    QC_LOGGERINTERFACE->addConstructor(LoggerInterface_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // nothing LoggerInterface::debug(string message, ...);
    QC_LOGGERINTERFACE->addAbstractMethod("debug", Public, QCF_USES_EXTRA_ARGS, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "message");

    // nothing LoggerInterface::debugVar(string var_name, auto value);
    QC_LOGGERINTERFACE->addAbstractMethod("debugVar", Public, QCF_NO_FLAGS, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "var_name", autoTypeInfo, QORE_PARAM_NO_ARG, "value");

    // *LoggerLevel LoggerInterface::decLevel();
    QC_LOGGERINTERFACE->addAbstractMethod("decLevel", Public, QCF_NO_FLAGS, QC_LOGGERLEVEL->getOrNothingTypeInfo());

    // nothing LoggerInterface::error(string message, ...);
    QC_LOGGERINTERFACE->addAbstractMethod("error", Public, QCF_USES_EXTRA_ARGS, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "message");

    // nothing LoggerInterface::fatal(string message, ...);
    QC_LOGGERINTERFACE->addAbstractMethod("fatal", Public, QCF_USES_EXTRA_ARGS, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "message");

    // *LoggerLevel LoggerInterface::getLevel(bool effective = True);
    QC_LOGGERINTERFACE->addAbstractMethod("getLevel", Public, QCF_NO_FLAGS, QC_LOGGERLEVEL->getOrNothingTypeInfo(), 1, boolTypeInfo, QoreSimpleValue().assign(true), "effective");

    // *LoggerLevel LoggerInterface::incLevel();
    QC_LOGGERINTERFACE->addAbstractMethod("incLevel", Public, QCF_NO_FLAGS, QC_LOGGERLEVEL->getOrNothingTypeInfo());

    // nothing LoggerInterface::info(string message, ...);
    QC_LOGGERINTERFACE->addAbstractMethod("info", Public, QCF_USES_EXTRA_ARGS, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "message");

    // bool LoggerInterface::isDebugEnabled();
    QC_LOGGERINTERFACE->addAbstractMethod("isDebugEnabled", Public, QCF_NO_FLAGS, boolTypeInfo);

    // bool LoggerInterface::isEnabledFor(Qore::Logger::LoggerLevel level);
    QC_LOGGERINTERFACE->addAbstractMethod("isEnabledFor", Public, QCF_NO_FLAGS, boolTypeInfo, 1, QC_LOGGERLEVEL->getTypeInfo(), QORE_PARAM_NO_ARG, "level");

    // bool LoggerInterface::isEnabledFor(int level);
    QC_LOGGERINTERFACE->addAbstractMethod("isEnabledFor", Public, QCF_NO_FLAGS, boolTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "level");

    // bool LoggerInterface::isEnabledFor(string level);
    QC_LOGGERINTERFACE->addAbstractMethod("isEnabledFor", Public, QCF_NO_FLAGS, boolTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "level");

    // bool LoggerInterface::isErrorEnabled();
    QC_LOGGERINTERFACE->addAbstractMethod("isErrorEnabled", Public, QCF_NO_FLAGS, boolTypeInfo);

    // bool LoggerInterface::isFatalEnabled();
    QC_LOGGERINTERFACE->addAbstractMethod("isFatalEnabled", Public, QCF_NO_FLAGS, boolTypeInfo);

    // bool LoggerInterface::isInfoEnabled();
    QC_LOGGERINTERFACE->addAbstractMethod("isInfoEnabled", Public, QCF_NO_FLAGS, boolTypeInfo);

    // bool LoggerInterface::isLogger(LoggerInterface logger){}
    QC_LOGGERINTERFACE->addMethod("isLogger", (q_method_n_t)LoggerInterface_isLogger_C15LoggerInterface, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, QC_LOGGERINTERFACE->getTypeInfo(), QORE_PARAM_NO_ARG, "logger");

    // bool LoggerInterface::isTraceEnabled();
    QC_LOGGERINTERFACE->addAbstractMethod("isTraceEnabled", Public, QCF_NO_FLAGS, boolTypeInfo);

    // bool LoggerInterface::isWarnEnabled();
    QC_LOGGERINTERFACE->addAbstractMethod("isWarnEnabled", Public, QCF_NO_FLAGS, boolTypeInfo);

    // nothing LoggerInterface::log(Qore::Logger::LoggerLevel level, string message, ...);
    QC_LOGGERINTERFACE->addAbstractMethod("log", Public, QCF_USES_EXTRA_ARGS, nothingTypeInfo, 2, QC_LOGGERLEVEL->getTypeInfo(), QORE_PARAM_NO_ARG, "level", stringTypeInfo, QORE_PARAM_NO_ARG, "message");

    // nothing LoggerInterface::log(int level, string message, ...);
    QC_LOGGERINTERFACE->addAbstractMethod("log", Public, QCF_USES_EXTRA_ARGS, nothingTypeInfo, 2, bigIntTypeInfo, QORE_PARAM_NO_ARG, "level", stringTypeInfo, QORE_PARAM_NO_ARG, "message");

    // nothing LoggerInterface::log(string level, string message, ...);
    QC_LOGGERINTERFACE->addAbstractMethod("log", Public, QCF_USES_EXTRA_ARGS, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "level", stringTypeInfo, QORE_PARAM_NO_ARG, "message");

    // nothing LoggerInterface::logArgs(Qore::Logger::LoggerLevel level, string message, *softlist<auto> args);
    QC_LOGGERINTERFACE->addAbstractMethod("logArgs", Public, QCF_NO_FLAGS, nothingTypeInfo, 3, QC_LOGGERLEVEL->getTypeInfo(), QORE_PARAM_NO_ARG, "level", stringTypeInfo, QORE_PARAM_NO_ARG, "message", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "args");

    // nothing LoggerInterface::logArgs(int level, string message, *softlist<auto> args);
    QC_LOGGERINTERFACE->addAbstractMethod("logArgs", Public, QCF_NO_FLAGS, nothingTypeInfo, 3, bigIntTypeInfo, QORE_PARAM_NO_ARG, "level", stringTypeInfo, QORE_PARAM_NO_ARG, "message", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "args");

    // nothing LoggerInterface::logArgs(string level, string message, *softlist<auto> args);
    QC_LOGGERINTERFACE->addAbstractMethod("logArgs", Public, QCF_NO_FLAGS, nothingTypeInfo, 3, stringTypeInfo, QORE_PARAM_NO_ARG, "level", stringTypeInfo, QORE_PARAM_NO_ARG, "message", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "args");

    // nothing LoggerInterface::logEvent(LoggerEvent event);
    QC_LOGGERINTERFACE->addAbstractMethod("logEvent", Public, QCF_NO_FLAGS, nothingTypeInfo, 1, QC_LOGGEREVENT->getTypeInfo(), QORE_PARAM_NO_ARG, "event");

    // nothing LoggerInterface::setLevel(*Qore::Logger::LoggerLevel level);
    QC_LOGGERINTERFACE->addAbstractMethod("setLevel", Public, QCF_NO_FLAGS, nothingTypeInfo, 1, QC_LOGGERLEVEL->getOrNothingTypeInfo(), QORE_PARAM_NO_ARG, "level");

    // nothing LoggerInterface::setLevel(string level);
    QC_LOGGERINTERFACE->addAbstractMethod("setLevel", Public, QCF_NO_FLAGS, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "level");

    // nothing LoggerInterface::setLevel(int level);
    QC_LOGGERINTERFACE->addAbstractMethod("setLevel", Public, QCF_NO_FLAGS, nothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "level");

    // nothing LoggerInterface::trace(string message, ...);
    QC_LOGGERINTERFACE->addAbstractMethod("trace", Public, QCF_USES_EXTRA_ARGS, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "message");

    // nothing LoggerInterface::traceVar(string var_name, auto value);
    QC_LOGGERINTERFACE->addAbstractMethod("traceVar", Public, QCF_NO_FLAGS, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "var_name", autoTypeInfo, QORE_PARAM_NO_ARG, "value");

    // nothing LoggerInterface::warn(string message, ...);
    QC_LOGGERINTERFACE->addAbstractMethod("warn", Public, QCF_USES_EXTRA_ARGS, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "message");

    return QC_LOGGERINTERFACE;
}
