/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_LoggerPattern.cpp LoggerPattern class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_logger.h"
#include "QC_LoggerPattern.h"
#include "QC_LoggerLayoutPattern.h"
/* Qore class Qore::Logger::LoggerPattern */

qore_classid_t CID_LOGGERPATTERN;
QoreClass* QC_LOGGERPATTERN;

// LoggerPattern::constructor(string pattern) {}
static void LoggerPattern_constructor_Vs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* pattern = HARD_QORE_VALUE_STRING(args, 0);
# 73 "QC_LoggerPattern.qpp"
ReferenceHolder<QoreLoggerPattern> pat(new QoreLoggerPattern(self), xsink);
    if (*xsink) {
        return;
    }
    if (pat->setPattern(pattern, xsink)) {
        assert(*xsink);
        return;
    }
    self->setPrivate(CID_LOGGERPATTERN, pat.release());
}

// string LoggerPattern::format(auto data){}
static QoreValue LoggerPattern_format_VA(QoreObject* self, QoreLoggerPattern* pat, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue data = get_param_value(args, 0);
# 115 "QC_LoggerPattern.qpp"
    // see if 'data' is a LoggerLayoutPattern object for optimized calls
    ReferenceHolder<QoreLoggerLayoutPattern> llp(xsink);
    if (data.getType() == NT_OBJECT) {
        QoreObject* obj = data.get<QoreObject>();
        // only set the private data object if the class is not overridden
        if (QC_LOGGERLAYOUTPATTERN->isEqual(*obj->getClass())) {
            llp = obj->tryGetReferencedPrivateData<QoreLoggerLayoutPattern>(CID_LOGGERLAYOUTPATTERN, xsink);
            if (*xsink) {
                return QoreValue();
            }
        }
    }
    return pat->format(data, *llp, xsink);
}

// list<auto> LoggerPattern::getParsedPattern(){}
static QoreValue LoggerPattern_getParsedPattern(QoreObject* self, QoreLoggerPattern* pat, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 105 "QC_LoggerPattern.qpp"
    return pat->getParsedPattern();
}

// string LoggerPattern::getPattern(){}
static QoreValue LoggerPattern_getPattern(QoreObject* self, QoreLoggerPattern* pat, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 98 "QC_LoggerPattern.qpp"
    return pat->getPattern();;
}

// nothing LoggerPattern::setPattern(string pattern){}
static QoreValue LoggerPattern_setPattern_Vs(QoreObject* self, QoreLoggerPattern* pat, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* pattern = HARD_QORE_VALUE_STRING(args, 0);
# 91 "QC_LoggerPattern.qpp"
    pat->setPattern(pattern, xsink);
    return QoreValue();
}

DLLLOCAL void preinitLoggerPatternClass() {
    QC_LOGGERPATTERN = new QoreBuiltinClass("LoggerPattern", "::Qore::Logger::LoggerPattern", QDOM_DEFAULT);
    CID_LOGGERPATTERN = QC_LOGGERPATTERN->getID();
    QC_LOGGERPATTERN->setSystem();
}

DLLLOCAL QoreClass* initLoggerPatternClass(QoreNamespace& ns) {
    if (!QC_LOGGERPATTERN)
        preinitLoggerPatternClass();

    // LoggerPattern::constructor(string pattern) {}
    QC_LOGGERPATTERN->addConstructor(LoggerPattern_constructor_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "pattern");

    // string LoggerPattern::format(auto data){}
    QC_LOGGERPATTERN->addMethod("format", (q_method_n_t)LoggerPattern_format_VA, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "data");

    // list<auto> LoggerPattern::getParsedPattern(){}
    QC_LOGGERPATTERN->addMethod("getParsedPattern", (q_method_n_t)LoggerPattern_getParsedPattern, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoListTypeInfo);

    // string LoggerPattern::getPattern(){}
    QC_LOGGERPATTERN->addMethod("getPattern", (q_method_n_t)LoggerPattern_getPattern, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);

    // private *string LoggerPattern::resolveField(auto data, string key, *string option);
    QC_LOGGERPATTERN->addAbstractMethod("resolveField", Private, QCF_NO_FLAGS, stringOrNothingTypeInfo, 3, autoTypeInfo, QORE_PARAM_NO_ARG, "data", stringTypeInfo, QORE_PARAM_NO_ARG, "key", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "option");

    // nothing LoggerPattern::setPattern(string pattern){}
    QC_LOGGERPATTERN->addMethod("setPattern", (q_method_n_t)LoggerPattern_setPattern_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "pattern");

    return QC_LOGGERPATTERN;
}
