/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_AbstractMember.cpp AbstractMember class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_reflection.h"
#include "QC_Class.h"
#include "QC_AbstractMember.h"
#include "QC_Type.h"
/* Qore class Qore::Reflection::AbstractMember */

qore_classid_t CID_ABSTRACTMEMBER;
QoreClass* QC_ABSTRACTMEMBER;

// AbstractMember::constructor() {}
static void AbstractMember_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 45 "QC_AbstractMember.qpp"
xsink->raiseException("ABSTRACTMEMBER-CONSTRUCTOR-ERROR", "this class cannot be constructed directly or inherited directly by a user-defined class");
}

// auto AbstractMember::getDefaultValue(){}
static QoreValue AbstractMember_getDefaultValue(QoreObject* self, QoreReflectionMember* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 117 "QC_AbstractMember.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return m->mem->getDefaultValue(xsink);
}

// string AbstractMember::getName(){}
static QoreValue AbstractMember_getName(QoreObject* self, QoreReflectionMember* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 59 "QC_AbstractMember.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return new QoreStringNode(m->name);
}

// hash<SourceLocationInfo> AbstractMember::getSourceLocation(){}
static QoreValue AbstractMember_getSourceLocation(QoreObject* self, QoreReflectionMember* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 138 "QC_AbstractMember.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the constant ptr must be valid
    return m->mem->getSourceLocation()->getHash();
}

// Type AbstractMember::getType(){}
static QoreValue AbstractMember_getType(QoreObject* self, QoreReflectionMember* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 96 "QC_AbstractMember.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return new QoreObject(QC_TYPE, m->pgm, new QoreType(m->mem->getTypeInfo()));
}

// bool AbstractMember::isEqual(AbstractMember mem){}
static QoreValue AbstractMember_isEqual_C14AbstractMember(QoreObject* self, QoreReflectionMember* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(mem, QoreReflectionMember, args, 0, CID_ABSTRACTMEMBER, "AbstractMember::isEqual()", "AbstractMember", xsink);
    if (*xsink)
        return 0;
# 80 "QC_AbstractMember.qpp"
    ReferenceHolder<QoreReflectionMember> holder(mem, xsink);

    return mem->mem == m->mem;
}

DLLLOCAL void preinitAbstractMemberClass() {
    QC_ABSTRACTMEMBER = new QoreBuiltinClass("AbstractMember", "::Qore::Reflection::AbstractMember", QDOM_REFLECTION);
    CID_ABSTRACTMEMBER = QC_ABSTRACTMEMBER->getID();
    QC_ABSTRACTMEMBER->setSystem();
}

DLLLOCAL QoreClass* initAbstractMemberClass(QoreNamespace& ns) {
    if (!QC_ABSTRACTMEMBER)
        preinitAbstractMemberClass();

    // AbstractMember::constructor() {}
    QC_ABSTRACTMEMBER->addConstructor(AbstractMember_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // auto AbstractMember::getDefaultValue(){}
    QC_ABSTRACTMEMBER->addMethod("getDefaultValue", (q_method_n_t)AbstractMember_getDefaultValue, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo);

    // string AbstractMember::getName(){}
    QC_ABSTRACTMEMBER->addMethod("getName", (q_method_n_t)AbstractMember_getName, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // hash<SourceLocationInfo> AbstractMember::getSourceLocation(){}
    QC_ABSTRACTMEMBER->addMethod("getSourceLocation", (q_method_n_t)AbstractMember_getSourceLocation, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, hashdeclSourceLocationInfo->getTypeInfo(false));

    // Type AbstractMember::getType(){}
    QC_ABSTRACTMEMBER->addMethod("getType", (q_method_n_t)AbstractMember_getType, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_TYPE->getTypeInfo());

    // bool AbstractMember::isEqual(AbstractMember mem){}
    QC_ABSTRACTMEMBER->addMethod("isEqual", (q_method_n_t)AbstractMember_isEqual_C14AbstractMember, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, QC_ABSTRACTMEMBER->getTypeInfo(), QORE_PARAM_NO_ARG, "mem");

    return QC_ABSTRACTMEMBER;
}
