/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_AbstractMethodVariant.cpp AbstractMethodVariant class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_reflection.h"
#include "QC_AbstractMethodVariant.h"
#include "QC_AbstractClass.h"
#include "QC_AbstractClass.h"
#include "QC_AbstractMethod.h"
/* Qore class Qore::Reflection::AbstractMethodVariant */

qore_classid_t CID_ABSTRACTMETHODVARIANT;
QoreClass* QC_ABSTRACTMETHODVARIANT;

// AbstractMethodVariant::constructor() {}
static void AbstractMethodVariant_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 51 "QC_AbstractMethodVariant.qpp"
xsink->raiseException("ABSTRACTMETHODVARIANT-CONSTRUCTOR-ERROR", "this class is an abstract class and cannot be constructed directly or inherited directly by a user-defined class");
}

// string AbstractMethodVariant::getAccessModifierString(){}
static QoreValue AbstractMethodVariant_getAccessModifierString(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 141 "QC_AbstractMethodVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    const QoreExternalMethodVariant* mv = reinterpret_cast<const QoreExternalMethodVariant*>(v->variant);
    return new QoreStringNode(mv->getAccessString());
}

// AbstractClass AbstractMethodVariant::getClass(){}
static QoreValue AbstractMethodVariant_getClass(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 123 "QC_AbstractMethodVariant.qpp"
    const QoreExternalMethodVariant* mv = reinterpret_cast<const QoreExternalMethodVariant*>(v->variant);
    return get_class_object(v->pgm, mv->getClass());
}

// AbstractMethod AbstractMethodVariant::getMethod(){}
static QoreValue AbstractMethodVariant_getMethod(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 108 "QC_AbstractMethodVariant.qpp"
    const QoreExternalMethodVariant* mv = reinterpret_cast<const QoreExternalMethodVariant*>(v->variant);
    ReferenceHolder<QoreReflectionMethod> m(new QoreReflectionMethod(v->pgm,
        mv->getMethod()), xsink);
    return get_method_object(m, xsink);
}

// bool AbstractMethodVariant::isAbstract(){}
static QoreValue AbstractMethodVariant_isAbstract(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 65 "QC_AbstractMethodVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    const QoreExternalMethodVariant* mv = reinterpret_cast<const QoreExternalMethodVariant*>(v->variant);
    return mv->isAbstract();
}

// bool AbstractMethodVariant::isFinal(){}
static QoreValue AbstractMethodVariant_isFinal(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 87 "QC_AbstractMethodVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    const QoreExternalMethodVariant* mv = reinterpret_cast<const QoreExternalMethodVariant*>(v->variant);
    return mv->isFinal();
}

DLLLOCAL void preinitAbstractMethodVariantClass() {
    QC_ABSTRACTMETHODVARIANT = new QoreBuiltinClass("AbstractMethodVariant", "::Qore::Reflection::AbstractMethodVariant", QDOM_REFLECTION);
    CID_ABSTRACTMETHODVARIANT = QC_ABSTRACTMETHODVARIANT->getID();
    QC_ABSTRACTMETHODVARIANT->setSystem();
}

DLLLOCAL QoreClass* initAbstractMethodVariantClass(QoreNamespace& ns) {
    if (!QC_ABSTRACTMETHODVARIANT)
        preinitAbstractMethodVariantClass();

    // set parent class
    assert(QC_ABSTRACTVARIANT);
    QC_ABSTRACTMETHODVARIANT->addBuiltinVirtualBaseClass(QC_ABSTRACTVARIANT);

    // AbstractMethodVariant::constructor() {}
    QC_ABSTRACTMETHODVARIANT->addConstructor(AbstractMethodVariant_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // string AbstractMethodVariant::getAccessModifierString(){}
    QC_ABSTRACTMETHODVARIANT->addMethod("getAccessModifierString", (q_method_n_t)AbstractMethodVariant_getAccessModifierString, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // AbstractClass AbstractMethodVariant::getClass(){}
    QC_ABSTRACTMETHODVARIANT->addMethod("getClass", (q_method_n_t)AbstractMethodVariant_getClass, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_ABSTRACTCLASS->getTypeInfo());

    // AbstractMethod AbstractMethodVariant::getMethod(){}
    QC_ABSTRACTMETHODVARIANT->addMethod("getMethod", (q_method_n_t)AbstractMethodVariant_getMethod, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_ABSTRACTMETHOD->getTypeInfo());

    // bool AbstractMethodVariant::isAbstract(){}
    QC_ABSTRACTMETHODVARIANT->addMethod("isAbstract", (q_method_n_t)AbstractMethodVariant_isAbstract, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractMethodVariant::isFinal(){}
    QC_ABSTRACTMETHODVARIANT->addMethod("isFinal", (q_method_n_t)AbstractMethodVariant_isFinal, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    return QC_ABSTRACTMETHODVARIANT;
}
