/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_ConstructorMethodVariant.cpp ConstructorMethodVariant class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_reflection.h"
#include "QC_ConstructorMethodVariant.h"
#include "QC_Class.h"
/* Qore class Qore::Reflection::ConstructorMethodVariant */

qore_classid_t CID_CONSTRUCTORMETHODVARIANT;
QoreClass* QC_CONSTRUCTORMETHODVARIANT;

// object ConstructorMethodVariant::call(...){}
static QoreValue ConstructorMethodVariant_call_VV(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 69 "QC_ConstructorMethodVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    const QoreExternalMethodVariant* mv = reinterpret_cast<const QoreExternalMethodVariant*>(v->variant);
    const QoreClass* cls = mv->getClass();
    if (check_constructor(cls, xsink)) {
        return QoreValue();
    }

    return cls->execConstructorVariant(mv, args, xsink);
}

// object ConstructorMethodVariant::callArgs(*softlist argv){}
static QoreValue ConstructorMethodVariant_callArgs_nl(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* argv = get_param_value(args, 0).get<const QoreListNode>();
# 101 "QC_ConstructorMethodVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    const QoreExternalMethodVariant* mv = reinterpret_cast<const QoreExternalMethodVariant*>(v->variant);
    const QoreClass* cls = mv->getClass();
    if (check_constructor(cls, xsink)) {
        return QoreValue();
    }

    return cls->execConstructorVariant(mv, argv, xsink);
}

// ConstructorMethodVariant::constructor() {}
static void ConstructorMethodVariant_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 49 "QC_ConstructorMethodVariant.qpp"
xsink->raiseException("CONSTRUCTORMETHODVARIANT-CONSTRUCTOR-ERROR", "this class is an abstract class and cannot be constructed directly or inherited directly by a user-defined class");
}

DLLLOCAL void preinitConstructorMethodVariantClass() {
    QC_CONSTRUCTORMETHODVARIANT = new QoreBuiltinClass("ConstructorMethodVariant", "::Qore::Reflection::ConstructorMethodVariant", QDOM_REFLECTION);
    CID_CONSTRUCTORMETHODVARIANT = QC_CONSTRUCTORMETHODVARIANT->getID();
    QC_CONSTRUCTORMETHODVARIANT->setSystem();
}

DLLLOCAL QoreClass* initConstructorMethodVariantClass(QoreNamespace& ns) {
    if (!QC_CONSTRUCTORMETHODVARIANT)
        preinitConstructorMethodVariantClass();

    // set parent class
    assert(QC_ABSTRACTMETHODVARIANT);
    QC_CONSTRUCTORMETHODVARIANT->addBuiltinVirtualBaseClass(QC_ABSTRACTMETHODVARIANT);

    // object ConstructorMethodVariant::call(...){}
    QC_CONSTRUCTORMETHODVARIANT->addMethod("call", (q_method_n_t)ConstructorMethodVariant_call_VV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, objectTypeInfo);

    // object ConstructorMethodVariant::callArgs(*softlist argv){}
    QC_CONSTRUCTORMETHODVARIANT->addMethod("callArgs", (q_method_n_t)ConstructorMethodVariant_callArgs_nl, Public, QCF_NO_FLAGS, QDOM_DEFAULT, objectTypeInfo, 1, softListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "argv");

    // ConstructorMethodVariant::constructor() {}
    QC_CONSTRUCTORMETHODVARIANT->addConstructor(ConstructorMethodVariant_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    return QC_CONSTRUCTORMETHODVARIANT;
}
