/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_CopyMethodVariant.cpp CopyMethodVariant class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_reflection.h"
#include "QC_CopyMethodVariant.h"
#include "QC_AbstractMethod.h"
#include "QC_NormalMethodVariant.h"
/* Qore class Qore::Reflection::CopyMethodVariant */

qore_classid_t CID_COPYMETHODVARIANT;
QoreClass* QC_COPYMETHODVARIANT;

// auto CopyMethodVariant::call(object obj){}
static QoreValue CopyMethodVariant_call_Vo(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
# 67 "QC_CopyMethodVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // we have to explictly test object compatibility here, otherwise QoreObject::evalMethod() will throw
    // a confusing exception
    const QoreExternalMethodVariant* mv = reinterpret_cast<const QoreExternalMethodVariant*>(v->variant);
    const QoreMethod* m = mv->getMethod();
    if (check_variant_call(v->variant, obj, *m, xsink)) {
        return QoreValue();
    }

    const QoreClass* cls = m->getClass();
    return cls->execCopy(obj, xsink);
}

// CopyMethodVariant::constructor() {}
static void CopyMethodVariant_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 50 "QC_CopyMethodVariant.qpp"
xsink->raiseException("COPYMETHODVARIANT-CONSTRUCTOR-ERROR", "this class is an abstract class and cannot be constructed directly or inherited directly by a user-defined class");
}

DLLLOCAL void preinitCopyMethodVariantClass() {
    QC_COPYMETHODVARIANT = new QoreBuiltinClass("CopyMethodVariant", "::Qore::Reflection::CopyMethodVariant", QDOM_REFLECTION);
    CID_COPYMETHODVARIANT = QC_COPYMETHODVARIANT->getID();
    QC_COPYMETHODVARIANT->setSystem();
}

DLLLOCAL QoreClass* initCopyMethodVariantClass(QoreNamespace& ns) {
    if (!QC_COPYMETHODVARIANT)
        preinitCopyMethodVariantClass();

    // set parent class
    assert(QC_ABSTRACTMETHODVARIANT);
    QC_COPYMETHODVARIANT->addBuiltinVirtualBaseClass(QC_ABSTRACTMETHODVARIANT);

    // auto CopyMethodVariant::call(object obj){}
    QC_COPYMETHODVARIANT->addMethod("call", (q_method_n_t)CopyMethodVariant_call_Vo, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 1, objectTypeInfo, QORE_PARAM_NO_ARG, "obj");

    // CopyMethodVariant::constructor() {}
    QC_COPYMETHODVARIANT->addConstructor(CopyMethodVariant_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    return QC_COPYMETHODVARIANT;
}
