/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_PseudoClass.cpp PseudoClass class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore_reflection.h>
#include <QC_PseudoClass.h>
#include <QC_Type.h>

QoreObject* get_pseudo_class_object(qore_type_t t) {
    QoreProgram* pgm = getProgram();
    const QoreClass* cls = qore_pseudo_get_class(t);
    assert(cls);
    return new QoreObject(QC_PSEUDOCLASS, pgm, new QoreReflectionClass(pgm, cls));
}
/* Qore class Qore::Reflection::PseudoClass */

qore_classid_t CID_PSEUDOCLASS;
QoreClass* QC_PSEUDOCLASS;

// PseudoClass::constructor(auto val) {}
static void PseudoClass_constructor_VA(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue val = get_param_value(args, 0);
# 153 "QC_PseudoClass.qpp"
self->setPrivate(CID_PSEUDOCLASS, new QoreReflectionClass(getProgram(), qore_pseudo_get_class(val.getType())));
}

// PseudoClass::constructor(Type type) {}
static void PseudoClass_constructor_C4Type(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(type, QoreType, args, 0, CID_TYPE, "PseudoClass::constructor()", "Type", xsink);
    if (*xsink)
        return;
# 167 "QC_PseudoClass.qpp"
ReferenceHolder<QoreType> holder(type, xsink);
    self->setPrivate(CID_PSEUDOCLASS, new QoreReflectionClass(getProgram(), qore_pseudo_get_class(type->typeInfo)));
}

// static PseudoClass PseudoClass::forType(Type type){}
static QoreValue static_PseudoClass_forType_C4Type(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(type, QoreType, args, 0, CID_TYPE, "PseudoClass::forType()", "Type", xsink);
    if (*xsink)
        return 0;
# 199 "QC_PseudoClass.qpp"
ReferenceHolder<QoreType> holder(type, xsink);
    QoreProgram* pgm = getProgram();
    return new QoreObject(QC_PSEUDOCLASS, pgm, new QoreReflectionClass(pgm, qore_pseudo_get_class(type->typeInfo)));
}

// static PseudoClass PseudoClass::forValue(auto val){}
static QoreValue static_PseudoClass_forValue_VA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue val = get_param_value(args, 0);
# 183 "QC_PseudoClass.qpp"
QoreProgram* pgm = getProgram();
    return new QoreObject(QC_PSEUDOCLASS, pgm, new QoreReflectionClass(pgm, qore_pseudo_get_class(val.getType())));
}

DLLLOCAL void preinitPseudoClassClass() {
    QC_PSEUDOCLASS = new QoreBuiltinClass("PseudoClass", "::Qore::Reflection::PseudoClass", QDOM_REFLECTION);
    CID_PSEUDOCLASS = QC_PSEUDOCLASS->getID();
    QC_PSEUDOCLASS->setSystem();
}

DLLLOCAL QoreClass* initPseudoClassClass(QoreNamespace& ns) {
    if (!QC_PSEUDOCLASS)
        preinitPseudoClassClass();

    // set parent class
    assert(QC_ABSTRACTCLASS);
    QC_PSEUDOCLASS->addBuiltinVirtualBaseClass(QC_ABSTRACTCLASS);

    // PseudoClass::constructor(auto val) {}
    QC_PSEUDOCLASS->addConstructor(PseudoClass_constructor_VA, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "val");

    // PseudoClass::constructor(Type type) {}
    QC_PSEUDOCLASS->addConstructor(PseudoClass_constructor_C4Type, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, QC_TYPE->getTypeInfo(), QORE_PARAM_NO_ARG, "type");

    // static PseudoClass PseudoClass::forType(Type type){}
    QC_PSEUDOCLASS->addStaticMethod("forType", (q_func_n_t)static_PseudoClass_forType_C4Type, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_PSEUDOCLASS->getTypeInfo(), 1, QC_TYPE->getTypeInfo(), QORE_PARAM_NO_ARG, "type");

    // static PseudoClass PseudoClass::forValue(auto val){}
    QC_PSEUDOCLASS->addStaticMethod("forValue", (q_func_n_t)static_PseudoClass_forValue_VA, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_PSEUDOCLASS->getTypeInfo(), 1, autoTypeInfo, QORE_PARAM_NO_ARG, "val");
# 43 "QC_PseudoClass.qpp"
# 53 "QC_PseudoClass.qpp"
# 58 "QC_PseudoClass.qpp"
# 63 "QC_PseudoClass.qpp"
# 68 "QC_PseudoClass.qpp"
# 73 "QC_PseudoClass.qpp"
# 78 "QC_PseudoClass.qpp"
# 83 "QC_PseudoClass.qpp"
# 88 "QC_PseudoClass.qpp"
# 93 "QC_PseudoClass.qpp"
# 98 "QC_PseudoClass.qpp"
# 103 "QC_PseudoClass.qpp"
# 108 "QC_PseudoClass.qpp"
    ns.addConstant("BinaryPseudoClass", (get_pseudo_class_object(NT_BINARY)));
    ns.addConstant("BoolPseudoClass", (get_pseudo_class_object(NT_BOOLEAN)));
    ns.addConstant("CallablePseudoClass", (get_pseudo_class_object(NT_FUNCREF)));
    ns.addConstant("ClosurePseudoClass", (get_pseudo_class_object(NT_RUNTIME_CLOSURE)));
    ns.addConstant("DatePseudoClass", (get_pseudo_class_object(NT_DATE)));
    ns.addConstant("FloatPseudoClass", (get_pseudo_class_object(NT_FLOAT)));
    ns.addConstant("HashPseudoClass", (get_pseudo_class_object(NT_HASH)));
    ns.addConstant("IntPseudoClass", (get_pseudo_class_object(NT_INT)));
    ns.addConstant("ListPseudoClass", (get_pseudo_class_object(NT_LIST)));
    ns.addConstant("NothingPseudoClass", (get_pseudo_class_object(NT_NOTHING)));
    ns.addConstant("NumberPseudoClass", (get_pseudo_class_object(NT_NUMBER)));
    ns.addConstant("ObjectPseudoClass", (get_pseudo_class_object(NT_OBJECT)));
    ns.addConstant("StringPseudoClass", (get_pseudo_class_object(NT_STRING)));
    ns.addConstant("ValuePseudoClass", (get_pseudo_class_object(-1)));

    return QC_PSEUDOCLASS;
}
