/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_StaticMember.cpp StaticMember class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore_reflection.h>
#include <QC_Class.h>
#include <QC_StaticMember.h>
/* Qore class Qore::Reflection::StaticMember */

qore_classid_t CID_STATICMEMBER;
QoreClass* QC_STATICMEMBER;

// StaticMember::constructor() {}
static void StaticMember_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 44 "QC_StaticMember.qpp"
xsink->raiseException("STATICMEMBER-CONSTRUCTOR-ERROR", "this class cannot be constructed directly or inherited directly by a user-defined class");
}

// auto StaticMember::getValue(){}
static QoreValue StaticMember_getValue(QoreObject* self, QoreReflectionClassMember* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 58 "QC_StaticMember.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    return reinterpret_cast<const QoreExternalStaticMember*>(m->mem)->getValue();
}

// nothing StaticMember::setValue(auto val){}
static QoreValue StaticMember_setValue_VA(QoreObject* self, QoreReflectionClassMember* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue val = get_param_value(args, 0);
# 79 "QC_StaticMember.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    reinterpret_cast<const QoreExternalStaticMember*>(m->mem)->setValue(val, xsink);
    return QoreValue();
}

DLLLOCAL void preinitStaticMemberClass() {
    QC_STATICMEMBER = new QoreBuiltinClass("StaticMember", "::Qore::Reflection::StaticMember", QDOM_REFLECTION);
    CID_STATICMEMBER = QC_STATICMEMBER->getID();
    QC_STATICMEMBER->setSystem();
}

DLLLOCAL QoreClass* initStaticMemberClass(QoreNamespace& ns) {
    if (!QC_STATICMEMBER)
        preinitStaticMemberClass();

    // set parent class
    assert(QC_ABSTRACTCLASSMEMBER);
    QC_STATICMEMBER->addBuiltinVirtualBaseClass(QC_ABSTRACTCLASSMEMBER);

    // StaticMember::constructor() {}
    QC_STATICMEMBER->addConstructor(StaticMember_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // auto StaticMember::getValue(){}
    QC_STATICMEMBER->addMethod("getValue", (q_method_n_t)StaticMember_getValue, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo);

    // nothing StaticMember::setValue(auto val){}
    QC_STATICMEMBER->addMethod("setValue", (q_method_n_t)StaticMember_setValue_VA, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "val");

    return QC_STATICMEMBER;
}
