/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_StaticMethodVariant.cpp StaticMethodVariant class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2024 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_reflection.h"
#include "QC_StaticMethodVariant.h"
/* Qore class Qore::Reflection::StaticMethodVariant */

qore_classid_t CID_STATICMETHODVARIANT;
QoreClass* QC_STATICMETHODVARIANT;

// auto StaticMethodVariant::call(...){}
static QoreValue StaticMethodVariant_call_VV(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 67 "QC_StaticMethodVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    const QoreExternalMethodVariant* mv = reinterpret_cast<const QoreExternalMethodVariant*>(v->variant);
    const QoreMethod* m = mv->getMethod();
    // the method ptr must be valid
    return QoreObject::evalStaticMethodVariant(*m, mv, args, xsink);
}

// auto StaticMethodVariant::callArgs(*softlist<auto> argv){}
static QoreValue StaticMethodVariant_callArgs_C15_softlist_auto_(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* argv = get_param_value(args, 0).get<const QoreListNode>();
# 95 "QC_StaticMethodVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    const QoreExternalMethodVariant* mv = reinterpret_cast<const QoreExternalMethodVariant*>(v->variant);
    const QoreMethod* m = mv->getMethod();
    // the method ptr must be valid
    return QoreObject::evalStaticMethodVariant(*m, mv, argv, xsink);
}

// StaticMethodVariant::constructor() {}
static void StaticMethodVariant_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 48 "QC_StaticMethodVariant.qpp"
xsink->raiseException("STATICMETHODVARIANT-CONSTRUCTOR-ERROR", "this class is an abstract class and cannot be constructed directly or inherited directly by a user-defined class");
}

DLLLOCAL void preinitStaticMethodVariantClass() {
    QC_STATICMETHODVARIANT = new QoreBuiltinClass("StaticMethodVariant", "::Qore::Reflection::StaticMethodVariant", QDOM_REFLECTION);
    CID_STATICMETHODVARIANT = QC_STATICMETHODVARIANT->getID();
    QC_STATICMETHODVARIANT->setSystem();
}

DLLLOCAL QoreClass* initStaticMethodVariantClass(QoreNamespace& ns) {
    if (!QC_STATICMETHODVARIANT)
        preinitStaticMethodVariantClass();

    // set parent class
    assert(QC_ABSTRACTMETHODVARIANT);
    QC_STATICMETHODVARIANT->addBuiltinVirtualBaseClass(QC_ABSTRACTMETHODVARIANT);

    // auto StaticMethodVariant::call(...){}
    QC_STATICMETHODVARIANT->addMethod("call", (q_method_n_t)StaticMethodVariant_call_VV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo);

    // auto StaticMethodVariant::callArgs(*softlist<auto> argv){}
    QC_STATICMETHODVARIANT->addMethod("callArgs", (q_method_n_t)StaticMethodVariant_callArgs_C15_softlist_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 1, softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "argv");

    // StaticMethodVariant::constructor() {}
    QC_STATICMETHODVARIANT->addConstructor(StaticMethodVariant_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    return QC_STATICMETHODVARIANT;
}
